@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/examples/winmanager/main.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/examples/winmanager/main.cpp   2.3.7   edited 2002-03-02 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -31,6 +31,5 @@ int main(int argc, char **argv)
 
     app.setMainWidget(&hello);
 
-    app.exec();
+    return app.exec();
 }
-
