@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/canvas/qcanvas.cpp   2.3.2   edited 2001-08-20 $
+** $Id: qt/src/canvas/qcanvas.cpp   2.3.7   edited 2003-07-01 $
 **
 ** Implementation of QCanvas and associated classes
 **
@@ -50,6 +50,11 @@
 
 class QCanvasData {
 public:
+    QCanvasData() :
+	itemDict(1013), animDict(503)
+    {
+    }
+
     QList<QCanvasView> viewList;
     QPtrDict<void> itemDict;
     QPtrDict<void> animDict;
@@ -2263,17 +2268,17 @@ QCanvasPixmap::~QCanvasPixmap()
   Constructs a null array.  You should call readPixmaps() before using it
   further.
 */
-QCanvasPixmapArray::QCanvasPixmapArray()
+QCanvasPixmapArray::QCanvasPixmapArray() :
+    framecount(0), img(0)
 {
-    img = 0;
 }
 
 /*!
 Constructs a QCanvasPixmapArray from files. See readPixmaps() for details.
 */
-QCanvasPixmapArray::QCanvasPixmapArray(const QString& datafilenamepattern, int fc)
+QCanvasPixmapArray::QCanvasPixmapArray(const QString& datafilenamepattern, int fc) :
+    framecount(0), img(0)
 {
-    img = 0;
     readPixmaps(datafilenamepattern,fc);
 }
 
@@ -2345,12 +2350,12 @@ bool QCanvasPixmapArray::readCollisionMasks(const QString& fname)
 bool QCanvasPixmapArray::readPixmaps(const QString& datafilenamepattern, int fc, bool maskonly)
 {
     if ( !maskonly ) {
-	delete [] img;
+	reset();
 	framecount = fc;
 	img = new QCanvasPixmap*[fc];
     }
     bool ok = TRUE;
-    bool arg = framecount > 1;
+    bool arg = fc > 0;
     if ( !arg )
 	framecount=1;
     for (int i=0; i<framecount; i++) {
@@ -2695,10 +2700,22 @@ void QCanvasView::setCanvas(QCanvas* canvas)
 
 void QCanvasView::updateContentsSize()
 {
-    if ( viewing )
+    if ( viewing ) {
+	if ( viewing->width() < contentsWidth() ) {
+	    QRect r(contentsToViewport(QPoint(viewing->width(),0)),
+		    QSize(contentsWidth()-viewing->width(),contentsHeight()));
+	    viewport()->erase(r);
+	}
+	if ( viewing->height() < contentsHeight() ) {
+	    QRect r(contentsToViewport(QPoint(0,viewing->height())),
+		    QSize(contentsWidth(),contentsHeight()-viewing->height()));
+	    viewport()->erase(r);
+	}
 	resizeContents(viewing->width(),viewing->height());
-    else
+    } else {
+	viewport()->erase();
 	resizeContents(1,1);
+    }
 }
 
 
@@ -3028,8 +3045,9 @@ void QCanvasPolygonalItem::draw(QPainter & p)
 void QCanvasPolygonalItem::setPen(QPen p)
 {
     if ( pn != p ) {
+	removeFromChunks();
 	pn = p;
-	changeChunks();
+	addToChunks();
     }
 }
 
@@ -3090,7 +3108,7 @@ void QCanvasPolygon::drawShape(QPainter & p)
 void QCanvasPolygon::setPoints(QPointArray pa)
 {
     removeFromChunks();
-    poly = pa;
+    poly = pa.copy();
     poly.translate((int)x(),(int)y());
     addToChunks();
 }
@@ -3133,7 +3151,7 @@ void QCanvasPolygon::moveBy(double dx, double dy)
 */
 QPointArray QCanvasPolygon::points() const
 {
-    QPointArray pa = areaPoints();
+    QPointArray pa = areaPoints().copy();
     pa.translate(int(-x()),int(-y()));
     return pa;
 }
@@ -3180,11 +3198,7 @@ QCanvasLine::~QCanvasLine()
 */
 void QCanvasLine::setPen(QPen p)
 {
-    if ( pen() != p ) {
-	removeFromChunks();
-	QCanvasPolygonalItem::setPen(p);
-	addToChunks();
-    }
+    QCanvasPolygonalItem::setPen(p);
 }
 
 /*!
