@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/dialogs/qmessagebox.cpp   2.3.2   edited 2001-10-21 $
+** $Id: qt/src/dialogs/qmessagebox.cpp   2.3.7   edited 2003-03-10 $
 **
 ** Implementation of QMessageBox class
 **
@@ -789,12 +789,20 @@ int QMessageBox::indexOf( int button ) const
 void QMessageBox::resizeButtons()
 {
     int i;
+#if defined(_WS_QWS_)
+    QSize maxSize( 0, 0 );
+#else
     QSize maxSize( style() == MotifStyle ? 0 : 75, 0 );
+#endif
     for ( i=0; i<mbd->numButtons; i++ ) {
 	QSize s = mbd->pb[i]->sizeHint();
 	maxSize.setWidth(  QMAX(maxSize.width(), s.width()) );
 	maxSize.setHeight( QMAX(maxSize.height(),s.height()) );
     }
+#if defined(_WS_QWS_)
+    // small buttons hard to hit with stylus
+    maxSize.setWidth( QMAX(maxSize.width(), 45) );
+#endif
     mbd->buttonSize = maxSize;
     for ( i=0; i<mbd->numButtons; i++ )
 	mbd->pb[i]->resize( maxSize );
@@ -1042,7 +1050,7 @@ void QMessageBox::adjustSize()
     // who commited QMessagebox? It really should use Layout. This way
     // we do not even have a proper minimum size we could take into
     // acount. ####
-    extern QRect qt_maxWindowRect;
+    extern Q_EXPORT QRect qt_maxWindowRect;
     QSize s( w, h );
     s = s.boundedTo( qt_maxWindowRect.size() );
     if ( s.width() < w && label->sizePolicy().hasHeightForWidth() ) {
