@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qaccel.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/src/kernel/qaccel.cpp   2.3.7   edited 2002-08-25 $
 **
 ** Implementation of QAccel class
 **
 ** Created : 950419
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -798,7 +798,7 @@ same dialog), you're on your own.
 <li><b><u>P</u></b>rint Here
 <li>P<b><u>r</u></b>operties
 <li><b><u>Q</u></b>uick View
-<li><b><u>R</u></b>edo (CDE: Ctrl-Y, Alt-Backspace)
+<li><b><u>R</u></b>edo (CDE: Ctrl-Y, Shift-Alt-Backspace)
 <li><b><u>R</u></b>epeat
 <li><b><u>R</u></b>estore
 <li><b><u>R</u></b>esume
