@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qapplication.cpp   2.3.2   edited 2001-10-13 $
+** $Id: qt/src/kernel/qapplication.cpp   2.3.7   edited 2002-10-31 $
 **
 ** Implementation of QApplication class
 **
@@ -296,6 +296,10 @@ QApplication::Type qt_appType=QApplication::Tty;
 QMutex * QApplication::qt_mutex		= 0;
 #endif
 
+#ifndef QT_NO_QWS_IM
+QBrush *qt_im_compose_background = 0;
+#endif
+
 // Default application palettes and fonts (per widget type)
 QAsciiDict<QPalette> *QApplication::app_palettes = 0;
 QAsciiDict<QFont>    *QApplication::app_fonts = 0;
@@ -305,7 +309,7 @@ QWidgetList *QApplication::popupWidgets = 0;	// has keyboard input focus
 static bool	   makeqdevel	 = FALSE;	// developer tool needed?
 static QWidget	  *desktopWidget = 0;		// root window widget
 extern QObject	  *qt_clipboard;		// global clipboard object
-#ifndef QT_NO_TRANSLATION
+#if !defined(QT_NO_TRANSLATION) && !defined(QT_NO_TEXTCODEC)
 static QTextCodec *default_codec = 0;		// root window widget
 #endif
 QWidgetList * qt_modal_stack=0; 		// stack of modal widgets
@@ -1834,6 +1838,7 @@ void QApplication::removeTranslator( QTranslator * mf )
 }
 
 
+#if !defined(QT_NO_TRANSLATION) && !defined(QT_NO_TEXTCODEC)
 /*!
   If the literal quoted text in the program is not in the Latin1
   encoding, this function can be used to set the appropriate encoding.
@@ -1877,6 +1882,7 @@ QTextCodec* QApplication::defaultCodec() const
 {
     return default_codec;
 }
+#endif
 
 /*!
   \overload
@@ -1935,9 +1941,11 @@ QString QApplication::translate( const char * scope, const char * key,
 		return result;
 	}
     }
+#if !defined(QT_NO_TEXTCODEC)
     if ( default_codec != 0 )
 	return default_codec->toUnicode(key);
     else
+#endif
 	return QString::fromLatin1(key);
 }
 
