@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qdragobject.cpp   2.3.2   edited 2001-08-30 $
+** $Id: qt/src/kernel/qdragobject.cpp   2.3.7   edited 2001-11-21 $
 **
 ** Implementation of Drag and Drop support
 **
@@ -92,157 +92,158 @@ struct QStoredDragData {
 
 // These pixmaps approximate the images in the Windows User Interface Guidelines.
 
-/* XPM */
+// XPM
+
 static const char * const move_xpm[] = {
 "11 20 3 1",
 ".	c None",
-#if defined(_WS_WIN_)
-" 	c #000000", // Windows cursor is traditionally white
-"X	c #FFFFFF",
+#if defined(Q_WS_WIN)
+"a	c #000000",
+"X	c #FFFFFF", // Windows cursor is traditionally white
 #else
-" 	c #FFFFFF", // X11 cursor is traditionally white
-"X	c #000000",
+"a	c #FFFFFF",
+"X	c #000000", // X11 cursor is traditionally black
 #endif
-"  .........",
-" X ........",
-" XX .......",
-" XXX ......",
-" XXXX .....",
-" XXXXX ....",
-" XXXXXX ...",
-" XXXXXXX ..",
-" XXXXXXXX .",
-" XXXXXXXXX ",
-" XXXXXX    ",
-" XXX XX ...",
-" XX  XX ...",
-" X .. XX ..",
-"  ... XX ..",
-" ..... XX .",
-"...... XX .",
-"....... XX ",
-"....... XX ",
-"........  ."};
+"aa.........",
+"aXa........",
+"aXXa.......",
+"aXXXa......",
+"aXXXXa.....",
+"aXXXXXa....",
+"aXXXXXXa...",
+"aXXXXXXXa..",
+"aXXXXXXXXa.",
+"aXXXXXXXXXa",
+"aXXXXXXaaaa",
+"aXXXaXXa...",
+"aXXaaXXa...",
+"aXa..aXXa..",
+"aa...aXXa..",
+"a.....aXXa.",
+"......aXXa.",
+".......aXXa",
+".......aXXa",
+"........aa."};
 
 /* XPM */
 static const char * const copy_xpm[] = {
 "24 30 3 1",
 ".	c None",
-" 	c #000000",
+"a	c #000000",
 "X	c #FFFFFF",
-#if defined(_WS_WIN_) // Windows cursor is traditionally white
-"  ......................",
-" X .....................",
-" XX ....................",
-" XXX ...................",
-" XXXX ..................",
-" XXXXX .................",
-" XXXXXX ................",
-" XXXXXXX ...............",
-" XXXXXXXX ..............",
-" XXXXXXXXX .............",
-" XXXXXX    .............",
-" XXX XX ................",
-" XX  XX ................",
-" X .. XX ...............",
-"  ... XX ...............",
-" ..... XX ..............",
-"...... XX ..............",
-"....... XX .............",
-"....... XX .............",
-"........  ...           ",
+#if defined(Q_WS_WIN) // Windows cursor is traditionally white
+"aa......................",
+"aXa.....................",
+"aXXa....................",
+"aXXXa...................",
+"aXXXXa..................",
+"aXXXXXa.................",
+"aXXXXXXa................",
+"aXXXXXXXa...............",
+"aXXXXXXXXa..............",
+"aXXXXXXXXXa.............",
+"aXXXXXXaaaa.............",
+"aXXXaXXa................",
+"aXXaaXXa................",
+"aXa..aXXa...............",
+"aa...aXXa...............",
+"a.....aXXa..............",
+"......aXXa..............",
+".......aXXa.............",
+".......aXXa.............",
+"........aa...aaaaaaaaaaa",
 #else
 "XX......................",
-"X X.....................",
-"X  X....................",
-"X   X...................",
-"X    X..................",
-"X     X.................",
-"X      X................",
-"X       X...............",
-"X        X..............",
-"X         X.............",
-"X      XXXX.............",
-"X   X  X................",
-"X  XX  X................",
-"X X..X  X...............",
-"XX...X  X...............",
-"X.....X  X..............",
-"......X  X..............",
-".......X  X.............",
-".......X  X.............",
-"........XX...           ",
+"XaX.....................",
+"XaaX....................",
+"XaaaX...................",
+"XaaaaX..................",
+"XaaaaaX.................",
+"XaaaaaaX................",
+"XaaaaaaaX...............",
+"XaaaaaaaaX..............",
+"XaaaaaaaaaX.............",
+"XaaaaaaXXXX.............",
+"XaaaXaaX................",
+"XaaXXaaX................",
+"XaX..XaaX...............",
+"XX...XaaX...............",
+"X.....XaaX..............",
+"......XaaX..............",
+".......XaaX.............",
+".......XaaX.............",
+"........XX...aaaaaaaaaaa",
 #endif
-"............. XXXXXXXXX ",
-"............. XXXXXXXXX ",
-"............. XXXX XXXX ",
-"............. XXXX XXXX ",
-"............. XX     XX ",
-"............. XXXX XXXX ",
-"............. XXXX XXXX ",
-"............. XXXXXXXXX ",
-"............. XXXXXXXXX ",
-".............           "};
+".............aXXXXXXXXXa",
+".............aXXXXXXXXXa",
+".............aXXXXaXXXXa",
+".............aXXXXaXXXXa",
+".............aXXaaaaaXXa",
+".............aXXXXaXXXXa",
+".............aXXXXaXXXXa",
+".............aXXXXXXXXXa",
+".............aXXXXXXXXXa",
+".............aaaaaaaaaaa"};
 
 /* XPM */
 static const char * const link_xpm[] = {
 "24 30 3 1",
 ".	c None",
-" 	c #000000",
+"a	c #000000",
 "X	c #FFFFFF",
-#if defined(_WS_WIN_) // Windows cursor is traditionally white
-"  ......................",
-" X .....................",
-" XX ....................",
-" XXX ...................",
-" XXXX ..................",
-" XXXXX .................",
-" XXXXXX ................",
-" XXXXXXX ...............",
-" XXXXXXXX ..............",
-" XXXXXXXXX .............",
-" XXXXXX    .............",
-" XXX XX ................",
-" XX  XX ................",
-" X .. XX ...............",
-"  ... XX ...............",
-" ..... XX ..............",
-"...... XX ..............",
-"....... XX .............",
-"....... XX .............",
-"........  ...           ",
+#if defined(Q_WS_WIN) // Windows cursor is traditionally white
+"aa......................",
+"aXa.....................",
+"aXXa....................",
+"aXXXa...................",
+"aXXXXa..................",
+"aXXXXXa.................",
+"aXXXXXXa................",
+"aXXXXXXXa...............",
+"aXXXXXXXXa..............",
+"aXXXXXXXXXa.............",
+"aXXXXXXaaaa.............",
+"aXXXaXXa................",
+"aXXaaXXa................",
+"aXa..aXXa...............",
+"aa...aXXa...............",
+"a.....aXXa..............",
+"......aXXa..............",
+".......aXXa.............",
+".......aXXa.............",
+"........aa...aaaaaaaaaaa",
 #else
 "XX......................",
-"X X.....................",
-"X  X....................",
-"X   X...................",
-"X    X..................",
-"X     X.................",
-"X      X................",
-"X       X...............",
-"X        X..............",
-"X         X.............",
-"X      XXXX.............",
-"X   X  X................",
-"X  XX  X................",
-"X X..X  X...............",
-"XX...X  X...............",
-"X.....X  X..............",
-"......X  X..............",
-".......X  X.............",
-".......X  X.............",
-"........XX...           ",
+"XaX.....................",
+"XaaX....................",
+"XaaaX...................",
+"XaaaaX..................",
+"XaaaaaX.................",
+"XaaaaaaX................",
+"XaaaaaaaX...............",
+"XaaaaaaaaX..............",
+"XaaaaaaaaaX.............",
+"XaaaaaaXXXX.............",
+"XaaaXaaX................",
+"XaaXXaaX................",
+"XaX..XaaX...............",
+"XX...XaaX...............",
+"X.....XaaX..............",
+"......XaaX..............",
+".......XaaX.............",
+".......XaaX.............",
+"........XX...aaaaaaaaaaa",
 #endif
-"............. XXXXXXXXX ",
-"............. XXX    XX ",
-"............. XXXX   XX ",
-"............. XXX    XX ",
-"............. XX   X XX ",
-"............. XX  XXXXX ",
-"............. XX XXXXXX ",
-"............. XXX XXXXX ",
-"............. XXXXXXXXX ",
-".............           "};
+".............aXXXXXXXXXa",
+".............aXXXaaaaXXa",
+".............aXXXXaaaXXa",
+".............aXXXaaaaXXa",
+".............aXXaaaXaXXa",
+".............aXXaaXXXXXa",
+".............aXXaXXXXXXa",
+".............aXXXaXXXXXa",
+".............aXXXXXXXXXa",
+".............aaaaaaaaaaa"};
 
 #ifndef QT_NO_DRAGANDDROP
 
