@@ -1,5 +1,5 @@
 /*****************************************************************************
-** $Id: qt/src/kernel/qgfxlinuxfb_qws.cpp   2.3.2   edited 2001-10-08 $
+** $Id: qt/src/kernel/qgfxlinuxfb_qws.cpp   2.3.7   edited 2003-05-26 $
 **
 ** Implementation of QLinuxFbScreen (unaccelerated Linux framebuffer) class for
 ** Embedded Qt
@@ -46,6 +46,10 @@
 #include "qgfxlinuxfb_qws.h"
 #include "qwindowsystem_qws.h"
 
+extern int qws_client_id;
+
+//#define DEBUG_CACHE
+
 // Used when there's no hardware acceleration, since there's no point
 // in storing the state in shared memory
 static int dummy_optype = 0;
@@ -74,6 +78,7 @@ static int dummy_lastop = 0;
 QLinuxFbScreen::QLinuxFbScreen( int display_id ) : QScreen( display_id )
 {
     canaccel=false;
+    clearCacheFunc = &clearCache;
 }
 
 /*!
@@ -111,6 +116,9 @@ bool QLinuxFbScreen::connect( const QString &displaySpec )
     fb_fix_screeninfo finfo;
     fb_var_screeninfo vinfo;
 
+    memset( &finfo, 0, sizeof(fb_fix_screeninfo) ); // keep valgrind happy
+    memset( &vinfo, 0, sizeof(fb_var_screeninfo) ); // keep valgrind happy
+
     /* Get fixed screen information */
     if (ioctl(fd, FBIOGET_FSCREENINFO, &finfo)) {
 	perror("reading /dev/fb0");
@@ -130,10 +138,9 @@ bool QLinuxFbScreen::connect( const QString &displaySpec )
     int xoff = vinfo.xoffset;
     int yoff = vinfo.yoffset;
     const char* qwssize;
-    if((qwssize=getenv("QWS_SIZE"))) {
-	sscanf(qwssize,"%dx%d",&w,&h);
+    if( (qwssize=getenv("QWS_SIZE")) && sscanf(qwssize,"%dx%d",&w,&h)==2 ) {
 	if ( (uint)w > vinfo.xres ) w = vinfo.xres;
-	if ( (uint)h > vinfo.xres ) h = vinfo.yres;
+	if ( (uint)h > vinfo.yres ) h = vinfo.yres;
 	dw=w;
 	dh=h;
 	xoff += (vinfo.xres - w)/2;
@@ -170,20 +177,7 @@ bool QLinuxFbScreen::connect( const QString &displaySpec )
     }
 
     if(canaccel) {
-	// Figure out position of offscreen memory
-	// Fetch size of pool entries table from memory
-	// Set up pool entries pointer table and 64-bit align it
-	int pos=(int)data;
-	pos+=size;
-	pos+=4096;
-	pos+=8;
-	pos&=~0x7;
-	entryp=((int *)pos);
-	lowest=((unsigned int *)pos)+1;
-	optype=((int *)pos)+2;
-	lastop=((int *)pos)+3;
-	pos+=(sizeof(int))*4;
-	entries=(QPoolEntry *)pos;
+	setupOffScreen();
     } else {
 	optype = &dummy_optype;
 	lastop = &dummy_lastop;
@@ -265,6 +259,9 @@ bool QLinuxFbScreen::initDevice()
     fb_var_screeninfo vinfo;
     fb_fix_screeninfo finfo;
 
+    memset( &finfo, 0, sizeof(fb_fix_screeninfo) ); // keep valgrind happy
+    memset( &vinfo, 0, sizeof(fb_var_screeninfo) ); // keep valgrind happy
+
     if (ioctl(fd, FBIOGET_VSCREENINFO, &vinfo)) {
 	qFatal("Error reading variable information in card init");
 	return false;
@@ -350,7 +347,7 @@ bool QLinuxFbScreen::initDevice()
 	    } else {
 		// Default 16 colour palette
 		// Green is now trolltech green so certain images look nicer 
-		//			     black  d_grey l_grey white  red  green  blue cyan magenta yellow
+		//			     black  d_grey l_grey white  red  green  blue yellow magenta cyan
 		unsigned char reds[16]   = { 0x00, 0x7F, 0xBF, 0xFF, 0xFF, 0xA2, 0x00, 0xFF, 0xFF, 0x00, 0x7F, 0x7F, 0x00, 0x00, 0x00, 0x82 };
 		unsigned char greens[16] = { 0x00, 0x7F, 0xBF, 0xFF, 0x00, 0xC5, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x7F };
 		unsigned char blues[16]  = { 0x00, 0x7F, 0xBF, 0xFF, 0x00, 0x11, 0xFF, 0x00, 0xFF, 0xFF, 0x00, 0x7F, 0x7F, 0x7F, 0x00, 0x00 };
@@ -462,25 +459,12 @@ bool QLinuxFbScreen::initDevice()
 	canaccel=false;
     }
 
+
     if(canaccel) {
-	// Figure out position of offscreen memory
-	// Set up pool entries pointer table and 64-bit align it
-	unsigned int pos=(unsigned int)data;
-	int psize;
-	psize=size;
-	psize+=4096;
-	psize+=8;
-	psize&=~0x7;
-	pos+=psize;
-	entryp=((int *)pos);
-	lowest=((unsigned int *)pos)+1;
-	// These keep track of accelerator state
-	optype=((int *)pos)+2;
-	lastop=((int *)pos)+3;
-	pos+=(sizeof(int))*4;
-	entries=(QPoolEntry *)pos;
+	setupOffScreen();
 	*entryp=0;
 	*lowest=mapsize;
+	insert_entry(*entryp,*lowest,*lowest);	// dummy entry to mark start
     } else {
 	optype = &dummy_optype;
 	lastop = &dummy_lastop;
@@ -511,44 +495,64 @@ bool QLinuxFbScreen::initDevice()
 
 void QLinuxFbScreen::delete_entry(int pos)
 {
-    if(pos>*entryp || pos<0) {
+    if (pos>*entryp || pos<0) {
 	qDebug("Attempt to delete odd pos! %d %d",pos,*entryp);
 	return;
     }
-    QPoolEntry * qpe=&entries[pos];
-    if(qpe->start<=*lowest && pos>0) {
+
+#ifdef DEBUG_CACHE
+    qDebug( "Remove entry: %d", pos );
+#endif
+
+    QPoolEntry *qpe = &entries[pos];
+    if (qpe->start <= *lowest) {
 	// Lowest goes up again
-	*lowest=(entries[pos-1].start);
+	*lowest = entries[pos-1].start;
+#ifdef DEBUG_CACHE
+	qDebug( "   moved lowest to %d", *lowest );
+#endif
     }
+
     (*entryp)--;
-    if(pos==*entryp) {
+    if (pos == *entryp)
 	return;
-    }
-    int size=(*entryp)-pos;
-    size++;
-    memmove(&entries[pos],&entries[pos+1],size*sizeof(QPoolEntry));
+
+    int size = (*entryp)-pos;
+    memmove(&entries[pos], &entries[pos+1], size*sizeof(QPoolEntry));
 }
 
 void QLinuxFbScreen::insert_entry(int pos,int start,int end)
 {
-    if(pos>*entryp) {
+    if (pos > *entryp) {
 	qDebug("Attempt to insert odd pos! %d %d",pos,*entryp);
 	return;
     }
 
-    if(pos==*entryp) {
+#ifdef DEBUG_CACHE
+    qDebug( "Insert entry: %d, %d -> %d", pos, start, end );
+#endif
+
+    if ( start < (int)*lowest ) {
+	*lowest = start;
+#ifdef DEBUG_CACHE
+	qDebug( "    moved lowest to %d", *lowest );
+#endif
+    }
+
+    if (pos==*entryp) {
 	entries[pos].start=start;
 	entries[pos].end=end;
+	entries[pos].clientId=qws_client_id;
 	(*entryp)++;
 	return;
     }
 
-    (*entryp)++;
     int size=(*entryp)-pos;
-    size++;
     memmove(&entries[pos+1],&entries[pos],size*sizeof(QPoolEntry));
     entries[pos].start=start;
     entries[pos].end=end;
+    entries[pos].clientId=qws_client_id;
+    (*entryp)++;
 }
 
 /*!
@@ -573,70 +577,95 @@ uchar * QLinuxFbScreen::cache(int amount, int optim)
 
     qt_fbdpy->grab();
 
+    int startp = cacheStart + (*entryp+1) * sizeof(QPoolEntry);
+    if ( startp >= (int)*lowest ) {
+	// We don't have room for another cache QPoolEntry.
+#ifdef DEBUG_CACHE
+	qDebug( "No room for pool entry in VRAM" );
+#endif
+	qt_fbdpy->ungrab();
+	return 0;
+    }
+
     int align=pixmapOffsetAlignment();
 
-    int hold=(*entryp-1);
-
-    for(int loopc=0;loopc<hold;loopc++) {
-	int freestart=entries[loopc+1].end;
-	int freeend=entries[loopc].start;
-	if(freestart!=freeend) {
-	    while(freestart % align) {
-		freestart++;
-	    }
-	    int len=freeend-freestart;
-	    if(len>amount) {
-		insert_entry(loopc+1,freestart,freestart+amount);
-		if(freestart % align) {
-		    qDebug("Wah, freed-block return unaligned %x",freestart);
+    if ( *entryp > 1 ) {
+	// Try to find a gap in the allocated blocks.
+	for (int loopc = 0; loopc < *entryp-1; loopc++) {
+	    int freestart = entries[loopc+1].end;
+	    int freeend = entries[loopc].start;
+	    if (freestart != freeend) {
+		while (freestart % align) {
+		    freestart++;
+		}
+		int len=freeend-freestart;
+		if (len >= amount) {
+		    insert_entry(loopc+1, freestart, freestart+amount);
+		    qt_fbdpy->ungrab();
+		    return data+freestart;
 		}
-		qt_fbdpy->ungrab();
-		return data+freestart;
 	    }
 	}
     }
 
     // No free blocks in already-taken memory; get some more
     // if we can
-    int stackend=(*entryp)*sizeof(QPoolEntry);
-    int startp=size+(sizeof(int)*2)+stackend;
-    int newlowest=(*lowest)-amount;
-    if(newlowest % align) {
-       	newlowest-=align;
-	while(newlowest % align) {
+    int newlowest = (*lowest)-amount;
+    if (newlowest % align) {
+       	newlowest -= align;
+	while (newlowest % align) {
 	    newlowest++;
 	}
     }
-    if(startp>=newlowest) {
-	//qDebug("No more memory, %d %d %d %d",startp,stackend,
-	       //*lowest,amount);
-	//canaccel=false;
+    if (startp >= newlowest) {
 	qt_fbdpy->ungrab();
+#ifdef DEBUG_CACHE
+	qDebug( "No VRAM available for %d bytes", amount );
+#endif
 	return 0;
     }
     insert_entry(*entryp,newlowest,*lowest);
-    *lowest=newlowest;
-    if(newlowest % align) {
-	//qDebug("Wah, new return is unaligned %x",newlowest);
-    }
     qt_fbdpy->ungrab();
     return data+newlowest;
 }
 
 /*!
-\fn void QLinuxFbScreen::uncache(uchar * c)
+Delete a block of memory allocated from graphics card memory.
+
+This function will first sync the graphics card to ensure the 
+memory isn't still being used by a command in the graphics 
+card fifo queue.
+
+You can speed up a driver by overriding uncache to avoid 
+syncing, however it will then be up to the driver to ensure
+the memory at \a c is no longer being used.  For example the
+driver might delay deleting the memory until it detects that
+all commands dealing with the memory are no longer in the
+queue.
+
+\sa deleteEntry() sync()
+*/
+void QLinuxFbScreen::uncache(uchar * c)
+{
+    sync();
+    deleteEntry(c);
+}
+
+/*!
 Delete a block of memory allocated from graphics card memory.
 */
-
-void QLinuxFbScreen::uncache(uchar * c)
+void QLinuxFbScreen::deleteEntry(uchar *c)
 {
     qt_fbdpy->grab();
     unsigned long pos=(unsigned long)c;
     pos-=((unsigned long)data);
     unsigned int hold=(*entryp);
-    for(unsigned int loopc=0;loopc<hold;loopc++) {
+    for(unsigned int loopc=1;loopc<hold;loopc++) {
 	if(entries[loopc].start==pos) {
-	    delete_entry(loopc);
+	    if (entries[loopc].clientId == qws_client_id)
+		delete_entry(loopc);
+	    else
+		qDebug("Attempt to delete client id %d cache entry", entries[loopc].clientId );
 	    qt_fbdpy->ungrab();
 	    return;
 	}
@@ -645,6 +674,49 @@ void QLinuxFbScreen::uncache(uchar * c)
     qDebug("Attempt to delete unknown offset %ld",pos);
 }
 
+/*
+Remove all entries from the cache for clientId.
+Should only be necessary if a client exits abnormally.
+*/
+void QLinuxFbScreen::clearCache( QScreen *instance, int clientId )
+{
+    QLinuxFbScreen *screen = (QLinuxFbScreen *)instance;
+    if ( !screen->canaccel || !screen->entryp )
+	return;
+    qt_fbdpy->grab();
+    for (int loopc = 0; loopc < *(screen->entryp); loopc++) {
+	if (screen->entries[loopc].clientId == clientId) {
+	    screen->delete_entry(loopc);
+	    loopc--;
+	}
+    }
+    qt_fbdpy->ungrab();
+}
+
+
+void QLinuxFbScreen::setupOffScreen()
+{
+    // Figure out position of offscreen memory
+    // Set up pool entries pointer table and 64-bit align it
+    int psize = size;
+    psize += 4096;  // cursor data
+    psize += 8;	    // for alignment
+    psize &= ~0x7;  // align
+
+    unsigned int pos=(unsigned int)data;
+    pos += psize;
+    entryp = ((int *)pos);
+    lowest = ((unsigned int *)pos)+1;
+    // These keep track of accelerator state
+    optype = ((int *)pos)+2;
+    lastop = ((int *)pos)+3;
+    pos += (sizeof(int))*4;
+    entries = (QPoolEntry *)pos;
+
+    // beginning of offscreen memory available for pixmaps.
+    cacheStart = psize + sizeof(int)*4 + sizeof(QPoolEntry);
+}
+
 /*!
 \fn void QLinuxFbScreen::shutdownDevice()
 This is called by the Qt/Embedded server when it shuts down, and should
@@ -655,11 +727,6 @@ and screen blanking.
 
 void QLinuxFbScreen::shutdownDevice()
 {
-    // Set back the original mode
-#ifndef QT_NO_QWS_CURSOR
-    qt_screencursor->hide();
-#endif
-
     // Causing crashes. Not needed.
     //setMode(startupw,startuph,startupd);
 /*
@@ -803,8 +870,9 @@ void QLinuxFbScreen::restore()
     }
 }
 
-void QLinuxFbScreen::blank(bool on)
+void QLinuxFbScreen::blank(bool /*on*/)
 {
+#if 0
 #if defined(QT_QWS_IPAQ)
     if ( on )
 	system("apm -suspend");
@@ -819,6 +887,7 @@ void QLinuxFbScreen::blank(bool on)
 #endif
 #endif
 #endif
+#endif
 }
 
 extern "C" QScreen * qt_get_screen_linuxfb(int display_id)
