@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qgfxmach64_qws.cpp   2.3.2   edited 2001-02-18 $
+** $Id: qt/src/kernel/qgfxmach64_qws.cpp   2.3.7   edited 2003-02-04 $
 **
 ** Implementation of QGfxMach64 (graphics context) class for Mach64 cards
 *
@@ -210,7 +210,7 @@ public:
 
     void hsync(int);
 
-    void setPixWidth(int,int,int=16,bool=true);
+    void setPixWidth(int,int,int=16,bool=TRUE);
 
 private:
 
@@ -230,6 +230,7 @@ template<const int depth,const int type>
 QGfxMach64<depth,type>::QGfxMach64(unsigned char * a,int b,int c)
     : QGfxRaster<depth,type>(a,b,c)
 {
+    qDebug("QGfxMach64::QGfxMach64");
 }
 
 // Sets up the graphics engine hardware scissors - these cut off
@@ -292,7 +293,7 @@ inline void QGfxMach64<depth,type>::setPixWidth(int d,int s,int sc,bool b)
 }
 
 // Set up SRC_OFF_PITCH and DST_OFF_PITCH
-// Return true if they're both on the card
+// Return TRUE if they're both on the card
 // (That's the source and destination's offset from graphics memory
 // start and linestep) - both are specified as multiples of 64 bits
 // This is also used to verify that both the source and destination
@@ -328,7 +329,7 @@ inline bool QGfxMach64<depth,type>::checkSourceDest()
     return TRUE;
 }
 
-// Set up DST_OFF_PITCH, return false if it's not on the card
+// Set up DST_OFF_PITCH, return FALSE if it's not on the card
 // For lines, filled rects etc
 // This is similar to checkSourceDest but is used when there is no
 // source image data
@@ -350,6 +351,7 @@ inline bool QGfxMach64<depth,type>::checkDest()
 template<const int depth,const int type>
 void QGfxMach64<depth,type>::drawLine(int x1,int y1,int x2,int y2)
 {
+    qDebug( "QGfxMach64::drawLine()" );
     // No point going any further if the window isn't visible
     if(ncliprect<1)
         return;
@@ -385,8 +387,6 @@ void QGfxMach64<depth,type>::drawLine(int x1,int y1,int x2,int y2)
     // Note that the last operation used the 2d engine
     (*optype)=1;
 
-    int loopc;
-
     // Add the offset of the gfx - used to make the origin the right
     // place for windows
     x1=x1+xoffs;
@@ -395,30 +395,23 @@ void QGfxMach64<depth,type>::drawLine(int x1,int y1,int x2,int y2)
     y2=y2+yoffs;
 
     // Only cope with lines going from left to right
-    // - swap them round if this isn't true
-    if(x1>x2) {
-      int x3,y3;
-      x3=x2;
-      y3=y2;
-      x2=x1;
-      y2=y1;
-      x1=x3;
-      y1=y3;
-      x1=x3;
-      y1=y3;
+    // - swap them round if this isn't TRUE
+    if (x1 > x2) {
+	int x3=x2;
+	int y3=y2;
+	x2=x1;
+	y2=y1;
+	x1=x3;
+	y1=y3;
     }
 
     wait_for_fifo(1);
     regw(DP_FRGD_CLR,cpen.color().pixel());
 
     // Figure out distance between endpoints
-    int dx,dy;
-    dx=abs(x2-x1);
-    dy=abs(y2-y1);
+    int dx=x2-x1;
+    int dy=y2-y1;
 
-    // This does maintenance needed by the software mouse cursor
-    // If you're /always/ going to use a hardware cursor you can
-    // omit GFX_START/END for a small performance advantage
     GFX_START(QRect(x1, y1 < y2 ? y1 : y2, dx+1, QABS(dy)+1))
 
     // The clip region is defined as a series of rectangles
@@ -426,7 +419,7 @@ void QGfxMach64<depth,type>::drawLine(int x1,int y1,int x2,int y2)
     // these rectangles and re-draw the line - an alternative approach
     // would be to clip to the rectangle in software
 
-    for(loopc=0;loopc<ncliprect;loopc++) {
+    for ( int loopc=0; loopc < ncliprect; loopc++ ) {
 	// Code taken from Mach64 Programmer's Manual
         // Sets up Bresenham parameters
 	int mindelta,maxdelta;
@@ -450,6 +443,8 @@ void QGfxMach64<depth,type>::drawLine(int x1,int y1,int x2,int y2)
 	unsigned int rval=0x00000003;
 	rval=(rval & ~0x7) | (unsigned long)(ymajor | ydir | xdir);
 
+    qDebug( "cliprect[%d](%d,%d,%d,%d)", loopc, cliprect[loopc].x(), cliprect[loopc].y(),
+	    cliprect[loopc].right(), cliprect[loopc].bottom() );
 	do_scissors(cliprect[loopc]);
 
 	wait_for_fifo(6);
@@ -586,7 +581,7 @@ inline void QGfxMach64<depth,type>::blt(int rx,int ry,int w,int h,int sx, int sy
 	QGfxRaster<depth,type>::blt(rx,ry,w,h,sx,sy);
     }
 
-    bool canaccel=false;
+    bool canaccel=FALSE;
 
     // We can only handle inline/solid alpha values using the 3d pipeline
     // and a texture. Textures have to be a power of 2 in sizes so the
@@ -595,7 +590,7 @@ inline void QGfxMach64<depth,type>::blt(int rx,int ry,int w,int h,int sx, int sy
     if(srcdepth==32) {
 	if(alphatype==IgnoreAlpha || alphatype==SolidAlpha ||
 	   alphatype==InlineAlpha) {
-	    canaccel=true;
+	    canaccel=TRUE;
 	}
 
 	if(alphatype!=IgnoreAlpha) {
@@ -603,16 +598,16 @@ inline void QGfxMach64<depth,type>::blt(int rx,int ry,int w,int h,int sx, int sy
 	    int p=srclinestep/4;
 	    if(p!=1024 && p!=512 && p!=256 && p!=128 && p!=64 && p!=32 &&
 	       p!=16 && p!=8) {
-		canaccel=false;
+		canaccel=FALSE;
 	    }
 	}
     } else if(srcdepth==16 || srcdepth==8) {
 	if(alphatype==IgnoreAlpha) {
-	    canaccel=true;
+	    canaccel=TRUE;
 	}
     }
 
-    if(srctype==SourceImage && canaccel==false) {
+    if(srctype==SourceImage && canaccel==FALSE) {
 	QGfxRaster<depth,type>::blt(rx,ry,w,h,sx,sy);
 	return;
     }
@@ -1337,7 +1332,7 @@ public:
     virtual bool initDevice();
     virtual int initCursor(void*, bool);
     virtual void shutdownDevice();
-    virtual bool useOffscreen() { return true; }
+    virtual bool useOffscreen() { return TRUE; }
     virtual QGfx * createGfx(unsigned char *,int,int,int,int);
 
 protected:
@@ -1366,9 +1361,9 @@ public:
     virtual void saveUnder() {}
     virtual void drawCursor() {}
     virtual void draw() {}
-    virtual bool supportsAlphaCursor() { return false; }
+    virtual bool supportsAlphaCursor() { return FALSE; }
 
-    static bool enabled() { return false; }
+    static bool enabled() { return FALSE; }
 
 private:
 
@@ -1388,7 +1383,7 @@ bool QMachScreen::connect( const QString &displaySpec )
     if ( !QLinuxFbScreen::connect( displaySpec ) )
 	return FALSE;
 
-    canaccel=false;
+    canaccel=FALSE;
 
     // This is the 256-byte PCI config space information for the
     // card pointed to by QWS_CARD_SLOT, as read from /proc/bus/pci
@@ -1462,7 +1457,7 @@ bool QMachScreen::connect( const QString &displaySpec )
 
     // Yes, we detected the card correctly so can safely make accelerated
     // gfxen
-    canaccel=true;
+    canaccel=TRUE;
 
     return TRUE;
 }
@@ -1575,7 +1570,7 @@ bool QMachScreen::initDevice()
     regw(CLR_CMP_MASK,0xffffffff);
     regw(CLR_CMP_CNTL,0);
 
-    return true;
+    return TRUE;
 }
 
 // Create a hardware cursor if a software one wasn't explicitly requested
@@ -1586,11 +1581,11 @@ int QMachScreen::initCursor(void* e, bool init)
 #ifndef QT_NO_QWS_CURSOR
     extern bool qws_sw_cursor;
 
-    if(qws_sw_cursor==true) {
+    if(qws_sw_cursor==TRUE) {
 	return QLinuxFbScreen::initCursor(e,init);
     }
     qt_screencursor=new QMachCursor();
-    qt_screencursor->init(0,false);
+    qt_screencursor->init(0,FALSE);
 #endif
     return 0;
 }
@@ -1651,6 +1646,7 @@ QMachCursor::QMachCursor()
 
 QMachCursor::~QMachCursor()
 {
+    hide();
 }
 
 void QMachCursor::init(SWCursorData *,bool)
@@ -1668,9 +1664,13 @@ void QMachCursor::set(const QImage& image,int hx,int hy)
     hoty=hy;
 
     if(cursor->isNull()) {
-        qDebug("Null cursor image!");
-	abort();
-        return;
+	static QImage *img = 0;
+	if ( !img ) {
+	    img = new QImage(8,8,8,3);
+	    img->setColor(2, 0x00000000);
+	    img->fill(2);
+	}
+	cursor = img;
     }
 
     unsigned int offset=myoffset;
