@@ -1,9 +1,7 @@
 /*****************************************************************************
-** $Id: qt/src/kernel/qgfxtransformed_qws.cpp   2.3.2   edited 2001-10-25 $
+** $Id: qt/src/kernel/qgfxtransformed_qws.cpp   2.3.7   edited 2003-01-10 $
 **
-** Implementation of QGfxRaster (unaccelerated graphics context) class for
-** Embedded Qt
-** Created : 940721
+** Implementation of transformed gfx raster.
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -45,18 +43,26 @@
 #include <fcntl.h>
 #include <errno.h>
 
+//#define QT_TRANSFORM_VFB
 
-#ifndef _OS_QNX6_
-#define QT_TRANS_SCREEN_BASE	QLinuxFbScreen
-#include "qgfxlinuxfb_qws.h"
+#ifndef QT_TRANSFORM_VFB
+# ifndef _OS_QNX6_
+#  include "qgfxlinuxfb_qws.h"
+#  define QT_TRANS_SCREEN_BASE	QLinuxFbScreen
+# else
+#  define QT_TRANS_SCREEN_BASE	QQnxScreen
+#  include "qwsgfx_qnx.h"
+# endif
+# define QT_TRANS_CURSOR_BASE	QScreenCursor
+# define QT_TRANS_GFX_BASE	QGfxRaster
 #else
-#define QT_TRANS_SCREEN_BASE	QQnxScreen
-#include "qwsgfx_qnx.h"
+# define QT_TRANS_SCREEN_BASE	QVFbScreen
+# define QT_TRANS_CURSOR_BASE	QVFbScreenCursor
+# define QT_TRANS_GFX_BASE	QGfxVFb
 #endif
 
-#define QT_TRANS_CURSOR_BASE	QScreenCursor
-#define QT_TRANS_GFX_BASE	QGfxRaster
-
+class QTransformedScreen;
+static QTransformedScreen *qt_trans_screen = 0;
 
 class QTransformedScreen : public QT_TRANS_SCREEN_BASE
 {
@@ -84,12 +90,19 @@ public:
     virtual QRegion mapFromDevice( const QRegion &, const QSize & ) const;
     virtual int transformOrientation() const;
 
+    void setTransformation( Transformation t );
+
 private:
     Transformation trans;
     QScreen *driver;
 };
 
-static QTransformedScreen *qt_trans_screen;
+void qws_setScreenTransformation( int t )
+{
+    if ( qt_trans_screen )
+	qt_trans_screen->setTransformation( (QTransformedScreen::Transformation)t );
+}
+
 
 // Unaccelerated screen/driver setup. Can be overridden by accelerated
 // drivers
@@ -105,6 +118,15 @@ QTransformedScreen::~QTransformedScreen()
 {
 }
 
+void QTransformedScreen::setTransformation( Transformation t )
+{
+    trans = t;
+
+    QSize s = mapFromDevice( QSize(dw,dh) );
+    w = s.width();
+    h = s.height();
+}
+
 bool QTransformedScreen::connect( const QString &displaySpec )
 {
     if ( displaySpec.find( ":Rot270" ) >= 0 )
@@ -243,6 +265,61 @@ QRect QTransformedScreen::mapFromDevice( const QRect &r, const QSize &s ) const
     return tr.normalize();
 }
 
+template<class T>
+static inline void rotateLoopTemplate(
+	uchar *src, int srcBytesPerLine,
+	uchar *dst, int dstBytesPerLine, 
+	int width, int height,
+	QTransformedScreen::Transformation trans, bool mapToDevice )
+{
+    int dstXAdd = 0;
+    int dstYAdd = 0;
+    int dstXOfs = 0;
+    int dstYOfs = 0;
+    int srcYAdd = srcBytesPerLine - width * sizeof(T);
+
+//static int times = 0;
+//printf("called %i times\n", times++ );
+
+    if ( !mapToDevice ) {
+	if ( trans == QTransformedScreen::Rot90 )
+	    trans = QTransformedScreen::Rot270;
+	else if ( trans == QTransformedScreen::Rot270 )
+	    trans = QTransformedScreen::Rot90;
+    }
+
+    switch ( trans ) {
+	case QTransformedScreen::Rot90:
+	    dstXOfs = 0;
+	    dstYOfs = width - 1;
+	    dstXAdd = -dstBytesPerLine;
+	    dstYAdd = 1 * sizeof(T) + width * dstBytesPerLine;
+	    break;
+	case QTransformedScreen::Rot270:
+	    dstXOfs = height - 1;
+	    dstYOfs = 0;
+	    dstXAdd = dstBytesPerLine;
+	    dstYAdd = -1 * sizeof(T) - width * dstBytesPerLine;
+	    break;
+	default:
+	    dstXOfs = width - 1;
+	    dstYOfs = height - 1;
+	    dstXAdd = -1 * sizeof(T);
+	    dstYAdd = -dstBytesPerLine + width * sizeof(T);
+	    break;
+    };
+
+    T *dstPtr = (T *)(dst + dstYOfs * dstBytesPerLine) + dstXOfs;
+    T *srcPtr = (T *)src;
+    for ( int y = 0; y < height; y++ ) {
+	for ( int x = 0; x < width; x++ ) {
+	    *dstPtr = *srcPtr++;
+	    dstPtr = (T *)((uchar*)dstPtr + dstXAdd); // add dstXAdd number of bytes
+	}
+	srcPtr = (T *)((uchar*)srcPtr + srcYAdd); // add srcYAdd number of bytes
+	dstPtr = (T *)((uchar*)dstPtr + dstYAdd); // add dstYAdd number of bytes
+    }
+}
 
 QImage QTransformedScreen::mapToDevice( const QImage &img ) const
 {
@@ -264,83 +341,60 @@ QImage QTransformedScreen::mapToDevice( const QImage &img ) const
 	rimg.colorTable()[i] = img.colorTable()[i];
     }
 
-    #define START_LOOP for ( int y = 0; y < ih; y++ ) { \
-			   for ( int x = 0; x < iw; x++ ) {
-    #define END_LOOP } }
+    // Optimized image rotation code for nice bit depths
+    int d = img.depth();
+    if ( d == 8 || d == 16 || d == 32 ) {
+	int srcBytesPerLine = img.bytesPerLine();
+	int dstBytesPerLine = rimg.bytesPerLine();
+	uchar *srcBits = img.bits();
+	uchar *dstBits = rimg.bits();
+	switch ( d ) {
+	    case 8:
+		rotateLoopTemplate<uchar>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, TRUE );
+		break;
+	    case 16:
+		rotateLoopTemplate<ushort>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, TRUE );
+		break;
+	    case 32:
+		rotateLoopTemplate<uint>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, TRUE );
+		break;
+	}
+	rimg.setAlphaBuffer( img.hasAlphaBuffer() );
+	rimg.setOffset( img.offset() );
+	return rimg;
+    }
+
+    // Slower fall back code for image rotation for 1-bit and other depths
+    #define ROTATE_LOOP( X, Y, VAL ) \
+		    for ( int y = 0; y < ih; y++ ) { \
+			for ( int x = 0; x < iw; x++ ) { \
+			    rimg.setPixel( X, Y, VAL ); \
+			} \
+		    } \
+		    break;
 
-    int pixel;
     if ( img.depth() > 8 ) {
 	switch ( trans ) {
 	    case Rot90:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( y, iw - x - 1, pixel );
-		END_LOOP
-		break;
-
+		ROTATE_LOOP( y, iw - x - 1, img.pixel(x, y) )
 	    case Rot270:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( ih - y - 1, x, pixel );
-		END_LOOP
-		break;
-
+		ROTATE_LOOP( ih - y - 1, x, img.pixel(x, y) );
 	    default:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( iw - x - 1, ih - y - 1, pixel );
-		END_LOOP
+		ROTATE_LOOP( iw - x - 1, ih - y - 1, img.pixel(x, y) );
 	}
     } else {
 	switch ( trans ) {
 	    case Rot90:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( y, iw - x - 1, pixel );
-		END_LOOP
-		break;
-
+		ROTATE_LOOP( y, iw - x - 1, img.pixelIndex(x, y) );
 	    case Rot270:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( ih - y - 1, x, pixel );
-		END_LOOP
-		break;
-
+		ROTATE_LOOP( ih - y - 1, x, img.pixelIndex(x, y) );
 	    default:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( iw - x - 1, ih - y - 1, pixel );
-		END_LOOP
+		ROTATE_LOOP( iw - x - 1, ih - y - 1, img.pixelIndex(x, y) );
 	}
     }
 
-    #undef START_LOOP
-    #undef END_LOOP
+    #undef ROTATE_LOOP
 
-/*
-    for ( int y = 0; y < img.height(); y++ ) {
-	for ( int x = 0; x < img.width(); x++ ) {
-	    int pixel;
-	    if ( img.depth() > 8 )
-		pixel = img.pixel(x, y);
-	    else
-		pixel = img.pixelIndex(x, y);
-	    switch ( trans ) {
-		case Rot90:
-		    rimg.setPixel( y, img.width() - x - 1, pixel );
-		    break;
-
-		case Rot270:
-		    rimg.setPixel( img.height() - y - 1, x, pixel );
-		    break;
-
-		default:
-		    rimg.setPixel( img.width() - x - 1, img.height() - y - 1, pixel );
-	    }
-	}
-    }
-*/
     rimg.setAlphaBuffer( img.hasAlphaBuffer() );
     rimg.setOffset( img.offset() );
 
@@ -357,8 +411,8 @@ QImage QTransformedScreen::mapFromDevice( const QImage &img ) const
     int w = iw;
     int h = ih;
     if ( trans == Rot90 || trans == Rot270 ) {
-	w = img.height();
-	h = img.width();
+	w = ih;
+	h = iw;
     }
 
     QImage rimg( w, h, img.depth(), img.numColors(), img.bitOrder() );
@@ -367,79 +421,60 @@ QImage QTransformedScreen::mapFromDevice( const QImage &img ) const
 	rimg.colorTable()[i] = img.colorTable()[i];
     }
 
-    #define START_LOOP for ( int y = 0; y < ih; y++ ) { \
-			   for ( int x = 0; x < iw; x++ ) {
-    #define END_LOOP } }
+    // Optimized image rotation code for nice bit depths
+    int d = img.depth();
+    if ( d == 8 || d == 16 || d == 32 ) {
+	int srcBytesPerLine = img.bytesPerLine();
+	int dstBytesPerLine = rimg.bytesPerLine();
+	uchar *srcBits = img.bits();
+	uchar *dstBits = rimg.bits();
+	switch ( d ) {
+	    case 8:
+		rotateLoopTemplate<uchar>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, FALSE );
+		break;
+	    case 16:
+		rotateLoopTemplate<ushort>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, FALSE );
+		break;
+	    case 32:
+		rotateLoopTemplate<uint>( srcBits, srcBytesPerLine, dstBits, dstBytesPerLine, iw, ih, trans, FALSE );
+		break;
+	}
+	rimg.setAlphaBuffer( img.hasAlphaBuffer() );
+	rimg.setOffset( img.offset() );
+	return rimg;
+    }
+
+    // Slower fall back code for image rotation for 1-bit and other depths
+    #define ROTATE_LOOP( X, Y, VAL ) \
+		    for ( int y = 0; y < ih; y++ ) { \
+			for ( int x = 0; x < iw; x++ ) { \
+			    rimg.setPixel( X, Y, VAL ); \
+			} \
+		    } \
+		    break;
 
-    int pixel;
     if ( img.depth() > 8 ) {
 	switch ( trans ) {
 	    case Rot90:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( ih - y - 1, x, pixel );
-		END_LOOP
-		break;
+		ROTATE_LOOP( ih - y - 1, x, img.pixel(x, y) );
 	    case Rot270:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( y, iw - x - 1, pixel );
-		END_LOOP
-		break;
+		ROTATE_LOOP( y, iw - x - 1, img.pixel(x, y) )
 	    default:
-		START_LOOP
-		    pixel = img.pixel(x, y);
-		    rimg.setPixel( iw - x - 1, ih - y - 1, pixel );
-		END_LOOP
+		ROTATE_LOOP( iw - x - 1, ih - y - 1, img.pixel(x, y) );
 	}
     } else {
 	switch ( trans ) {
 	    case Rot90:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( ih - y - 1, x, pixel );
-		END_LOOP
-		break;
+		ROTATE_LOOP( ih - y - 1, x, img.pixelIndex(x, y) );
 	    case Rot270:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( y, iw - x - 1, pixel );
-		END_LOOP
-		break;
+		ROTATE_LOOP( y, iw - x - 1, img.pixelIndex(x, y) );
 	    default:
-		START_LOOP
-		    pixel = img.pixelIndex(x, y);
-		    rimg.setPixel( iw - x - 1, ih - y - 1, pixel );
-		END_LOOP
+		ROTATE_LOOP( iw - x - 1, ih - y - 1, img.pixelIndex(x, y) );
 	}
     }
 
-    #undef START_LOOP
-    #undef END_LOOP
+    #undef ROTATE_LOOP
 
-/*
-    for ( int y = 0; y < img.height(); y++ ) {
-	for ( int x = 0; x < img.width(); x++ ) {
-	    int pixel;
-	    if ( img.depth() > 8 )
-		pixel = img.pixel(x, y);
-	    else
-		pixel = img.pixelIndex(x, y);
-	    switch ( trans ) {
-		case Rot90:
-		    rimg.setPixel( img.height() - y - 1, x, pixel );
-		    break;
-
-		case Rot270:
-		    rimg.setPixel( y, img.width() - x - 1, pixel );
-		    break;
-
-		default:
-		    rimg.setPixel( img.width() - x - 1, img.height() - y - 1, pixel );
-	    }
-	}
-    }
-*/
     rimg.setAlphaBuffer( img.hasAlphaBuffer() );
     rimg.setOffset( img.offset() );
 
@@ -602,10 +637,6 @@ public:
 void QTransformedScreenCursor::init( SWCursorData *da, bool init )
 {
     QT_TRANS_CURSOR_BASE::init( da, init );
-    QSize s = qt_trans_screen->mapFromDevice( QSize(clipWidth, clipHeight) );
-    clipWidth = s.width();
-    clipHeight = s.height();
-    gfx->setClipRect( 0, 0, clipWidth, clipHeight );
 }
 
 void QTransformedScreenCursor::set( const QImage &image, int hotx, int hoty )
@@ -635,6 +666,7 @@ public:
 #if !defined(QT_NO_MOVIE) || !defined(QT_NO_TRANSFORMATIONS)
     virtual void stretchBlt( int rx,int ry,int w,int h, int sw,int sh );
 #endif
+    virtual void tiledBlt( int rx,int ry,int w,int h );
     
     inline int tx( int x, int y ) {
 	switch ( qt_trans_screen->transformation() ) {
@@ -663,6 +695,9 @@ public:
 
 protected:
     virtual void setSourceWidgetOffset( int x, int y );
+    void processSpans( int n, QPoint* point, int* width );
+    
+
     bool inDraw;
 };
 
@@ -757,21 +792,13 @@ void QGfxTransformedRaster<depth,type>::fillRect( int x, int y, int w, int h )
 template <const int depth, const int type>
 void QGfxTransformedRaster<depth,type>::drawPolygon( const QPointArray &a, bool w, int idx, int num )
 {
-    switch ( qt_trans_screen->transformation() ) {
-	case QTransformedScreen::Rot90:
-	    stitchedges=QPolygonScanner::Edge(QPolygonScanner::Bottom+QPolygonScanner::Left);
-	    break;
-	case QTransformedScreen::Rot180:
-	    stitchedges=QPolygonScanner::Edge(QPolygonScanner::Bottom+QPolygonScanner::Right);
-	    break;
-	case QTransformedScreen::Rot270:
-	    stitchedges=QPolygonScanner::Edge(QPolygonScanner::Top+QPolygonScanner::Right);
-	    break;
-	default:
-	    stitchedges=QPolygonScanner::Edge(QPolygonScanner::Left+QPolygonScanner::Top);
-	    break;
-    }
-    if ( inDraw ) {
+    // Because of stitchedges, we cannot transform first. However, if
+    // we draw an outline, edges do not matter and we can do a fastpath 
+    // solution. The brush offset logic is complicated enough, so we don't
+    // fastpath patternedbrush.
+    
+    if ( inDraw  || cpen.style()==NoPen || patternedbrush ) {
+	//slowpath
 	QT_TRANS_GFX_BASE<depth,type>::drawPolygon( a, w, idx, num );
     } else {
 	inDraw = TRUE;
@@ -786,9 +813,51 @@ void QGfxTransformedRaster<depth,type>::drawPolygon( const QPointArray &a, bool
 	QT_TRANS_GFX_BASE<depth,type>::drawPolygon( na, w, 0, num );
 	inDraw = FALSE;
     }
-    stitchedges=QPolygonScanner::Edge(QPolygonScanner::Left+QPolygonScanner::Top);
 }
 
+
+template <const int depth, const int type>
+void QGfxTransformedRaster<depth,type>::processSpans( int n, QPoint* point, int* width )
+{
+    if ( inDraw || patternedbrush && srcwidth != 0 && srcheight != 0 ) {
+	//in the patternedbrush case, we let blt do the transformation
+	// so we leave inDraw false.
+	QT_TRANS_GFX_BASE<depth,type>::processSpans( n, point, width );		    
+    } else {
+	inDraw = TRUE;
+	while (n--) {
+	    if ( *width > 0 ) {
+		int x=tx(point->x(),point->y())+xoffs;
+		int y=ty(point->x(),point->y())+yoffs;
+
+		switch( qt_trans_screen->transformation() ) {
+		case QTransformedScreen::Rot90:
+		    vline( x, y-(*width-1), y );
+		    break;
+		case QTransformedScreen::Rot180:
+		    hline( x - (*width-1), x, y );
+		    break;
+		case QTransformedScreen::Rot270:
+		    vline( x, y, y+*width-1 );
+		    break;
+		default:
+		    hline( x, x+*width-1, y );
+		    break;
+		}
+	    }
+	    point++;
+	    width++;
+	}
+	inDraw = FALSE;
+    }
+
+}
+
+
+
+
+
+
 template <const int depth, const int type>
 void QGfxTransformedRaster<depth,type>::drawPolyline( const QPointArray &a, int idx, int num )
 {
@@ -815,11 +884,16 @@ void QGfxTransformedRaster<depth,type>::blt( int x, int y, int w, int h, int sx,
     if ( w == 0 || h == 0 )
 	return;
     QRect r;
-    r.setCoords( tx(x,y), ty(x,y), tx(x+w-1,y+h-1), ty(x+w-1,y+h-1) );
-    r = r.normalize();
     int rsx;
     int rsy;
-    switch ( qt_trans_screen->transformation() ) {
+    if ( inDraw ) {
+	r = QRect( x, y, w, h );
+	rsx = sx;
+	rsy = sy;
+    } else {
+	r.setCoords( tx(x,y), ty(x,y), tx(x+w-1,y+h-1), ty(x+w-1,y+h-1) );
+	r = r.normalize();
+	switch ( qt_trans_screen->transformation() ) {
 	case QTransformedScreen::Rot90:
 	    rsx = sy;
 	    rsy = srcwidth - sx - w;
@@ -836,6 +910,7 @@ void QGfxTransformedRaster<depth,type>::blt( int x, int y, int w, int h, int sx,
 	    rsx = sx;
 	    rsy = sy;
 	    break;
+	}
     }
     QT_TRANS_GFX_BASE<depth,type>::blt( r.x(), r.y(), r.width(), r.height(), rsx, rsy );
 }
@@ -856,6 +931,52 @@ void QGfxTransformedRaster<depth,type>::stretchBlt( int x, int y, int w, int h,
 }
 #endif
 
+template <const int depth, const int type>
+void QGfxTransformedRaster<depth,type>::tiledBlt( int rx,int ry,int w,int h )
+{
+    if ( w <= 0 || h <= 0 )
+	return;
+    inDraw = TRUE;
+    QRect r;
+    r.setCoords( tx(rx,ry), ty(rx,ry), tx(rx+w-1,ry+h-1), ty(rx+w-1,ry+h-1) );
+    r = r.normalize();
+
+    QPoint oldBrushOffs = brushoffs;
+    int brx, bry;
+    switch ( qt_trans_screen->transformation() ) {
+	case QTransformedScreen::Rot90:
+	    brx = brushoffs.y();
+	    bry = srcwidth - brushoffs.x() - w;
+	    break;
+	case QTransformedScreen::Rot180:
+	    brx = srcwidth - brushoffs.x() - w;
+	    bry = srcheight - brushoffs.y() - h;
+	    break;
+	case QTransformedScreen::Rot270:
+	    brx = srcheight - brushoffs.y() - h;
+	    bry = brushoffs.x();
+	    break;
+	default:
+	    brx = brushoffs.x();
+	    bry = brushoffs.y();
+	    break;
+    }
+    brushoffs = QPoint( brx, bry );
+
+    int oldsw = srcwidth;
+    int oldsh = srcheight;
+    QSize s = qt_screen->mapToDevice( QSize(srcwidth,srcheight) );
+    srcwidth = s.width();
+    srcheight = s.height();
+
+    QT_TRANS_GFX_BASE<depth,type>::tiledBlt( r.x(), r.y(), r.width(), r.height() );
+
+    srcwidth = oldsw;
+    srcheight = oldsh;
+    brushoffs = oldBrushOffs;
+    inDraw = FALSE;
+}
+
 int QTransformedScreen::initCursor(void* e, bool init)
 {
 #ifndef QT_NO_QWS_CURSOR
