@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qgfxvga16_qws.cpp   2.3.2   edited 2001-03-16 $
+** $Id: qt/src/kernel/qgfxvga16_qws.cpp   2.3.7   edited 2003-01-17 $
 **
 ** Implementation of QGfxVga16 (graphics context) class for VGA cards
 **
@@ -111,9 +111,9 @@ class Vga16ScreenGlobals
 	    vga_register_values = NULL;
 	    fb_line_ptrs = NULL;
 	    db_line_ptrs = NULL;
-	    
+
 	}
-	
+
 	~Vga16ScreenGlobals()
 	{
 	}
@@ -154,14 +154,14 @@ static QRect	Vga16DrawArea;
 	    				qt_vga16cursor->drawCursor(Vga16DrawArea); \
 				    endDraw(); \
 				}
-		
+
 #else //QT_NO_QWS_CURSOR
 
 # define VGA16_GFX_START(r)	if(is_screen_gfx) \
 				    beginDraw();
 # define VGA16_GFX_END		if(is_screen_gfx) \
 				    endDraw();
-			    
+
 #endif //QT_NO_QWS_CURSOR
 
 
@@ -170,13 +170,13 @@ static inline unsigned int closestMatch(int r, int g, int b)
 {
     QRgb *clut = qt_screen->clut();
     int clutcols = qt_screen->numCols();
-    
+
     if ( r>255 || g>255 || b>255 || r<0 || g<0 || b<0 )
 	abort();
 
     unsigned int distance, closest_distance = ~0x00;
     int pos = 0;
-    
+
     for (int loopc = 0; loopc < clutcols; loopc++)
     {
 	QRgb a = clut[loopc];
@@ -281,7 +281,7 @@ class QGfxVga16 : public QGfxRasterBase , private QPolygonScanner
 
 	QGfxVga16(unsigned char *,int w,int h);
 	virtual ~QGfxVga16();
-	
+
 	virtual void drawPoint( int,int );
 	virtual void drawPoints( const QPointArray &,int,int );
 	virtual void drawLine( int,int,int,int );
@@ -309,8 +309,8 @@ class QGfxVga16 : public QGfxRasterBase , private QPolygonScanner
 
     private:
 
-	
-	void drawrect_4(unsigned int x1, unsigned int y1, 
+
+	void drawrect_4(unsigned int x1, unsigned int y1,
     	    unsigned int x2, unsigned int y2, unsigned char col);
 	unsigned int get_pixel_4_32(unsigned int x, unsigned int y);
 	void himageline_4(unsigned int y, unsigned int x1, unsigned int x2, unsigned char *srcdata, bool reverse = false);
@@ -329,7 +329,7 @@ class QGfxVga16 : public QGfxRasterBase , private QPolygonScanner
 QGfxVga16::QGfxVga16(unsigned char * a,int b,int c) : QGfxRasterBase(a,b,c)
 {
     BEGIN_PROFILING
-    
+
 //    setLineStep((depth*width+7)/8);
     Q_UNUSED(a);
     Q_UNUSED(b);
@@ -362,7 +362,7 @@ void QGfxVga16::setPixel( int x, int y, unsigned char col)
     }
 
     VGA16_CriticalSection cs;
-    
+
     unsigned char *fb_ptr = fb_line_ptrs[y] + (x >> 3);
     set_enable_set_reset_vga16(0x0F, 0);
     set_rotate_vga16(0x00, 0);
@@ -377,7 +377,7 @@ void QGfxVga16::setPixel( int x, int y, unsigned char col)
 unsigned char QGfxVga16::getPixel( int x, int y )
 {
     BEGIN_PROFILING
-    return (*(db_line_ptrs[y] + (x >> 1)) >> ((x & 1) << 2)) & 0x0F; 
+    return (*(db_line_ptrs[y] + (x >> 1)) >> ((x & 1) << 2)) & 0x0F;
 }
 
 
@@ -392,7 +392,7 @@ unsigned int QGfxVga16::get_pixel_4_32(unsigned int x, unsigned int y)
 void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
 {
     BEGIN_PROFILING
-    
+
     // Doing large areas a plane at a time looks ugly because
     // you may see breifly the screen while it is partially red or
     // not all planes filled in if this operation doesn't complete
@@ -417,13 +417,13 @@ void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
     int dst_y = y1, h = y2 - y1 + 1;
     int new_x1 = x1 & (~0x07);
     int new_x2 = (x2 & (~0x07)) + 7;
-    
+
     // alignment thingo
     new_x1 = new_x1 & ~31;
 
     new_x1 = (new_x1 <   0) ?   0 : new_x1;
     new_x2 = (new_x2 > 639) ? 639 : new_x2;
-    
+
     if (new_x1 <= new_x2)
     {
 	VGA16_CriticalSection cs;
@@ -434,7 +434,7 @@ void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
 	set_bit_mask_vga16(0xFF, 0);
 
 	for (unsigned int plane = 0; plane < 4; plane++ ) {
-	    
+
 	    set_write_planes_vga16(1 << plane, 0);
 
 	    // Inlining this for each plane might help performance heaps
@@ -444,20 +444,20 @@ void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
 		unsigned int *dst_ptr = (unsigned int *)dst_ptr1;
 		unsigned int *src_row = (unsigned int *)(db_line_ptrs[dst_y + j] + (new_x1 >> 1));
 		unsigned int src_pixels;
-		
+
 		for (int x = new_x1; x <= new_x2; x += 32) {
-		    
+
 		    // Optimised to 71 operations to do 32 pixels
 		    // This code doesn't look very readable though
 		    // It just moves bits around rearranging them
 		    // as required, going from pixels in 4 bit packed
 		    // mode to pixels in planar mode (4 1-bit planes).
-	    
+
 		    unsigned int src_dword4 = src_row[0] >> plane;
 		    unsigned int src_dword3 = src_row[1] >> plane;
 		    unsigned int src_dword2 = src_row[2] >> plane;
 		    unsigned int src_dword1 = src_row[3] >> plane;
-		    
+
 		    unsigned int tmp_dword1 = ((src_dword1) << 16)
 					    | ((src_dword3 & 0x00001111));
 		    unsigned int tmp_dword2 = ((src_dword2) << 16)
@@ -472,27 +472,27 @@ void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
 					    | ((tmp_dword2 & 0x00110011));
 		    unsigned int tmp_dword6 = ((tmp_dword1 & 0x11001100))
 					    | ((tmp_dword2 & 0x11001100) >> 8);
-					    
+
 		    unsigned int tmp_dword7 = ((tmp_dword3 & 0x00110011) << 8)
 					    | ((tmp_dword4 & 0x00110011));
 		    unsigned int tmp_dword8 = ((tmp_dword3 & 0x11001100))
 					    | ((tmp_dword4 & 0x11001100) >> 8);
-					    
+
 		    unsigned int tmp_dword9  = ((tmp_dword5 & 0x01010101) << 4)
 					     | ((tmp_dword7 & 0x01010101));
 		    unsigned int tmp_dword10 = ((tmp_dword5 & 0x10101010))
 					     | ((tmp_dword7 & 0x10101010) >> 4);
-					    
+
 		    unsigned int tmp_dword11 = ((tmp_dword6 & 0x01010101) << 4)
 					     | ((tmp_dword8 & 0x01010101));
 		    unsigned int tmp_dword12 = ((tmp_dword6 & 0x10101010))
 					     | ((tmp_dword8 & 0x10101010) >> 4);
-					     
+
 		    src_pixels  = (tmp_dword9  << 3)
 			       	| (tmp_dword10 << 2)
 			        | (tmp_dword11 << 1)
 				| (tmp_dword12 << 0);
-				
+
 		    src_row += 4;
 		    *dst_ptr++ = src_pixels;
 		}
@@ -505,7 +505,7 @@ void vga16_exposeDoubleBufferedRegion(int x1, int y1, int x2, int y2)
 static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w, int h)
 {
     BEGIN_PROFILING
-    
+
     // Plan of attack for this special case -
     // We copy the pixels from the double buffer in src to the dst
     // we then copy the double buffer to the screen at the 8 pixel
@@ -520,7 +520,7 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
     h = (src_y + h > 480) ? 480 - src_y : h;
     src_x = (src_x > 639) ? 639 : (src_x < 0) ? 0 : src_x;
     w = (src_x + w > 640) ? 640 - src_x : w;
-    
+
     int i_1st = 0, i_inc = 1;
     int j_1st = 0, j_inc = 320;
 
@@ -540,9 +540,9 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
     unsigned char *dst_line_ptr = db_line_ptrs[j_1st+dst_y];
 
     if (i_inc == 1) {
-	
+
 	for (int jcx = 0; jcx < h; jcx++) {
-	
+
 	    int i = 0;
 	    unsigned char shift1 = (src_x & 1) << 2;
 	    unsigned char shift2 = (dst_x & 1) << 2;
@@ -559,7 +559,7 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 		shift2 = 4 - shift2;
 		i++;
 	    }
-	    
+
 	    // we know shift1 will be now 0, therefore if shift2 is 0 too
 	    // then the src and dst are aligned nicely such that we can
 	    // do a memcpy. Else if shift2 is 4 then we have to fix
@@ -567,10 +567,10 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 	    if (shift2) {
 
 		// this is the slow case where we can't just memcpy
-		
-		unsigned char *src_ptr = src_line_ptr + ((src_x + i) >> 1);	
+
+		unsigned char *src_ptr = src_line_ptr + ((src_x + i) >> 1);
 		unsigned char *dst_ptr = dst_line_ptr + ((dst_x + i) >> 1);
-		
+
 		unsigned int *idst_ptr = (unsigned int *)dst_ptr;
 		unsigned int *isrc_ptr = (unsigned int *)src_ptr;
 		unsigned char last_src_pixel = *dst_ptr & 0x0F;
@@ -587,9 +587,9 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 		    last_src_pixel = src_pixel >> 28;
 		}
 
-		src_ptr = src_line_ptr + ((src_x + i) >> 1);	
+		src_ptr = src_line_ptr + ((src_x + i) >> 1);
 		dst_ptr = dst_line_ptr + ((dst_x + i) >> 1);
-		
+
 		// finishing off loop that does 2 pixels at a time
 		for (; i < w - 1; i += 2) {
 		    unsigned char src_pixel = *src_ptr++;
@@ -607,11 +607,11 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 	    } else {
 
 		// Fast case where we quickly move the pixels around with a memcpy
-		
+
 		unsigned char *src_ptr = src_line_ptr + ((src_x + i) >> 1);
 		unsigned char *dst_ptr = dst_line_ptr + ((dst_x + i) >> 1);
 		unsigned int memcpy_width = (w - i) >> 1;
-		
+
 		//for (; i < w - 1; i += 2)
 		//    *dst_ptr++ = *src_ptr++;
 
@@ -628,12 +628,12 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 	    src_line_ptr += j_inc;
 	    dst_line_ptr += j_inc;
 	}
-    
+
     } else {
 
 	// this section is the reverse case (src_y == dst_y && src_x < dst_x)
 	// horizontal scrolling uses this so it may need more optimization
-	
+
 	for (int jcx = 0; jcx < h; jcx++) {
 	    unsigned char shift1 = ((src_x + i_1st) & 1) << 2;
 	    unsigned char shift2 = ((dst_x + i_1st) & 1) << 2;
@@ -650,7 +650,7 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 	    src_line_ptr += 320;
 	    dst_line_ptr += 320;
 	}
-	
+
     }
 
 #if 0
@@ -685,7 +685,7 @@ static void vga16_bltScrToScr(int dst_x, int dst_y, int src_x, int src_y, int w,
 static void vga16_blt16BitBufToScr(int dst_x, int dst_y, unsigned short *srcdata, int w, int h)
 {
     BEGIN_PROFILING
-    
+
     unsigned int r,g,b, x = dst_x, i = w;
 
     while ( i-- ) {
@@ -698,7 +698,7 @@ static void vga16_blt16BitBufToScr(int dst_x, int dst_y, unsigned short *srcdata
 	*(db_line_ptrs[dst_y] + (x >> 1)) |= (col&0x0F) << ((x & 1) << 2);
 	x++;
     }
-    
+
     vga16_exposeDoubleBufferedRegion(dst_x, dst_y, dst_x + w - 1, dst_y + h - 1);
 }
 
@@ -706,7 +706,7 @@ static void vga16_blt16BitBufToScr(int dst_x, int dst_y, unsigned short *srcdata
 static void vga16_blt8BitBufToScr(int dst_x, int dst_y, unsigned char *src_data, int w, int h)
 {
     BEGIN_PROFILING
-    
+
     unsigned int x = dst_x, i = w;
 
     while ( i-- ) {
@@ -715,7 +715,7 @@ static void vga16_blt8BitBufToScr(int dst_x, int dst_y, unsigned char *src_data,
 	*(db_line_ptrs[dst_y] + (x >> 1)) |= (col&0x0F) << ((x & 1) << 2);
 	x++;
     }
-    
+
     vga16_exposeDoubleBufferedRegion(dst_x, dst_y, dst_x + w - 1, dst_y + h - 1);
 }
 
@@ -723,17 +723,17 @@ static void vga16_blt8BitBufToScr(int dst_x, int dst_y, unsigned char *src_data,
 //
 // This is fairly well optimised
 //
-void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1, 
+void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1,
 	unsigned int x2, unsigned int y2, unsigned char col)
 {
     BEGIN_PROFILING
-    
+
     // unsigned char shift;
     unsigned int i, j;
     unsigned char *db_line_ptr = db_line_ptrs[y1];
     unsigned char *fb_line_ptr = fb_line_ptrs[y1];
     unsigned char *first_db_line_ptr = db_line_ptr;
-    
+
     unsigned char middle_char = ((col & 0x0F) << 4) | (col & 0x0F);
     unsigned int tx1 = x1, tx2 = x2;
 
@@ -767,13 +767,13 @@ void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1,
     }
 
     VGA16_CriticalSection cs;
-    
+
     set_enable_set_reset_vga16(0x0F, 0);
     set_rotate_vga16(0x00, 0);
     set_mode_vga16(0x00, 0);
     set_write_planes_vga16(0x0F, 0);
     set_set_reset_vga16(col & 0x0F, 0);
-   
+
     // this is a special case for narrow vertical strips that will
     // be in the same byte and need a special mask.
     if ((x1 >> 3) == (x2 >> 3)) {
@@ -785,8 +785,8 @@ void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1,
     	    *fb_ptr |= 0xFE;
 	return;
     }
-    
-    unsigned char *fb_ptr = fb_line_ptr + (x1 >> 3); 
+
+    unsigned char *fb_ptr = fb_line_ptr + (x1 >> 3);
     set_bit_mask_vga16(0xFF >> (x1 & 7), 0);
     for (i = y1; i <= y2; i++, fb_ptr += 80)
         *fb_ptr |= 0xFE;
@@ -797,7 +797,7 @@ void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1,
     for (i = y1; i <= y2; i++, fb_ptr += 80)
         *fb_ptr |= 0xFE;
     x2 -= x2 & 7;
- 
+
     set_bit_mask_vga16(0xFF, 0);
     if (x1 < x2) {
 	unsigned int width = (x2 - x1) >> 3;
@@ -809,10 +809,10 @@ void QGfxVga16::drawrect_4(unsigned int x1, unsigned int y1,
 }
 
 
-void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, unsigned char *srcdata, bool reverse = false)
+void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, unsigned char *srcdata, bool reverse )
 {
     BEGIN_PROFILING
-    
+
     unsigned char SavedRow[640];
     unsigned int x = (!reverse) ? x1 : x2;
     unsigned int inc = (!reverse) ? 1 : -1;
@@ -821,13 +821,13 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 
     w = (w > 640) ? 640 : w;
 
-    
+
     if ((srcdepth==4) && (!ismasking))
     {
 	exit(0); // Should this really ever get called???
 
     } else if ((srcdepth==16) && (!ismasking)) {
-	
+
 	exit(0); // see if this is really getting called!!!
 
 	// If this hypothetically was needed it would like like this:
@@ -835,21 +835,21 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 	return;
 
     } else if (srcdepth==8) {
-    
+
 	if ( !ismasking ) {
 
 	    if (src_normal_palette) {
-		
+
 		// reverse is not expected
 		if (reverse) printf("reverse!\n"), exit(0);
-		
+
 		if (reverse)
 		    // If this hypothetically was needed it would like like this:
 		    vga16_blt8BitBufToScr(x1,y,srcdata-w+1,w,1);
 		else
 		    vga16_blt8BitBufToScr(x1,y,srcdata,w,1);
 		return;
-		
+
 	    } else {
 		while ( w-- ) {
 		    SavedRow[x] = transclut[*srcdata];
@@ -858,7 +858,7 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 		}
 	    }
 	} else {
-	
+
 	    while ( w-- ) {
 		if ( srctype==SourceImage ) {
 		    if (src_normal_palette)
@@ -868,17 +868,17 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 		    srcdata++;
 		}
 		bool masked = TRUE;
-		GET_MASKED(reverse);
+		GET_MASKED(reverse, w);
 		SavedRow[x] = (!masked) ? gv : getPixel( x, y );
 		x += inc;
 	    }
 	}
-	
+
 	vga16_blt8BitBufToScr(x1,y,SavedRow+x1,x2-x1+1,1);
 	return;
-	
+
     } else {
-	
+
 	if ( !ismasking ) {
 	    while ( w-- ) {
 		unsigned int val = get_value_32(srcdepth,&srcdata,reverse);
@@ -900,7 +900,7 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 		    gv = closestMatchUsingTable(r,g,b);
 		}
 		bool masked = TRUE;
-		GET_MASKED(reverse);
+		GET_MASKED(reverse, w);
 		SavedRow[x] = (!masked) ? gv : getPixel( x, y );
 		x += inc;
 	    }
@@ -914,10 +914,10 @@ void QGfxVga16::himageline_4(unsigned int y, unsigned int x1, unsigned int x2, u
 // Convert between pixel values for different depths
 // reverse can only be true if sdepth == depth
 GFX_INLINE unsigned int QGfxVga16::get_value_32(
-		       int sdepth, unsigned char **srcdata, bool reverse = false)
+		       int sdepth, unsigned char **srcdata, bool reverse )
 {
     BEGIN_PROFILING
-    
+
     unsigned int ret;
     if(sdepth==32) {
 	ret = *((unsigned int *)(*srcdata));
@@ -985,9 +985,9 @@ GFX_INLINE void QGfxVga16::calcPacking(
 			  int & frontadd,int & backadd,int & count)
 {
     BEGIN_PROFILING
-    
+
     Q_UNUSED(m);
-    
+
     frontadd = x2-x1+1;
     backadd = 0;
     count = 0;
@@ -1000,7 +1000,7 @@ GFX_INLINE void QGfxVga16::calcPacking(
 void QGfxVga16::setSource(const QPaintDevice * p)
 {
     BEGIN_PROFILING
-    
+
     QPaintDeviceMetrics qpdm(p);
     srclinestep=((QPaintDevice *)p)->bytesPerLine();
     srcdepth=qpdm.depth();
@@ -1047,7 +1047,7 @@ void QGfxVga16::setSource(const QPaintDevice * p)
 void QGfxVga16::setSource(const QImage * i)
 {
     BEGIN_PROFILING
-    
+
     srctype=SourceImage;
     srclinestep=i->bytesPerLine();
     srcdepth=i->depth();
@@ -1080,7 +1080,7 @@ void QGfxVga16::buildSourceClut(QRgb * cols,int numcols)
     }
 
     int loopc;
-    
+
     // Copy clut
     for(loopc=0;loopc<numcols;loopc++)
 	srcclut[loopc] = cols[loopc];
@@ -1096,7 +1096,7 @@ void QGfxVga16::buildSourceClut(QRgb * cols,int numcols)
 void QGfxVga16::drawPoint( int x, int y )
 {
     BEGIN_PROFILING
-    
+
     if(cpen.style()==NoPen)
 	return;
     x += xoffs;
@@ -1113,7 +1113,7 @@ void QGfxVga16::drawPoint( int x, int y )
 void QGfxVga16::drawPoints( const QPointArray & pa, int index, int npoints )
 {
     BEGIN_PROFILING
-    
+
     if(cpen.style()==NoPen)
 	return;
     usePen();
@@ -1138,7 +1138,7 @@ void QGfxVga16::drawPoints( const QPointArray & pa, int index, int npoints )
 void QGfxVga16::drawLine( int x1, int y1, int x2, int y2 )
 {
     BEGIN_PROFILING
-    
+
     if(cpen.style()==NoPen)
 	return;
 
@@ -1287,7 +1287,7 @@ void QGfxVga16::drawLine( int x1, int y1, int x2, int y2 )
 void QGfxVga16::drawThickLine( int x1, int y1, int x2, int y2 )
 {
     BEGIN_PROFILING
-    
+
     QPointArray pa(5);
     int w = cpen.width() - 1;
     double a = atan2( y2 - y1, x2 - x1 );
@@ -1295,14 +1295,14 @@ void QGfxVga16::drawThickLine( int x1, int y1, int x2, int y2 )
     double iy = sin(a) * w / 2;
 
     // No cap.
-    pa[0].setX( x1 + iy );
-    pa[0].setY( y1 - ix );
-    pa[1].setX( x2 + iy );
-    pa[1].setY( y2 - ix );
-    pa[2].setX( x2 - iy );
-    pa[2].setY( y2 + ix );
-    pa[3].setX( x1 - iy );
-    pa[3].setY( y1 + ix );
+    pa[0].setX( (int)(x1 + iy) );
+    pa[0].setY( (int)(y1 - ix) );
+    pa[1].setX( (int)(x2 + iy) );
+    pa[1].setY( (int)(y2 - ix) );
+    pa[2].setX( (int)(x2 - iy) );
+    pa[2].setY( (int)(y2 + ix) );
+    pa[3].setX( (int)(x1 - iy) );
+    pa[3].setY( (int)(y1 + ix) );
 
     pa[4] = pa[0];
 
@@ -1321,7 +1321,7 @@ void QGfxVga16::drawThickLine( int x1, int y1, int x2, int y2 )
 GFX_INLINE void QGfxVga16::hline( int x1,int x2,int y)
 {
     BEGIN_PROFILING
-    
+
     QRect cr;
     bool plot=inClip(x1,y,&cr);
     int x=x1;
@@ -1347,7 +1347,7 @@ GFX_INLINE void QGfxVga16::hAlphaLineUnclipped( int x1,int x2,
 						    unsigned char * alphas)
 {
     BEGIN_PROFILING
-    
+
     // First read in the destination line
     unsigned char *avp = alphas;
     unsigned char *srcptr = srcdata;
@@ -1360,7 +1360,7 @@ GFX_INLINE void QGfxVga16::hAlphaLineUnclipped( int x1,int x2,
     // SourcePen
     if (srctype == SourceImage) {
 	for (int loopc = 0, i = x1; i <= x2; loopc++, i++) {
-	
+
 	    srcval32 = get_value_32(srcdepth,&srcptr);
 
 	    if (alphatype == InlineAlpha)
@@ -1396,7 +1396,7 @@ GFX_INLINE void QGfxVga16::hAlphaLineUnclipped( int x1,int x2,
 	// srctype != SourceImage
 	srcval32 = clut[srccol];
 	srcval4 = srccol;
-	
+
     	for (int loopc = 0, i = x1; i <= x2; loopc++, i++) {
 
 	    if (alphatype == InlineAlpha)
@@ -1434,7 +1434,7 @@ GFX_INLINE void QGfxVga16::hAlphaLineUnclipped( int x1,int x2,
     if (!set_back_buffer) {
 	memcpy(TmpRow,db_line_ptrs[y],320);
     }
-    
+
     vga16_blt8BitBufToScr(x1,y,alphabuf4,x2-x1+1,1);
 
     // restore the double buffer row if drawing the mouse
@@ -1449,7 +1449,7 @@ GFX_INLINE void QGfxVga16::hAlphaLineUnclipped( int x1,int x2,
 void QGfxVga16::fillRect( int rx,int ry,int w,int h )
 {
     BEGIN_PROFILING
-    
+
     setAlphaType(IgnoreAlpha);
     ismasking=FALSE;
     if ( w <= 0 || h <= 0 ) return;
@@ -1546,7 +1546,7 @@ void QGfxVga16::fillRect( int rx,int ry,int w,int h )
 void QGfxVga16::drawPolyline( const QPointArray &a,int index, int npoints )
 {
     BEGIN_PROFILING
-    
+
     //int m=QMIN( index+npoints-1, int(a.size())-1 );
     VGA16_GFX_START(clipbounds)
     int loopc;
@@ -1564,7 +1564,7 @@ void QGfxVga16::drawPolyline( const QPointArray &a,int index, int npoints )
 void QGfxVga16::drawPolygon( const QPointArray &pa, bool winding, int index, int npoints )
 {
     BEGIN_PROFILING
-    
+
     useBrush();
     VGA16_GFX_START(clipbounds)
     if ( cbrush.style()!=QBrush::NoBrush )
@@ -1581,7 +1581,7 @@ void QGfxVga16::drawPolygon( const QPointArray &pa, bool winding, int index, int
 void QGfxVga16::processSpans( int n, QPoint* point, int* width )
 {
     BEGIN_PROFILING
-    
+
     while (n--) {
 	int x=point->x()+xoffs;
 	if ( *width > 0 ) {
@@ -1599,7 +1599,7 @@ void QGfxVga16::processSpans( int n, QPoint* point, int* width )
 void QGfxVga16::scroll( int rx,int ry,int w,int h,int sx, int sy )
 {
     BEGIN_PROFILING
-    
+
     if (!w || !h)
 	return;
 
@@ -1628,11 +1628,11 @@ void QGfxVga16::scroll( int rx,int ry,int w,int h,int sx, int sy )
 void QGfxVga16::blt( int rx,int ry,int w,int h,int sx,int sy )
 {
     BEGIN_PROFILING
-    
+
     if ( !w || !h ) return;
 
     int osrcdepth=srcdepth;
-    
+
     if(srctype==SourcePen) {
 	srclinestep=0;//w;
 	srcdepth=0;
@@ -1701,11 +1701,11 @@ void QGfxVga16::blt( int rx,int ry,int w,int h,int sx,int sy )
 	    if (plot) {
 
 		if ((srcbits == buffer) && (srctype == SourceImage)) {
-		
+
 	    	    vga16_bltScrToScr(x,ry,x-rx+sx,j+sy,x2-x+1,1);
-		    
+
 	    	} else {
-		
+
 		    unsigned char *srcptr = srcline;
 		    if ( srctype == SourceImage ) {
 			if ( srcdepth == 1) {
@@ -1739,7 +1739,7 @@ void QGfxVga16::blt( int rx,int ry,int w,int h,int sx,int sy )
 					       +(x-rx);
 			hAlphaLineUnclipped(x,x2,ry,srcptr,alphap);
 		    }
-		    
+
 		}
 
 	    }
@@ -1759,7 +1759,7 @@ void QGfxVga16::blt( int rx,int ry,int w,int h,int sx,int sy )
     }
 
     srcdepth=osrcdepth;
-    
+
     VGA16_GFX_END
 }
 
@@ -1768,7 +1768,7 @@ void QGfxVga16::blt( int rx,int ry,int w,int h,int sx,int sy )
 void QGfxVga16::stretchBlt( int rx,int ry,int w,int h,int sw,int sh )
 {
     BEGIN_PROFILING
-    
+
     Q_UNUSED(rx);
     Q_UNUSED(ry);
     Q_UNUSED(w);
@@ -1783,7 +1783,7 @@ void QGfxVga16::stretchBlt( int rx,int ry,int w,int h,int sw,int sh )
 void QGfxVga16::tiledBlt( int rx,int ry,int w,int h )
 {
     BEGIN_PROFILING
-    
+
     VGA16_GFX_START(QRect(rx+xoffs, ry+yoffs, w+1, h+1))
 
     useBrush();
@@ -1823,7 +1823,7 @@ void QGfxVga16::tiledBlt( int rx,int ry,int w,int h )
         yPos += drawH;
         yOff = 0;
     }
-    
+
     VGA16_GFX_END
 }
 
@@ -1866,7 +1866,7 @@ static int VGA16DummyOpType;
 bool QVga16Screen::connect( const QString &displaySpec )
 {
     BEGIN_PROFILING
-    
+
     fb_fix_screeninfo finfo;
     QRegExp r( "/dev/fb[0-9]+" );
     int len;
@@ -1879,7 +1879,7 @@ bool QVga16Screen::connect( const QString &displaySpec )
     close( fd );
     if ( res || finfo.type != FB_TYPE_VGA_PLANES )
 	return FALSE;
-	
+
     if ( !QLinuxFbScreen::connect( displaySpec ) )
 	return FALSE;
 
@@ -1919,7 +1919,7 @@ bool QVga16Screen::connect( const QString &displaySpec )
     if (screen_double_buffer == NULL)
 	return FALSE;
 	//printf("error getting screen_double_buffer memory\n"), exit(0);
-	
+
     unsigned char *db_line_ptr = screen_double_buffer;
 
     for (int i = 0; i < 480; i++)
@@ -1927,7 +1927,7 @@ bool QVga16Screen::connect( const QString &displaySpec )
 	db_line_ptrs[i] = db_line_ptr;
 	db_line_ptr += 640/2;
     }
-    
+
     return TRUE;
 }
 
@@ -1960,10 +1960,10 @@ bool QVga16Screen::initDevice()
 
     QLinuxFbScreen::initDevice();
     optype = &VGA16DummyOpType;
-    
+
     // turn off the blinking cursor
     write(1, "\033[?25l", 6);
-    
+
     // set the registers with default values
     set_read_map_vga16(0x00,1);
     set_memory_mode_vga16(0x06,1);
@@ -1979,14 +1979,14 @@ bool QVga16Screen::initDevice()
     memset(base_ptr, *base_ptr, 640*480/8);
     // clear the shared memory
     memset(screen_double_buffer, 0, 640*480/2);
-		
+
     return true;
 }
 
 int QVga16Screen::initCursor(void* e, bool init)
 {
     BEGIN_PROFILING
-    
+
 #ifndef QT_NO_QWS_CURSOR
     qt_screencursor=new QVga16Cursor();
     qt_screencursor->init((SWCursorData*)e - 1,init);
@@ -1999,14 +1999,14 @@ void QVga16Screen::shutdownDevice()
 {
     BEGIN_PROFILING
     // ProfiledApp.DumpRawProfileStats(stdout);
-    
+
     QLinuxFbScreen::shutdownDevice();
 }
 
 int QVga16Screen::alloc(unsigned int r, unsigned int g, unsigned int b)
 {
     BEGIN_PROFILING
-    
+
     return closestMatchUsingTable(r,g,b);
 }
 
@@ -2017,7 +2017,7 @@ QGfx * QVga16Screen::createGfx(unsigned char * b,int w,int h,int d_arg,int lines
     if (d_arg == 4) {
 	d = d_arg;
     	QGfx *ret = new QGfxVga16(b,w,h);
-	    
+
 	unsigned char *fb_line_ptr = b;
 
 	int x_offset = (640 - w) / 2;
@@ -2034,12 +2034,12 @@ QGfx * QVga16Screen::createGfx(unsigned char * b,int w,int h,int d_arg,int lines
 		fb_line_ptrs[i] = fb_line_ptr;
 		fb_line_ptr += 640/8;
 	}
-	
+
 	if (ret) {
     	    ret->setLineStep(linestep);
 
 	    VGA16_CriticalSection cs;
-	    
+
 	    // set the registers with default values
 	    // memset(vga_register_values, 0, 512);
 	    set_read_map_vga16(0x00,1);
@@ -2054,7 +2054,7 @@ QGfx * QVga16Screen::createGfx(unsigned char * b,int w,int h,int d_arg,int lines
     	    return ret;
        	}
     }
-    
+
     return QLinuxFbScreen::createGfx(b,w,h,d_arg,linestep);
 }
 
@@ -2071,7 +2071,7 @@ extern "C" QScreen * qt_get_screen_vga16( int display_id )
 void QVga16Cursor::set(const QImage& image, int hotx, int hoty)
 {
     BEGIN_PROFILING
-    
+
     restoreUnder();
     data->hotx = hotx;
     data->hoty = hoty;
@@ -2099,7 +2099,7 @@ void QVga16Cursor::set(const QImage& image, int hotx, int hoty)
 void QVga16Cursor::move(int x, int y)
 {
     BEGIN_PROFILING
-    
+
     restoreUnder();
 
     data->x = x;
@@ -2107,14 +2107,14 @@ void QVga16Cursor::move(int x, int y)
 
     data->bound = QRect( data->x - data->hotx, data->y - data->hoty,
 		   data->width+1, data->height+1 );
-   
+
     drawCursor(data->bound);
 }
 
 void QVga16Cursor::restoreUnder()
 {
     BEGIN_PROFILING
-    
+
     if (!data || !data->enable)
 	return;
 
@@ -2130,7 +2130,7 @@ void QVga16Cursor::restoreUnder()
 void QVga16Cursor::drawCursor(QRect &r)
 {
     BEGIN_PROFILING
-    
+
     int x = data->x - data->hotx;
     int y = data->y - data->hoty;
 
@@ -2152,7 +2152,7 @@ void QVga16Cursor::drawCursor(QRect &r)
 	gfx->blt(x,y,data->width,data->height,0,0);
 	force_set_pixel = false;
 	set_back_buffer = true;
-	
+
 	qt_sw_cursor = TRUE;
     }
 }
