@@ -1,5 +1,5 @@
 /*****************************************************************************
-** $Id: qt/src/kernel/qgfxvnc_qws.cpp   2.3.2   edited 2001-10-25 $
+** $Id: qt/src/kernel/qgfxvnc_qws.cpp   2.3.7   edited 2003-05-20 $
 **
 ** Implementation of QGfxvnc (remote frame buffer driver)
 ** Proof of concept driver only, but quite complete.
@@ -42,14 +42,21 @@
 #include "qwindowsystem_qws.h"
 #include "qgfxvnc_qws.h"
 #include "qsharedmemory.h"
+#include "qapplication.h"
 #include <qglobal.h>
 
-#if defined(_OS_QNX6_)
-#define VNCSCREEN_BASE QQnxScreen
-#include "qwsgfx_qnx.h"
+//#define QT_VNC_VFB
+
+#ifdef QT_VNC_VFB
+# define VNCSCREEN_BASE	QVFbScreen
 #else
-#define VNCSCREEN_BASE QLinuxFbScreen
-#include "qgfxlinuxfb_qws.h"
+# if defined(_OS_QNX6_)
+#  define VNCSCREEN_BASE QQnxScreen
+#  include "qwsgfx_qnx.h"
+# else
+#  define VNCSCREEN_BASE QLinuxFbScreen
+#  include "qgfxlinuxfb_qws.h"
+# endif
 #endif
 
 
@@ -93,7 +100,7 @@ public:
     bool success;
     QVNCServer *vncServer;
     unsigned char *shmrgn;
-	QSharedMemory shm;
+    QSharedMemory shm;
     QVNCHeader *hdr;
     bool virtualBuffer;
 };
@@ -365,7 +372,20 @@ bool QRfbClientCutText::read( QSocket *s )
 /*
  */
 QVNCServer::QVNCServer()
-    : QServerSocket( 5900, 0, 0 )
+    : QServerSocket( 5900, (int) 0, (QObject *) 0 )
+{
+    qDebug( "QVNCServer created" );
+    handleMsg = FALSE;
+    client = 0;
+    encodingsPending = 0;
+    cutTextPending = 0;
+    keymod = 0;
+    timer = new QTimer( this );
+    connect( timer, SIGNAL(timeout()), this, SLOT(checkUpdate()) );
+}
+
+QVNCServer::QVNCServer( int id )
+    : QServerSocket( 5900 + id, (int) 0, (QObject *) 0 )
 {
     qDebug( "QVNCServer created" );
     handleMsg = FALSE;
@@ -973,6 +993,7 @@ void QGfxVNC<depth,type>::drawLine( int x1,int y1,int x2,int y2 )
     QWSDisplay::grab( TRUE );
     QRect r;
     r.setCoords( x1+xoffs, y1+yoffs, x2+xoffs, y2+yoffs );
+    r = r.normalize();
     qvnc_screen->setDirty( r & clipbounds );
     QGfxRaster<depth,type>::drawLine( x1, y1, x2, y2 );
     QWSDisplay::ungrab();
@@ -1060,10 +1081,11 @@ QVNCScreen::QVNCScreen( int display_id ) : VNCSCREEN_BASE( display_id )
     qvnc_screen = this;
     optype = (int *)malloc(sizeof(int));
     lastop = (int *)malloc(sizeof(int));
+    screencols = 0;
 }
 
 QVNCScreen::~QVNCScreen() {
-	shm.destroy();
+//	shm.destroy();
 }
 
 bool QVNCScreen::connect( const QString &displaySpec )
@@ -1087,7 +1109,7 @@ bool QVNCScreen::connect( const QString &displaySpec )
 	    dh=h=480;
 	}
 	lstep = ( dw * d + 7 ) / 8;
-#if !defined(_OS_QNX6_)
+#if !defined(_OS_QNX6_) && !defined(QT_VNC_VFB)
 	dataoffset = 0;
 	canaccel = FALSE;
 	optype = &dummy_optype;
@@ -1106,10 +1128,11 @@ bool QVNCScreen::connect( const QString &displaySpec )
 		tmpSpec.remove (0, next + 1); 
 		VNCSCREEN_BASE::connect( tmpSpec );
     }
-	shm = QSharedMemory(sizeof(QVNCHeader) + vsize + 8, qws_qtePipeFilename() );
-	shm.create();
-	shm.attach();
-	shmrgn = (unsigned char*)shm.base();
+
+    shm = QSharedMemory(sizeof(QVNCHeader) + vsize + 8, qws_qtePipeFilename() );
+    shm.create();
+    shm.attach();
+    shmrgn = (unsigned char*)shm.base();
 
     hdr = (QVNCHeader *) shmrgn;
 
@@ -1129,7 +1152,7 @@ bool QVNCScreen::initDevice()
 {
     if ( !virtualBuffer ) 
 		VNCSCREEN_BASE::initDevice();
-    vncServer = new QVNCServer();
+    vncServer = new QVNCServer( displayId );
 
     hdr->dirty = FALSE;
     memset( qvnc_screen->hdr->map, 0, MAP_WIDTH*MAP_HEIGHT );
