@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qkeyboard_qws.cpp   2.3.2   edited 2001-10-21 $
+** $Id: qt/src/kernel/qkeyboard_qws.cpp   2.3.7   edited 2003-05-30 $
 **
 ** Implementation of Qt/Embedded keyboard drivers
 **
@@ -44,6 +44,9 @@
 #include <ctype.h>
 
 #include <unistd.h>
+#ifdef _OS_LINUX_
+#include <linux/kd.h>
+#endif
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -66,20 +69,37 @@ extern "C" {
 #if !defined(_OS_QNX6_)
 
 #include <termios.h>
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
 #include <sys/kd.h>
 #include <sys/vt.h>
 #endif
 
-#ifdef QT_QWS_CUSTOM
+#ifdef QT_QWS_SL5XXX
 #include <asm/sharp_char.h>
 #endif
 
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
+static int dir_keyrot = -1;
 
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
 static int xform_dirkey(int key)
 {
-    int xf = qt_screen->transformOrientation();
+    if (dir_keyrot < 0) {
+	// get the rotation
+	char *kerot = getenv("QWS_CURSOR_ROTATION");
+	if (kerot) {
+	    if (strcmp(kerot, "90") == 0)
+		dir_keyrot = 1;
+	    else if (strcmp(kerot, "180") == 0)
+		dir_keyrot = 2;
+	    else if (strcmp(kerot, "270") == 0)
+		dir_keyrot = 3;
+	    else
+		dir_keyrot = 0;
+	} else {
+	    dir_keyrot = 0;
+	}
+    }
+    int xf = qt_screen->transformOrientation() + dir_keyrot;
     return (key-Qt::Key_Left+xf)%4+Qt::Key_Left;
 }
 #endif
@@ -111,7 +131,7 @@ private:
 
 #endif // QNX6
 
-#ifdef QT_QWS_CUSTOM
+#ifdef QT_QWS_SL5XXX
 static const QWSServer::KeyMap keyM[] = {
     {	Qt::Key_unknown,	0xffff  , 0xffff  , 0xffff  }, // 00
     {	Qt::Key_A,		'a'     , 'A'     , 'A'-64  }, // 01
@@ -266,7 +286,7 @@ static const QWSServer::KeyMap keyM[] = {
     {   Qt::Key_O,      'o'     , 'O'     , 'O'-64  },
     {   Qt::Key_P,      'p'     , 'P'     , 'P'-64  },
     {   Qt::Key_BraceLeft,  '['     , '{'     , 0xffff  },
-    {   Qt::Key_Escape,     ']'     , '}'     , 0xffff  },
+    {   Qt::Key_BraceRight, ']'     , '}'     , 0xffff  },
     {   Qt::Key_Return,     13      , 13      , 0xffff  },
     {   Qt::Key_Control,    0xffff  , 0xffff  , 0xffff  },
     {   Qt::Key_A,      'a'     , 'A'     , 'A'-64  },  // 30
@@ -334,6 +354,22 @@ static const QWSServer::KeyMap keyM[] = {
 };
 #endif
 static const int keyMSize = sizeof(keyM)/sizeof(QWSServer::KeyMap)-1;
+static QIntDict<QWSServer::KeyMap> *overrideMap = 0;
+
+/*!
+  Changes the mapping of the keyboard; adding the scancode to Unicode
+  mappings from \a map. The server takes over ownership of \a map
+  and will delete it. Use QCollection::setAutoDelete() to control
+  whether the contents of \a map should be deleted.
+
+  Passing a null pointer for \a map will revert to the default keymap.
+*/
+
+void QWSServer::setOverrideKeys( QIntDict<QWSServer::KeyMap> *map )
+{
+    delete overrideMap;
+    overrideMap = map;
+}
 
 #if !defined(_OS_QNX6_)
 
@@ -355,7 +391,7 @@ static const int keyMSize = sizeof(keyM)/sizeof(QWSServer::KeyMap)-1;
   filter it before sending it on to applications.
 
   <ul>
-  <li>\a unicode is the Unicode value for the key, or 0xFFFF is none is appropriate.
+  <li>\a unicode is the Unicode value for the key, or 0xffff if none is appropriate.
   <li>\a keycode is the Qt keycode for the key (see Qt::Key).
        for the list of codes).
   <li>\a modifiers is the set of modifier keys (see Qt::Modifier).
@@ -370,6 +406,11 @@ void QWSKeyboardHandler::processKeyEvent(int unicode, int keycode, int modifiers
     qwsServer->processKeyEvent( unicode, keycode, modifiers, isPress, autoRepeat );
 }
 
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
+#define QT_QWS_AUTOREPEAT_MANUALLY
+#endif
+
+
 class QWSPC101KeyboardHandler : public QWSKeyboardHandler
 {
     Q_OBJECT
@@ -379,24 +420,36 @@ public:
 
     void doKey(uchar scancode);
 
+    
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
+    void setAutoRepeat(int d, int p) { if ( d > 0 ) repeatdelay=d; 
+				       if ( p > 0 ) repeatperiod=p;}
+    void getAutoRepeat(int *d ,int *p ) { if (d) *d=repeatdelay; 
+    					  if (p) *p=repeatperiod; }
+#endif
+    void restoreLeds();
+    
 private slots:
     void autoRepeat();
 private:
     bool shift;
     bool alt;
     bool ctrl;
-#if defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_SL5XXX)
     bool meta;
     bool fn;
     bool numLock;
 #endif
     bool caps;
-    bool extended;
+#if defined(QT_QWS_IPAQ)
+    uint ipaq_return_pressed:1;
+#endif
+    int extended;
     int modifiers;
     int prevuni;
     int prevkey;
 
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
     // Could be used by other subclasses
     QTimer* repeater;
     int repeatdelay, repeatperiod;
@@ -405,19 +458,27 @@ private:
 
 void QWSPC101KeyboardHandler::autoRepeat()
 {
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
     processKeyEvent( prevuni, prevkey, modifiers, FALSE, TRUE );
     processKeyEvent( prevuni, prevkey, modifiers, TRUE, TRUE );
     repeater->start(repeatperiod);
 #endif
 }
 
+void QWSPC101KeyboardHandler::restoreLeds()
+{
+    char leds;
+    ioctl(0, KDGETLED, &leds);
+    leds = leds & ~LED_CAP;
+    if ( caps ) leds |= LED_CAP;
+    ioctl(0, KDSETLED, leds);
+}
 
 class QWSTtyKeyboardHandler : public QWSPC101KeyboardHandler
 {
     Q_OBJECT
 public:
-    QWSTtyKeyboardHandler();
+    QWSTtyKeyboardHandler(const QString&);
     virtual ~QWSTtyKeyboardHandler();
 
 private slots:
@@ -431,7 +492,7 @@ class QWSUsbKeyboardHandler : public QWSPC101KeyboardHandler
 {
     Q_OBJECT
 public:
-    QWSUsbKeyboardHandler();
+    QWSUsbKeyboardHandler(const QString& device);
     virtual ~QWSUsbKeyboardHandler();
 
 private slots:
@@ -486,7 +547,7 @@ private:
 
 static void vtSwitchHandler(int /*sig*/)
 {
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
     if (vtActive) {
 	qwsServer->enablePainting(false);
 	qt_screen->save();
@@ -516,16 +577,50 @@ static void vtSwitchHandler(int /*sig*/)
 // PC-101 type keyboards
 //
 
+
+
+static QWSPC101KeyboardHandler *currentPC101=0;
+
+bool qwsSetKeyboardAutoRepeat( int delay, int period )
+{
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
+    if ( currentPC101 )
+	currentPC101->setAutoRepeat( delay, period );
+    return currentPC101 != 0;
+#else
+    return FALSE;
+#endif
+}
+
+bool qwsGetKeyboardAutoRepeat( int *delay, int *period )
+{
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
+    if ( currentPC101 )
+	currentPC101->getAutoRepeat( delay, period );
+    return currentPC101 != 0;
+#else
+    return FALSE;
+#endif
+}
+
+void qwsRestoreKeyboardLeds()
+{
+    if ( currentPC101 )
+	currentPC101->restoreLeds();
+}
+
+
+
 QWSPC101KeyboardHandler::QWSPC101KeyboardHandler()
 {
     shift = false;
     alt   = false;
     ctrl  = false;
-    extended = false;
+    extended = 0;
     prevuni = 0;
     prevkey = 0;
     caps = FALSE;
-#if defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_SL5XXX)
     meta = FALSE;
     fn = FALSE;
 
@@ -541,79 +636,139 @@ QWSPC101KeyboardHandler::QWSPC101KeyboardHandler()
 	::close(dev);
     }
 #endif
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#if defined(QT_QWS_IPAQ)
+    // iPAQ Action Key has ScanCode 0x60: 0x60|0x80 = 0xe0 == extended mode 1 !
+    ipaq_return_pressed = FALSE;
+#endif
+#ifdef QT_QWS_AUTOREPEAT_MANUALLY
     repeatdelay = 400;
     repeatperiod = 80;
     repeater = new QTimer(this);
     connect(repeater, SIGNAL(timeout()), this, SLOT(autoRepeat()));
 #endif
+    currentPC101 = this;
 }
 
 QWSPC101KeyboardHandler::~QWSPC101KeyboardHandler()
 {
+    if ( currentPC101 == this )
+	currentPC101 = 0;
 }
 
 void QWSPC101KeyboardHandler::doKey(uchar code)
 {
+#if defined(QT_QWS_IPAQ)
+    // map ipaq 'action' key (0x60, 0xe0)
+    // to keycode for space.
+    if ((code & 0x7f) == 0x60) {
+	code = (code & 0x80) | 57;
+    }
+#endif
+
+    const QWSServer::KeyMap *currentKey = 0;
     int keyCode = Qt::Key_unknown;
     bool release = false;
     int keypad = 0;
-#if !defined(QT_QWS_CUSTOM)
-    if (code == 224) {
+#if !defined(QT_QWS_SL5XXX)
+    if (code == 224
+#if defined(QT_QWS_IPAQ) 
+	&& !ipaq_return_pressed
+#endif
+	) {
 	// extended
-	extended = true;
+	extended = 1;
 	return;
     }
+    else if (code == 225) {
+    	// extended 2
+    	extended = 2;
+    	return;
+    }
 #endif
 
+
+    /*------------------------------------------------------------------
+      First find the Qt KeyCode
+      ------------------------------------------------------------------*/
+
     if (code & 0x80) {
 	release = true;
 	code &= 0x7f;
     }
 
-    if (extended) {
+    if (extended == 1) {
+	currentKey = overrideMap ? overrideMap->find( code+0xe000 ) : 0;
+	if ( currentKey )
+	    keyCode = currentKey->key_code;
+	else
+	    switch (code) {
+	    case 72:
+		keyCode = Qt::Key_Up;
+		break;
+	    case 75:
+		keyCode = Qt::Key_Left;
+		break;
+	    case 77:
+		keyCode = Qt::Key_Right;
+		break;
+	    case 80:
+		keyCode = Qt::Key_Down;
+		break;
+	    case 82:
+		keyCode = Qt::Key_Insert;
+		break;
+	    case 71:
+		keyCode = Qt::Key_Home;
+		break;
+	    case 73:
+		keyCode = Qt::Key_Prior;
+		break;
+	    case 83:
+		keyCode = Qt::Key_Delete;
+		break;
+	    case 79:
+		keyCode = Qt::Key_End;
+		break;
+	    case 81:
+		keyCode = Qt::Key_Next;
+		break;
+	    case 28:
+		keyCode = Qt::Key_Enter;
+		break;
+	    case 53:
+		keyCode = Qt::Key_Slash;
+		break;
+	    case 0x1d:
+		keyCode = Qt::Key_Control;
+		break;
+	    case 0x2a:
+		keyCode = Qt::Key_SysReq;
+		break;
+	    case 0x38:
+		keyCode = Qt::Key_Alt;
+		break;
+	    case 0x5b:
+		keyCode = Qt::Key_Super_L;
+		break;
+	    case 0x5c:
+		keyCode = Qt::Key_Super_R;
+		break;
+	    case 0x5d:
+		keyCode = Qt::Key_Menu;
+		break;
+	    }
+    } else if ( extended == 2 ) {
 	switch (code) {
-	case 72:
-	    keyCode = Qt::Key_Up;
-	    break;
-	case 75:
-	    keyCode = Qt::Key_Left;
-	    break;
-	case 77:
-	    keyCode = Qt::Key_Right;
-	    break;
-	case 80:
-	    keyCode = Qt::Key_Down;
-	    break;
-	case 82:
-	    keyCode = Qt::Key_Insert;
-	    break;
-	case 71:
-	    keyCode = Qt::Key_Home;
-	    break;
-	case 73:
-	    keyCode = Qt::Key_Prior;
-	    break;
-	case 83:
-	    keyCode = Qt::Key_Delete;
-	    break;
-	case 79:
-	    keyCode = Qt::Key_End;
-	    break;
-	case 81:
-	    keyCode = Qt::Key_Next;
-	    break;
-	case 28:
-	    keyCode = Qt::Key_Enter;
-	    break;
-	case 53:
-	    keyCode = Qt::Key_Slash;
+	case 0x1d: 
+	    return;
+	case 0x45:
+	    keyCode = Qt::Key_Pause;
 	    break;
 	}
     } else {
-#if defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_SL5XXX)
 	if ( fn && !meta && (code >= 0x42 && code <= 0x52) ) {
-	    ushort unicode=0;
+	    ushort unicode=0xffff;
 	    int scan=0;
 	    if ( code == 0x42 ) { unicode='X'-'@'; scan=Key_X; } // Cut
 	    else if ( code == 0x43 ) { unicode='C'-'@'; scan=Key_C; } // Copy
@@ -625,73 +780,93 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
 	    }
 	}
 #endif
-	if (code < keyMSize) {
-	    keyCode = QWSServer::keyMap()[code].key_code;
+	currentKey = overrideMap ? overrideMap->find( code ) : 0;
+	if ( !currentKey && code < keyMSize ) {
+	    currentKey = &QWSServer::keyMap()[code];
 	}
+	if ( currentKey )
+	    keyCode = currentKey->key_code;
 
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX) // need autorepeat implemented here?
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX) // need autorepeat implemented here?
 	bool repeatable = TRUE;
 
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX) && !defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX) && !defined(QT_QWS_CUSTOM)
 	switch (code) {
-	    case 0x7a: case 0x7b: case 0x7c: case 0x7d:
-		keyCode = code - 0x7a + Key_F9;
-		repeatable = FALSE;
-		break;
-	    case 0x79:
-		keyCode = Key_SysReq;
-		repeatable = FALSE;
-		break;
-	    case 0x78:
-		keyCode = Key_Escape;
-		repeatable = FALSE;
-		break;
-	    case 0x60:
-		keyCode = Key_Return;
-		break;
-	    case 0x67:
-		keyCode = Key_Right;
-		break;
-	    case 0x69:
-		keyCode = Key_Up;
-		break;
-	    case 0x6a:
-		keyCode = Key_Down;
-		break;
-	    case 0x6c:
-		keyCode = Key_Left;
-		break;
+	case 0x7a: case 0x7b: case 0x7c: case 0x7d:
+	    keyCode = code - 0x7a + Key_F9;
+	    repeatable = FALSE;
+	    break;
+	case 0x79:
+	    keyCode = Key_SysReq;
+	    repeatable = FALSE;
+	    break;
+	case 0x78:
+# if defined(QT_QWS_IPAQ)
+	    keyCode = Key_F24;  // record
+# else
+	    keyCode = Key_Escape;
+# endif
+	    repeatable = FALSE;
+	    break;
+	case 0x60:
+	    keyCode = Key_Return;
+# ifdef QT_QWS_IPAQ
+	    ipaq_return_pressed = !release;
+# endif
+	    break;
+	case 0x67:
+	    keyCode = Key_Right;
+	    break;
+	case 0x69:
+	    keyCode = Key_Up;
+	    break;
+	case 0x6a:
+	    keyCode = Key_Down;
+	    break;
+	case 0x6c:
+	    keyCode = Key_Left;
+	    break;
 	}
 #endif
 
-#if defined(QT_QWS_CUSTOM)
+	/*------------------------------------------------------------------
+	  Then do special processing of magic keys
+	  ------------------------------------------------------------------*/
+
+
+#if defined(QT_QWS_SL5XXX)
 	if ( release && ( keyCode == Key_F34 || keyCode == Key_F35 ) )
 	    return; // no release for power and light keys
 	if ( keyCode >= Key_F1 && keyCode <= Key_F35
-		|| keyCode == Key_Escape || keyCode == Key_Home
-		|| keyCode == Key_Shift || keyCode == Key_Meta )
+	     || keyCode == Key_Escape || keyCode == Key_Home
+	     || keyCode == Key_Shift || keyCode == Key_Meta )
 	    repeatable = FALSE;
 #endif
 
 	if ( qt_screen->isTransformed()
-		&& keyCode >= Qt::Key_Left && keyCode <= Qt::Key_Down )
-	{
-	    keyCode = xform_dirkey(keyCode);
-	}
+	     && keyCode >= Qt::Key_Left && keyCode <= Qt::Key_Down )
+	    {
+		keyCode = xform_dirkey(keyCode);
+	    }
 
 	if ( repeatable && !release )
 	    repeater->start(repeatdelay,TRUE);
 	else
 	    repeater->stop();
 #endif
+	/*
+	  Translate shift+Key_Tab to Key_Backtab
+	*/
+	if (( keyCode == Key_Tab ) && shift )
+	    keyCode = Key_Backtab;
     }
 
     /*
       Keypad consists of extended keys 53 and 28,
       and non-extended keys 55 and 71 through 83.
     */
-    if ( extended ? (code == 53 || code == 28) :
-	 (code == 55 || ( code >= 71 && code <= 83 )) )
+    if (( extended == 1 ) ? (code == 53 || code == 28) :
+	(code == 55 || ( code >= 71 && code <= 83 )) )
 	keypad = Qt::Keypad;
 
     // Virtual console switching
@@ -705,13 +880,13 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
     if (term && !release) {
 	ctrl = false;
 	alt = false;
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
 	ioctl(kbdFD, VT_ACTIVATE, term);
 #endif
 	return;
     }
 
-#if defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_SL5XXX)
     // Ctrl-Alt-Delete exits qws
     if (ctrl && alt && keyCode == Qt::Key_Delete) {
 	qApp->quit();
@@ -723,7 +898,7 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
     }
 #endif
 
-#if defined(QT_QWS_CUSTOM)
+#if defined(QT_QWS_SL5XXX)
     if (keyCode == Qt::Key_F22) { /* Fn key */
 	fn = !release;
     } else if ( keyCode == Key_NumLock ) {
@@ -732,68 +907,84 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
     } else
 #endif
 
-    if (keyCode == Qt::Key_Alt) {
-	alt = !release;
-    } else if (keyCode == Qt::Key_Control) {
-	ctrl = !release;
-    } else if (keyCode == Qt::Key_Shift) {
-	shift = !release;
-#if defined(QT_QWS_CUSTOM)
-    } else if (keyCode == Qt::Key_Meta) {
-	meta = !release;
+	if (keyCode == Qt::Key_Alt) {
+	    alt = !release;
+	} else if (keyCode == Qt::Key_Control) {
+	    ctrl = !release;
+	} else if (keyCode == Qt::Key_Shift) {
+	    shift = !release;
+#if defined(QT_QWS_SL5XXX)
+	} else if (keyCode == Qt::Key_Meta) {
+	    meta = !release;
 #endif
-    } else if ( keyCode == Qt::Key_CapsLock && release ) {
-	caps = !caps;
-    }
-    if (keyCode != Qt::Key_unknown) {
+	} else if ( keyCode == Qt::Key_CapsLock && release ) {
+	    caps = !caps;
+#if defined(_OS_LINUX_) && !defined(QT_QWS_SL5XXX)
+	    char leds;
+	    ioctl(0, KDGETLED, &leds);
+	    leds = leds & ~LED_CAP;
+	    if ( caps ) leds |= LED_CAP;
+	    ioctl(0, KDSETLED, leds);
+#endif
+	}
+
+    /*------------------------------------------------------------------
+      Then find the Unicode value and send the event
+      ------------------------------------------------------------------*/
+    //If we map the keyboard to a non-latin1 layout, we may have
+    //valid keys with unknown key codes.
+    if ( currentKey || keyCode != Qt::Key_unknown ) {
 	bool bAlt = alt;
 	bool bCtrl = ctrl;
 	bool bShift = shift;
-	int unicode = 0;
-	if (code < keyMSize) {
-	    if (!extended) {
-                bool bCaps = caps ^ shift;
-#if defined(QT_QWS_CUSTOM)
-		if (fn) {
-		    if ( shift ) {
-			bCaps = bShift = FALSE;
-			bCtrl = TRUE;
-		    }
-		    if ( meta ) {
-			bCaps = bShift = TRUE;
-			bAlt = TRUE;
-		    }
-		} else if ( meta ) {
+	int unicode = 0xffff;
+	if ( currentKey ) {
+#if !defined(QT_QWS_SL5XXX)
+	    bool bCaps = shift ||
+			 (caps ? QChar(QWSServer::keyMap()[code].unicode).isLetter() : FALSE);
+#else
+	    bool bCaps = caps ^ shift;
+	    if (fn) {
+		if ( shift ) {
+		    bCaps = bShift = FALSE;
+		    bCtrl = TRUE;
+		}
+		if ( meta ) {
 		    bCaps = bShift = TRUE;
+		    bAlt = TRUE;
 		}
-		if ( code > 40 && caps ) {
-		    // fn-keys should only react to shift, not caps
-		    bCaps = bShift = shift;
-		}
-		if ( numLock ) {
-		    if ( keyCode != Key_Space && keyCode != Key_Tab )
-			bCaps = bShift = FALSE;
-		}
-		if ( keyCode == Key_Delete && (bAlt || bCtrl) ) {
-		    keyCode = Key_BraceLeft;
-		    unicode = '[';
-		    bCaps = bShift = bAlt = bCtrl = FALSE;
-		} else if (keyCode == Qt::Key_F31 && bCtrl) {
-		    keyCode = Key_QuoteLeft;
-		    unicode = '`';
-		} else
+	    } else if ( meta ) {
+		bCaps = bShift = TRUE;
+	    }
+	    if ( code > 40 && caps ) {
+		// fn-keys should only react to shift, not caps
+		bCaps = bShift = shift;
+	    }
+	    if ( numLock ) {
+		if ( keyCode != Key_Space && keyCode != Key_Tab )
+		    bCaps = bShift = FALSE;
+	    }
+	    if ( keyCode == Key_Delete && (bAlt || bCtrl) ) {
+		keyCode = Key_BraceLeft;
+		unicode = '[';
+		bCaps = bShift = bAlt = bCtrl = FALSE;
+	    } else if (keyCode == Qt::Key_F31 && bCtrl) {
+		keyCode = Key_QuoteLeft;
+		unicode = '`';
+	    } else
 #endif
 		
 		if (bCtrl)
-		    unicode =  QWSServer::keyMap()[code].ctrl_unicode ?  QWSServer::keyMap()[code].ctrl_unicode : 0xffff;
+		    unicode = currentKey->ctrl_unicode;
 		else if (bCaps)
-		    unicode =  QWSServer::keyMap()[code].shift_unicode ?  QWSServer::keyMap()[code].shift_unicode : 0xffff;
+		    unicode = currentKey->shift_unicode;
 		else
-		    unicode =  QWSServer::keyMap()[code].unicode ?  QWSServer::keyMap()[code].unicode : 0xffff;
-	    } else {
-		if ( code == 53 )
-		    unicode = '/';
-	    }
+		    unicode = currentKey->unicode;
+	} else if ( extended == 1 ) {
+	    if ( keyCode == Qt::Key_Slash )
+		unicode = '/';
+	    else if ( keyCode == Qt::Key_Enter )
+		unicode = 0xd;
 	}
 
 	modifiers = 0;
@@ -816,7 +1007,7 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
 	    prevkey = prevuni = 0;
 	}
     }
-    extended = false;
+    extended = 0;
 }
 
 
@@ -824,60 +1015,60 @@ void QWSPC101KeyboardHandler::doKey(uchar code)
 // Tty keyboard
 //
 
-QWSTtyKeyboardHandler::QWSTtyKeyboardHandler()
+QWSTtyKeyboardHandler::QWSTtyKeyboardHandler(const QString& device)
 {
-    kbdFD=open("/dev/tty0", O_RDWR | O_NDELAY, 0);
+    kbdFD=open(device.isEmpty() ? "/dev/tty0" : device.latin1(), O_RDWR | O_NDELAY, 0);
 
     if ( kbdFD >= 0 ) {
 	QSocketNotifier *notifier;
 	notifier = new QSocketNotifier( kbdFD, QSocketNotifier::Read, this );
 	connect( notifier, SIGNAL(activated(int)),this,
 		 SLOT(readKeyboardData()) );
+
+	// save for restore.
+	tcgetattr( kbdFD, &origTermData );
+
+	struct termios termdata;
+	tcgetattr( kbdFD, &termdata );
+
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
+	ioctl(kbdFD, KDSKBMODE, K_RAW);
+#endif
+
+	termdata.c_iflag = (IGNPAR | IGNBRK) & (~PARMRK) & (~ISTRIP);
+	termdata.c_oflag = 0;
+	termdata.c_cflag = CREAD | CS8;
+	termdata.c_lflag = 0;
+	termdata.c_cc[VTIME]=0;
+	termdata.c_cc[VMIN]=1;
+	cfsetispeed(&termdata, 9600);
+	cfsetospeed(&termdata, 9600);
+	tcsetattr(kbdFD, TCSANOW, &termdata);
+
+	signal(VTSWITCHSIG, vtSwitchHandler);
+
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
+	struct vt_mode vtMode;
+	ioctl(kbdFD, VT_GETMODE, &vtMode);
+
+	// let us control VT switching
+	vtMode.mode = VT_PROCESS;
+	vtMode.relsig = VTSWITCHSIG;
+	vtMode.acqsig = VTSWITCHSIG;
+	ioctl(kbdFD, VT_SETMODE, &vtMode);
+
+	struct vt_stat vtStat;
+	ioctl(kbdFD, VT_GETSTATE, &vtStat);
+	vtQws = vtStat.v_active;
+#endif
     }
-
-    // save for restore.
-    tcgetattr( kbdFD, &origTermData );
-
-    struct termios termdata;
-    tcgetattr( kbdFD, &termdata );
-
-#if !defined(_OS_FREEBSD_)
-    ioctl(kbdFD, KDSKBMODE, K_RAW);
-#endif
-
-    termdata.c_iflag = (IGNPAR | IGNBRK) & (~PARMRK) & (~ISTRIP);
-    termdata.c_oflag = 0;
-    termdata.c_cflag = CREAD | CS8;
-    termdata.c_lflag = 0;
-    termdata.c_cc[VTIME]=0;
-    termdata.c_cc[VMIN]=1;
-    cfsetispeed(&termdata, 9600);
-    cfsetospeed(&termdata, 9600);
-    tcsetattr(kbdFD, TCSANOW, &termdata);
-
-    signal(VTSWITCHSIG, vtSwitchHandler);
-
-#if !defined(_OS_FREEBSD_)
-    struct vt_mode vtMode;
-    ioctl(kbdFD, VT_GETMODE, &vtMode);
-
-    // let us control VT switching
-    vtMode.mode = VT_PROCESS;
-    vtMode.relsig = VTSWITCHSIG;
-    vtMode.acqsig = VTSWITCHSIG;
-    ioctl(kbdFD, VT_SETMODE, &vtMode);
-
-    struct vt_stat vtStat;
-    ioctl(kbdFD, VT_GETSTATE, &vtStat);
-    vtQws = vtStat.v_active;
-#endif
 }
 
 QWSTtyKeyboardHandler::~QWSTtyKeyboardHandler()
 {
     if (kbdFD >= 0)
     {
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
 	ioctl(kbdFD, KDSKBMODE, K_XLATE);
 #endif
 	tcsetattr(kbdFD, TCSANOW, &origTermData);
@@ -897,9 +1088,9 @@ void QWSTtyKeyboardHandler::readKeyboardData()
 
 /* USB driver */
 
-QWSUsbKeyboardHandler::QWSUsbKeyboardHandler()
+QWSUsbKeyboardHandler::QWSUsbKeyboardHandler(const QString& device)
 {
-    fd = open(getenv("QWS_USB_KEYBOARD"),O_RDONLY, 0);
+    fd = open(device.isEmpty()?"/dev/input/event0":device.latin1(),O_RDONLY, 0);
     if ( fd >= 0 ) {
 	QSocketNotifier *notifier;
 	notifier = new QSocketNotifier( fd, QSocketNotifier::Read, this );
@@ -1257,22 +1448,30 @@ QWSKeyboardHandler *QWSServer::newKeyboardHandler( const QString &spec )
 {
     QWSKeyboardHandler *handler = 0;
 
-    if ( spec == "Buttons" ) {
+    QString device;
+    QString type;
+    int colon=spec.find(':');
+    if ( colon>=0 ) {
+	type = spec.left(colon);
+	device = spec.mid(colon+1);
+    } else {
+	type = spec;
+    }
+
+    if ( type == "Buttons" ) {
 #if defined(QT_QWS_YOPY)
 	handler = new QWSyopyButtonsHandler();
 #elif defined(QT_QWS_CASSIOPEIA)
 	handler = new QWSVr41xxButtonsHandler();
 #endif
-    } else if ( spec == "QVFbKeyboard" ) {
+    } else if ( type == "QVFbKeyboard" ) {
 	handler = new QWSVFbKeyboardHandler();
-    } else if ( spec == "TTY" ) {
-	if(getenv("QWS_USB_KEYBOARD")) {
-	    handler = new QWSUsbKeyboardHandler();
-	} else {
-	    handler = new QWSTtyKeyboardHandler();
-	}
+    } else if ( type == "USB" ) {
+	handler = new QWSUsbKeyboardHandler(device);
+    } else if ( type == "TTY" ) {
+	handler = new QWSTtyKeyboardHandler(device);
     } else {
-	qWarning( "Keyboard type %s unsupported", spec.latin1() );
+	qWarning( "Keyboard type %s:%s unsupported", spec.latin1(), device.latin1() );
     }
 
     return handler;
