@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qlayout.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/src/kernel/qlayout.cpp   2.3.7   edited 2001-12-07 $
 **
 ** Implementation of layout classes
 **
@@ -354,6 +354,11 @@ void QLayoutArray::setSize( int r, int c )
 	for ( int i = cc; i < newC; i++ )
 	    colData[i].init();
     }
+    if ( hfwData && (int)hfwData->size() < r ) {
+	delete hfwData;
+	hfwData = 0;
+	hfw_width = -1;
+    }
     rr = r;
     cc = c;
 }
