@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qmemorymanager_qws.cpp   2.3.2   edited 2001-10-24 $
+** $Id: qt/src/kernel/qmemorymanager_qws.cpp   2.3.7   edited 2003-02-11 $
 **
 ** Implementation of Qt/Embedded memory management routines
 **
@@ -106,7 +106,7 @@ static void getProcessedGlyph(QGlyph& tmp, QGlyph& g, bool smooth)
 	    } else {
 		QRgb colTable[2] = {0, 1};
 		QImage img( tmp.data, width, height,
-			    1, tmp.metrics->linestep, 
+			    1, tmp.metrics->linestep,
 			    colTable, 2, QImage::BigEndian);
 		for(int y=0; y<height; y++) {
 		    unsigned char *dst = newdata+y*pitch;
@@ -116,7 +116,7 @@ static void getProcessedGlyph(QGlyph& tmp, QGlyph& g, bool smooth)
 			 + ( y!=0 && img.pixel(x,y-1)? 85 :0);
 		    }
 		}
-	    
+	
 	
 	    }
 	
@@ -124,8 +124,8 @@ static void getProcessedGlyph(QGlyph& tmp, QGlyph& g, bool smooth)
 	    tmp.data = newdata;
 	    tmp.metrics->height = height;
 	    tmp.metrics->linestep = pitch;
-	}	      
-#endif //QT_NO_QWS_INTERLACE	    
+	}	
+#endif //QT_NO_QWS_INTERLACE	
 	g = tmp;
     }
 }
@@ -209,9 +209,10 @@ public:
 	if ( more ) more->clear();
 	int n = max.unicode()-min.unicode()+1;
 	for (int i=0; i<n; i++) {
-	    delete glyph[i].metrics;
-	    if ( glyph[i].data )
+	    if ( glyph[i].data && !(glyph[i].metrics->flags & QGlyphMetrics::RendererOwnsData) ) {
 		delete [] glyph[i].data;
+	    }
+	    delete glyph[i].metrics;
 	}
     }
 
@@ -544,6 +545,7 @@ public:
 	if ( tree ) {
 	    if ( renderer ) {
 		tree->clear();
+		delete renderer;
 	    }
 	    delete tree;
 	}
@@ -564,12 +566,11 @@ public:
 	    memset(d+m->linestep*(m->height-1),255,m->linestep);
 	    for (int i=1; i<m->height-1; i++) {
 		int j=m->linestep*i;
+		memset(d+j,0,m->linestep);
 		if ( fm.flags & FM_SMOOTH ) {
-		    memset(d+j,31,m->linestep); // semi-transparent
 		    d[j]=255;
 		    d[j+m->linestep-1]=255;
 		} else {
-		    memset(d+j,0,m->linestep);
 		    d[j]=128;
 		    d[j+m->linestep-1]|=128>>((m->width+7)%8);
 		}
@@ -616,14 +617,9 @@ QMemoryManager::QMemoryManager(
 }
 
 
-//### must follow QWS_PACKING_4BYTE in qgfxraster_qws.cpp
-#ifdef __i386__
-static const int memAlign = 64;
-#else
 static const int memAlign = 32;
-#endif
 
-inline static const int calcLineStep( int w, int d, bool vram ) 
+inline static const int calcLineStep( int w, int d, bool vram )
 {
     int align = vram ? qt_screen->pixmapLinestepAlignment() : memAlign;
     return ((w*d+align-1)/align)*align/8;
@@ -667,6 +663,8 @@ QMemoryManager::PixmapID QMemoryManager::newPixmap(int w, int h, int d,
 	next_pixmap_id += 2; // stay even
     }
 
+    memset((void *)data,0,siz);
+
     QMemoryManagerPixmap mmp;
     mmp.data = data;
     mmp.xoffset = xoffset;
@@ -726,15 +724,24 @@ static QString fontKey(const QFontDef& font)
 
 static QFontDef fontFilenameDef( const QString& key )
 {
+    // font name has the form of
+    // (/w)+_(\d)+_(\d)+(i|) the rest of it we don't care about.
     QFontDef result;
     int u0 = key.find('_');
     int u1 = key.find('_',u0+1);
-    int u2 = key.find('_',u1+1);
+
+    // next field will be deleminted by either a _, . or i.
+    // find the first non-diget
+    int u2 = u1+1;
+    while(key[u2].isDigit())
+	u2++;
+
     result.family = key.left(u0);
     result.pointSize = key.mid(u0+1,u1-u0-1).toInt();
     result.weight = key.mid(u1+1,u2-u1-1).toInt();
-    result.italic = key.mid(u2-1,1) == "i";
+    result.italic = key[u2] == 'i';
     // #### ignores _t and _I fields
+
     return result;
 }
 
@@ -771,41 +778,41 @@ QMemoryManager::FontID QMemoryManager::refFont(const QFontDef& font)
 #ifndef QT_NO_DIR
 	    // QPF close-font matching
 	    if ( qdf )
-		bestmatch = QFontManager::cmpFontDef(font, qdf->fontDef());
+	        bestmatch = QFontManager::cmpFontDef(font, qdf->fontDef());
 	    QDir dir(fontDir(),"*.qpf");
-            for (int i=0; i<(int)dir.count(); i++) {
-                QFontDef d = fontFilenameDef(dir[i]);
-                int match = QFontManager::cmpFontDef(font,d);
-                if ( match >= bestmatch ) {
+	    for (int i=0; i<(int)dir.count(); i++) {
+	        QFontDef d = fontFilenameDef(dir[i]);
+	        int match = QFontManager::cmpFontDef(font,d);
+	        if ( match >= bestmatch ) {
 		    QString ff = fontFilename(d);
 		    if ( QFile::exists(ff) ) {
-			 filename = ff;
-			 bestmatch = match;
+		        filename = ff;
+		        bestmatch = match;
 		    }
-                }
+	        }
 	    }
 #endif
 	    if ( filename.isNull() ) {
-		if ( qdf ) {
+	        if ( qdf ) {
 		    mmf->renderer = qdf->load(font);
-		} else {
+	        } else {
 		    // last-ditch attempt to find a font...
 		    QFontDef d = font;
 		    d.family = "helvetica";
 		    filename = fontFilename(d);
 		    if ( !QFile::exists(filename) ) {
-			d.pointSize = 120;
-			filename = fontFilename(d);
-			if ( !QFile::exists(filename) ) {
-			    d.italic = FALSE;
-			    filename = fontFilename(d);
-			    if ( !QFile::exists(filename) ) {
-				d.weight = 50;
-				filename = fontFilename(d);
-			    }
-			}
+		        d.pointSize = 120;
+		        filename = fontFilename(d);
+		        if ( !QFile::exists(filename) ) {
+		            d.italic = FALSE;
+		            filename = fontFilename(d);
+		            if ( !QFile::exists(filename) ) {
+		                d.weight = 50;
+		                filename = fontFilename(d);
+		            }
+		        }
 		    }
-		}
+	        }
 	    }
 	}
 	if ( !mmf->renderer && QFile::exists(filename) ) {
@@ -818,7 +825,11 @@ QMemoryManager::FontID QMemoryManager::refFont(const QFontDef& font)
 	    uchar* data = (uchar*)mmap( 0, // any address
 		    st.st_size, // whole file
                     PROT_READ, // read-only memory
+#if !defined(_OS_SOLARIS_)
                     MAP_FILE | MAP_PRIVATE, // swap-backed map from file
+#else
+		    MAP_PRIVATE,
+#endif
                     f, 0 ); // from offset 0 of f
 	    if ( !data || data == (uchar*)MAP_FAILED )
 		qFatal("Failed to mmap %s",QFile::encodeName(filename).data());
@@ -863,6 +874,9 @@ QMemoryManager::FontID QMemoryManager::refFont(const QFontDef& font)
 void QMemoryManager::derefFont(FontID id)
 {
     QMemoryManagerFont* font = (QMemoryManagerFont*)id;
+    if( font->renderer ) {
+        font->renderer->deref();
+    }
     if ( font->deref() ) {
 	QString key = fontKey(font->def);
 	font_map.remove(key);
@@ -880,7 +894,7 @@ bool QMemoryManager::inFont(FontID id, const QChar& ch) const
 {
     QMemoryManagerFont* font = (QMemoryManagerFont*)id;
     if ( font->renderer )
-	return ch.unicode() < font->renderer->maxchar && font->renderer->inFont(ch);
+	return ch.unicode() <= font->renderer->maxchar && font->renderer->inFont(ch);
     else
 	return font->tree->inFont(ch);
 }
@@ -939,10 +953,7 @@ void QMemoryManager::savePrerenderedFont(FontID id, bool all)
 	    //qDebug("Rendering %s",fontFilename(mmf->def).ascii());
 	    for (int i=0; i<=mmf->renderer->maxchar; i++) {
 		QChar ch((ushort)i);
-		if (
-			/* XXX BUG IN CYBERBIT OR FREETYPE ??? */i!=9660&&/*XXX*/
-
-			mmf->renderer->inFont(ch) ) {
+		if ( mmf->renderer->inFont(ch) ) {
 		    mmf->tree->get(ch,mmf->renderer);
 		    if ( !(j++ & 0x3f)  ) {
 			// XXX keep it from becoming degenerate - should be in QGlyphTree
