@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qnetworkprotocol.cpp   2.3.2   edited 2001-05-28 $
+** $Id: qt/src/kernel/qnetworkprotocol.cpp   2.3.7   edited 2002-01-30 $
 **
 ** Implementation of QNetworkProtocol class
 **
@@ -776,14 +776,12 @@ void QNetworkProtocol::processNextOperation( QNetworkOperation *old )
 
     QNetworkOperation *op = d->operationQueue.head();
 
-    d->opInProgress = 0;
+    d->opInProgress = op;
 
     if ( !checkConnection( op ) ) {
 	if ( op->state() != QNetworkProtocol::StFailed ) {
 	    d->opStartTimer->start( 0, TRUE );
-	    d->opInProgress = op;
 	} else {
-	    d->opInProgress = op;
 	    d->operationQueue.dequeue();
 	    clearOperationQueue();
 	    emit finished( op );
