@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qpainter.cpp   2.3.2   edited 2001-10-13 $
+** $Id: qt/src/kernel/qpainter.cpp   2.3.7   edited 2002-06-07 $
 **
 ** Implementation of QPainter, QPen and QBrush classes
 **
@@ -247,6 +247,8 @@ typedef QStack<QWMatrix> QWMatrixStack;
   </ul>
 
   By far the most useful ones are \c CopyROP and \c XorROP.
+
+  On Qt/Embedded, only \c CopyROP, \c XorROP, and \c NotROP are supported.
 */
 
 /*! \enum Qt::AlignmentFlags
@@ -679,6 +681,7 @@ void QPainter::restore()
 #else
     xlatex = ps->xlatex;
     xlatey = ps->xlatey;
+    setf( VxF, xlatex || xlatey );
 #endif
     if ( ps->rgn != crgn || hardRestore ) {
 #ifdef _WS_QWS_
@@ -1305,7 +1308,7 @@ void QPainter::translate( double dx, double dy )
 #else
     xlatex += (int)dx;
     xlatey += (int)dy;
-    setf( VxF );
+    setf( VxF, xlatex || xlatey );
 #endif
 }
 
@@ -1369,7 +1372,7 @@ void QPainter::resetXForm()
 }
 
 const int TxNone      = 0;			// transformation codes
-const int TxTranslate = 1;			// copy in qptr_xyz.cpp
+const int TxTranslate = 1;			// copy in qpainter_*.cpp
 const int TxScale     = 2;
 const int TxRotShear  = 3;
 
@@ -1454,6 +1457,7 @@ void QPainter::resetXForm()
 {
     xlatex = 0;
     xlatey = 0;
+    clearf( VxF );
 }
 #endif // QT_NO_TRANSFORMATIONS
 
@@ -1530,7 +1534,6 @@ void QPainter::map( int x, int y, int w, int h,
 #endif
 	    break;
     }
-
 #else
     *rx = x + xlatex;
     *ry = y + xlatey;
@@ -1554,7 +1557,6 @@ void QPainter::mapInv( int x, int y, int *rx, int *ry ) const
     double ty = im12()*x + im22()*y+idy();
     *rx = tx >= 0 ? int(tx + 0.5) : int(tx - 0.5);
     *ry = ty >= 0 ? int(ty + 0.5) : int(ty - 0.5);
-
 #else
     *rx = x - xlatex;
     *ry = y - xlatey;
@@ -1583,7 +1585,6 @@ void QPainter::mapInv( int x, int y, int w, int h,
     *ry = ty >= 0 ? int(ty + 0.5) : int(ty - 0.5);
     *rw = tw >= 0 ? int(tw + 0.5) : int(tw - 0.5);
     *rh = th >= 0 ? int(th + 0.5) : int(th - 0.5);
-
 #else
     *rx = x - xlatex;
     *ry = y - xlatey;
@@ -1605,10 +1606,12 @@ QPoint QPainter::xForm( const QPoint &pv ) const
 #ifndef QT_NO_TRANSFORMATIONS
     if ( txop == TxNone )
 	return pv;
-#endif
     int x=pv.x(), y=pv.y();
     map( x, y, &x, &y );
     return QPoint( x, y );
+#else
+    return QPoint( pv.x()+xlatex, pv.y()+xlatey );
+#endif
 }
 
 /*! \overload
@@ -1626,19 +1629,19 @@ QRect QPainter::xForm( const QRect &rv ) const
 #ifndef QT_NO_TRANSFORMATIONS
     if ( txop == TxNone )
 	return rv;
-
     if ( txop == TxRotShear ) {			// rotation/shear
 	QPointArray a( rv );
 	a = xForm( a );
 	return a.boundingRect();
     }
-#endif
-
     // Just translation/scale
     int x, y, w, h;
     rv.rect( &x, &y, &w, &h );
     map( x, y, w, h, &x, &y, &w, &h );
     return QRect( x, y, w, h );
+#else
+    return QRect( rv.x()+xlatex, rv.y()+xlatey, rv.width(), rv.height() );
+#endif
 }
 
 /*! \overload
@@ -1705,7 +1708,7 @@ QPointArray QPainter::xForm( const QPointArray &av, int index,
 }
 
 /*!
-  Returns the point \a pv transformed from device coordinates to model
+  Returns the point \a pd transformed from device coordinates to model
   coordinates.
   \sa xForm(), QWMatrix::map()
 */
@@ -1726,7 +1729,7 @@ QPoint QPainter::xFormDev( const QPoint &pd ) const
 }
 
 /*!
-  Returns the rectangle \a rv transformed from device coordinates to model
+  Returns the rectangle \a rd transformed from device coordinates to model
   coordinates.
 
   If world transformation is enabled and rotation or shearing is used,
@@ -1750,7 +1753,6 @@ QRect QPainter::xFormDev( const QRect &rd ) const
 	return a.boundingRect();
     }
 #endif
-
     // Just translation/scale
     int x, y, w, h;
     rd.rect( &x, &y, &w, &h );
@@ -1758,7 +1760,8 @@ QRect QPainter::xFormDev( const QRect &rd ) const
     return QRect( x, y, w, h );
 }
 
-/*! Returns the point array \a av transformed from device coordinates
+/*!
+  Returns the point array \a ad transformed from device coordinates
   to model coordinates.
 
   \sa xForm(), QWMatrix::map()
