@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qpixmap_qws.cpp   2.3.2   edited 2001-08-14 $
+** $Id: qt/src/kernel/qpixmap_qws.cpp   2.3.7   edited 2003-02-13 $
 **
 ** Implementation of QPixmap class for FB
 **
@@ -38,6 +38,7 @@
 #include "qwmatrix.h"
 #include "qapplication.h"
 #include "qpainter.h"
+#include "qptrdict.h"
 #include "qwsdisplay_qws.h"
 #include "qgfx_qws.h"
 #include <stdlib.h>
@@ -141,6 +142,71 @@ static void build_scale_table( uint **table, uint nBits )
 }
 */
 
+static QList<QShared> *qws_pixmapData = 0;
+static bool qws_trackPixmapData = TRUE;
+
+class QwsPixmap : public QPixmap
+{
+public:
+    QwsPixmap() : QPixmap() {}
+    static void mapPixmaps( bool from );
+    static QPtrDict<QImage> *images;
+};
+
+QPtrDict<QImage> *QwsPixmap::images = 0;
+
+void QwsPixmap::mapPixmaps( bool from )
+{
+    if ( !qws_pixmapData )
+	return;
+    if ( !images )
+	images = new QPtrDict<QImage>;
+    qws_trackPixmapData = FALSE;
+    QListIterator<QShared> it( *qws_pixmapData );
+    while ( it.current() ) {
+	QPixmapData *d = (QPixmapData*)it.current();
+	++it;
+	if ( d->w && d->h ) {
+	    if ( from ) {
+		QwsPixmap p;
+		QPixmapData *tmp = p.data;
+		p.data = d;
+		QImage *img = new QImage(p.convertToImage());
+		images->insert( d, img );
+		p.data = tmp;
+	    } else {
+		QImage *img = images->take( d );
+		if ( img ) {
+		    if ( d->clut )
+			delete [] d->clut;
+		    if ( memorymanager )
+			memorymanager->deletePixmap(d->id);
+		    QwsPixmap p;
+		    p.convertFromImage( *img );
+		    int cnt = d->count-1;
+		    p.data->mask = d->mask;
+		    *d = *p.data;
+		    while (cnt > 0) {
+			d->ref();
+			--cnt;
+		    }
+		    delete img;
+		    delete p.data;
+		    p.data = 0;
+		}
+	    }
+	}
+    }
+    if ( !from )
+	images->clear();
+    qws_trackPixmapData = TRUE;
+}
+
+void qws_mapPixmaps( bool from )
+{
+    QwsPixmap::mapPixmaps( from );
+}
+
 /*****************************************************************************
   QPixmap member functions
  *****************************************************************************/
@@ -150,12 +216,18 @@ void QPixmap::init( int w, int h, int d, bool bitmap, Optimization optim )
     static int serial = 0;
     int dd = defaultDepth();
 
+    if ( !qws_pixmapData )
+	qws_pixmapData = new QList<QShared>;
+
     if ( optim == DefaultOptim )		// use default optimization
 	optim = defOptim;
 
     data = new QPixmapData;
     CHECK_PTR( data );
 
+    if ( qws_trackPixmapData )
+	qws_pixmapData->append( data );
+
     memset( data, 0, sizeof(QPixmapData) );
     data->id=0;
     data->count  = 1;
@@ -213,6 +285,8 @@ void QPixmap::init( int w, int h, int d, bool bitmap, Optimization optim )
 void QPixmap::deref()
 {
     if ( data && data->deref() ) {			// last reference lost
+	if ( qws_trackPixmapData )
+	    qws_pixmapData->removeRef( data );
 	if ( data->mask )
 	    delete data->mask;
 	if ( data->clut )
@@ -249,7 +323,8 @@ QPixmap::QPixmap( int w, int h, const uchar *bits, bool isXbitmap )
     }
 
     if ( qt_screen->isTransformed() ) {
-	QImage img( (uchar *)bits, w, h, 1, 0, 0, QImage::LittleEndian );
+	int bpl = isXbitmap ? (w+7)/8 : ((w+31)/32)*4;
+	QImage img( (uchar *)bits, w, h, 1, bpl, 0, 0, QImage::LittleEndian );
 	convertFromImage( img, MonoOnly );
 	if ( flipped_bits )
 	    delete [] flipped_bits;
@@ -356,6 +431,7 @@ QImage QPixmap::convertToImage() const
     int	d  = depth();
     bool mono = d == 1;
 
+    const QBitmap* msk = mask();
 
     if( d == 15 || d == 16 ) {
 #ifndef QT_NO_QWS_DEPTH_16
@@ -366,9 +442,26 @@ QImage QPixmap::convertToImage() const
 	    register uint *p = (uint *)image.scanLine(y);
 	    ushort  *s = (ushort*)scanLine(y);
 	    uint *end = p + w;
-	    while ( p < end )
-		*p++ = qt_conv16ToRgb( *s++ );
+	    if ( msk ) {
+		uchar* a = msk->scanLine(y);
+		uchar bit = 1; // mask is LittleEndian
+		while ( p < end ) {
+		    uint rgb = qt_conv16ToRgb( *s++ );
+		    if ( !(*a & bit) )
+			rgb &= 0x00ffffff;
+		    *p++ = rgb;
+		    if (!(bit <<= 1)) {
+			++a;
+			bit = 1;
+		    }
+		}
+	    } else {
+		while ( p < end )
+		    *p++ = qt_conv16ToRgb( *s++ );
+	    }
 	}
+	if ( msk )
+	    image.setAlphaBuffer( TRUE );
 #endif
     } else {
 	// We can only create little-endian pixmaps
@@ -445,7 +538,9 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    if ( image.numColors() == 2 ) {
 		QRgb c0 = image.color(0);	// Auto: convert to best
 		QRgb c1 = image.color(1);
-		conv8 = QMIN(c0,c1) != 0 || QMAX(c0,c1) != qRgb(255,255,255);
+		conv8 = QMIN(c0,c1) != qRgb(0,0,0) || QMAX(c0,c1) != qRgb(255,255,255);
+		if ( !conv8 )
+		    force_mono = TRUE;
 	    } else {
 		// eg. 1-color monochrome images (they do exist).
 		conv8 = TRUE;
@@ -490,13 +585,16 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
     	dd=32;
     }
 
+    // for drivers that do not accelerate alpha blt
+    Optimization optim = partialalpha ? NoOptim : defOptim;
+
     QImage rimg = qt_screen->mapToDevice( image );
 
     // detach other references and re-init()
     bool ibm = isQBitmap();
     detach();
     deref();
-    init( w, h, dd, ibm, defOptim );
+    init( w, h, dd, ibm, optim );
 
     QGfx * mygfx=graphicsContext();
     if ( mygfx ) {
@@ -585,19 +683,22 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
     QWMatrix mat( 1, 0, 0, 1, -xmin, -ymin );	// true matrix
     mat = matrix * mat;
 
+    // calculate new width and height
+    QPointArray a( QRect(0,0,ws,hs) );
+    a = mat.map( a );
+    QRect r = a.boundingRect().normalize();
+    w = r.width();
+    h = r.height();
+
     if ( matrix.m12() == 0.0F  && matrix.m21() == 0.0F &&
 	 matrix.m11() >= 0.0F  && matrix.m22() >= 0.0F &&
-	 depth() == defaultDepth() // ### stretchBlt limitation
+	 depth() == defaultDepth() && // ### stretchBlt limitation
+	 !data->mask
 	)
     {
 	if ( mat.m11() == 1.0F && mat.m22() == 1.0F )
 	    return *this;			// identity matrix
 
-	h = qRound( mat.m22()*hs );
-	w = qRound( mat.m11()*ws );
-	h = QABS( h );
-	w = QABS( w );
-
 	if(w==0 || h==0) {
 	    return *this;
 	}
@@ -618,12 +719,6 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 	}
 	return pm;
 
-    } else {					// rotation or shearing
-	QPointArray a( QRect(0,0,ws,hs) );
-	a = mat.map( a );
-	QRect r = a.boundingRect().normalize();
-	w = r.width();
-	h = r.height();
     }
     bool invertible;
     mat = mat.invert( &invertible );		// invert matrix
