@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qpngio.cpp   2.3.2   edited 2001-03-19 $
+** $Id: qt/src/kernel/qpngio.cpp   2.3.7   edited 2003-01-31 $
 **
 ** Implementation of PNG QImage IOHandler
 **
@@ -35,11 +35,10 @@
 **
 **********************************************************************/
 
-#include "qfeatures.h"
+#include "qimage.h"
 
 #ifndef QT_NO_IMAGEIO_PNG
 
-#include "qimage.h"
 #include "qasyncimageio.h"
 #include "qiodevice.h"
 #include "qpngio.h"
@@ -320,6 +319,24 @@ png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)
 
     delete [] row_pointers;
 
+    if ( image.hasAlphaBuffer() ) {
+	// Many PNG files lie (eg. from PhotoShop). Fortunately this loop will
+	// usually be quick to find those that tell the truth.
+	QRgb* c;
+	int n;
+	if (image.depth()==32) {
+	    c = (QRgb*)image.bits();
+	    n = image.bytesPerLine() * image.height() / 4;
+	} else {
+	    c = image.colorTable();
+	    n = image.numColors();
+	}
+	while ( n-- && qAlpha(*c++)==0xff )
+	    ;
+	if ( n<0 ) // LIAR!
+	    image.setAlphaBuffer(FALSE);
+    }
+
     iio->setImage(image);
 
     png_read_end(png_ptr, end_info);
@@ -451,10 +468,8 @@ bool QPNGImageWriter::writeImage(const QImage& image, int quality, int off_x, in
     info_ptr->sig_bit.green = 8;
     info_ptr->sig_bit.blue = 8;
 
-#if 0 // libpng takes care of this.
-    if (image.depth() == 1 && image.bitOrder() == QImage::BigEndian)
+    if (image.depth() == 1 && image.bitOrder() == QImage::LittleEndian)
        png_set_packswap(png_ptr);
-#endif
 
     png_colorp palette = 0;
     png_bytep copy_trans = 0;
@@ -1048,6 +1063,7 @@ void QPNGFormat::end(png_structp png, png_infop info)
 #endif
     QRect r(0,0,image->width(),image->height());
     consumer->frameDone(QPoint(offx,offy),r);
+    consumer->end();
     state = FrameStart;
     unused_data = png->buffer_size; // Since libpng doesn't tell us
 }
