@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qstylesheet.cpp   2.3.2   edited 2001-10-13 $
+** $Id: qt/src/kernel/qstylesheet.cpp   2.3.7   edited 2003-01-07 $
 **
 ** Implementation of the QStyleSheet class
 **
@@ -758,6 +758,8 @@ void QStyleSheetItem::setSelfNesting( bool nesting )
 	<li> \c link
 	- The link color, for example \c link="green"
 	</ul>
+    <li>\c &lt;title&gt;...&lt;/h1&gt;
+	- The title of the page, equivalent to \c &lt;qt \c title="..." \c &gt;.
     <li>\c &lt;h1&gt;...&lt;/h1&gt;
 	- A top-level heading.
     <li>\c &lt;h2&gt;...&lt;/h2&gt;
