@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qtranslator.cpp   2.3.2   edited 2001-10-13 $
+** $Id: qt/src/kernel/qtranslator.cpp   2.3.7   edited 2003-04-02 $
 **
 ** Localization database support.
 **
@@ -930,45 +930,49 @@ QTranslatorMessage QTranslator::findMessage( const char* context,
 	}
     }
 
-    Q_UINT32 h = elfHash( QCString(sourceText) + comment );
-
-    Q_UINT32 rh;
-    Q_UINT32 ro;
-
     size_t numItems = d->offsetArray->size() / ( 2 * sizeof(Q_UINT32) );
     if ( !numItems )
 	return QTranslatorMessage();
 
     if ( systemWordSize == 0 )
 	qSysInfo( &systemWordSize, &systemBigEndian );
-    char *r = (char *) bsearch( &h, d->offsetArray->data(), numItems,
-				2 * sizeof(Q_UINT32),
-				systemBigEndian ? cmp_uint32_big
-				: cmp_uint32_little );
-    if ( r == 0 )
-	return QTranslatorMessage();
 
-    while( r != d->offsetArray->data() && cmp_uint32_big( r - 8, r ) == 0 )
-	r -= 8;
+    for ( ;; ) {
+	Q_UINT32 h = elfHash( QCString(sourceText) + comment );
 
-    QDataStream s( *d->offsetArray, IO_ReadOnly );
-    s.device()->at( r - d->offsetArray->data() );
+	char *r = (char *) bsearch( &h, d->offsetArray->data(), numItems,
+				    2 * sizeof(Q_UINT32),
+				    systemBigEndian ? cmp_uint32_big
+				    : cmp_uint32_little );
+	if ( r != 0 ) {
+	    // go back on equal key
+	    while ( r != d->offsetArray->data() &&
+		    cmp_uint32_big(r - 8, r) == 0 )
+		r -= 8;
 
-    s >> rh >> ro;
+	    QDataStream s( *d->offsetArray, IO_ReadOnly );
+	    s.device()->at( r - d->offsetArray->data() );
 
-    QDataStream ms( *d->messageArray, IO_ReadOnly );
-    while( rh == h ) {
-	ms.device()->at( ro );
-	QTranslatorMessage m( ms );
-	if ( match(m.context(), context)
-		&& match(m.sourceText(), sourceText)
-		&& match(m.comment(), comment) )
-	    return m;
-	if ( s.atEnd() )
-	    return QTranslatorMessage();
-	s >> rh >> ro;
+	    Q_UINT32 rh, ro;
+	    s >> rh >> ro;
+
+	    QDataStream ms( *d->messageArray, IO_ReadOnly );
+	    while ( rh == h ) {
+		ms.device()->at( ro );
+		QTranslatorMessage m( ms );
+		if ( match(m.context(), context)
+			&& match(m.sourceText(), sourceText)
+			&& match(m.comment(), comment) )
+		    return m;
+		if ( s.atEnd() )
+		    break;
+		s >> rh >> ro;
+	    }
+	}
+	if ( !comment[0] )
+	    break;
+	comment = "";
     }
-
     return QTranslatorMessage();
 }
 
