@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qurl.cpp   2.3.2   edited 2001-10-13 $
+** $Id: qt/src/kernel/qurl.cpp   2.3.7   edited 2002-01-31 $
 **
 ** Implementation of QUrl class
 **
@@ -145,7 +145,7 @@ QUrl::QUrl()
 }
 
 /*!
-  Constructs and URL using \a url and parses this string.
+  Constructs an URL using \a url and parses this string.
 
   You can pass strings like "/home/qt", in this case the protocol
   "file" is assumed.
@@ -182,7 +182,7 @@ bool QUrl::isRelativeUrl( const QString &url )
 }
 
 /*!
-  Constructs and URL taking \a url as base (context) and
+  Constructs an URL taking \a url as base (context) and
   \a relUrl as relative URL to \a url. If \a relUrl is not relative,
   \a relUrl is taken as new URL.
 
@@ -225,39 +225,49 @@ QUrl::QUrl( const QUrl& url, const QString& relUrl, bool checkSlash )
     QString rel = relUrl;
     slashify( rel );
 
-    if ( !isRelativeUrl( rel ) ) {
-	if ( rel[ 0 ] == QChar( '/' ) ) {
-	    *this = url;
-	    setEncodedPathAndQuery( rel );
-	} else {
-	    *this = rel;
-	}
-    } else {
+    QUrl urlTmp( url );
+    if ( !urlTmp.isValid() ) {
+	urlTmp.reset();
+    }
+    if ( isRelativeUrl( rel ) ) {
 	if ( rel[ 0 ] == '#' ) {
-	    *this = url;
+	    *this = urlTmp;
 	    rel.remove( 0, 1 );
 	    decode( rel );
 	    setRef( rel );
 	} else if ( rel[ 0 ] == '?' ) {
-	    *this = url;
+	    *this = urlTmp;
 	    rel.remove( 0, 1 );
 	    setQuery( rel );
 	} else {
 	    decode( rel );
-	    *this = url;
-	    if ( !checkSlash || d->cleanPath[ (int)path().length() - 1 ] == '/' ) {
-		QString p = url.path();
-		if ( p.isEmpty() )
-		    p = "/";
-		if ( p.right( 1 ) != "/" )
+	    *this = urlTmp;
+	    if ( checkSlash && d->cleanPath[(int)path().length()-1] != '/' ) {
+		if ( isRelativeUrl( path() ) )
+		    setEncodedPathAndQuery( rel );
+		else
+		    setFileName( rel );
+	    } else {
+		QString p = urlTmp.path();
+		if ( p.isEmpty() ) {
+		    // allow URLs like "file:foo"
+		    if ( !d->host.isEmpty() && !d->user.isEmpty() && !d->pass.isEmpty() )
+			p = "/";
+		}
+		if ( !p.isEmpty() && p.right(1)!="/" )
 		    p += "/";
 		p += rel;
 		d->path = p;
 		d->cleanPathDirty = TRUE;
-	    } else {
-		setFileName( rel );
 	    }
 	}
+    } else {
+	if ( rel[ 0 ] == QChar( '/' ) ) {
+	    *this = urlTmp;
+	    setEncodedPathAndQuery( rel );
+	} else {
+	    *this = rel;
+	}
     }
 }
 
@@ -822,14 +832,17 @@ void QUrl::setFileName( const QString& name )
     while ( fn[ 0 ] == '/' )
 	fn.remove( 0, 1 );
 
-    QString p = path().isEmpty() ?
-		QString( "/" ) : path();
-    if ( !path().isEmpty() ) {
+    QString p;
+    if ( path().isEmpty() ) {
+	p = "/";
+    } else {
+	p = path();
 	int slash = p.findRev( QChar( '/' ) );
 	if ( slash == -1 ) {
 	    p = "/";
-    } else if ( p[ (int)p.length() - 1 ] != '/' )
-	p.truncate( slash + 1 );
+	} else if ( p[ (int)p.length() - 1 ] != '/' ) {
+	    p.truncate( slash + 1 );
+	}
     }
 
     p += fn;
