@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qurloperator.cpp   2.3.2   edited 2001-07-10 $
+** $Id: qt/src/kernel/qurloperator.cpp   2.3.7   edited 2002-01-31 $
 **
 ** Implementation of QUrlOperator class
 **
@@ -724,13 +724,13 @@ bool QUrlOperator::isDir( bool *ok )
 
 const QNetworkOperation *QUrlOperator::get( const QString &location )
 {
-    if ( !checkValid() )
-	return 0;
-
     QUrl u( *this );
     if ( !location.isEmpty() )
 	u = QUrl( *this, location );
 
+    if ( !u.isValid() )
+	return 0;
+
     QNetworkOperation *res = new QNetworkOperation( QNetworkProtocol::OpGet,
 						    u,
 						    QString::null, QString::null );
@@ -793,13 +793,13 @@ const QNetworkOperation *QUrlOperator::get( const QString &location )
 
 const QNetworkOperation *QUrlOperator::put( const QByteArray &data, const QString &location )
 {
-    if ( !checkValid() )
-	return 0;
-
     QUrl u( *this );
     if ( !location.isEmpty() )
 	u = QUrl( *this, location );
 
+    if ( !u.isValid() )
+	return 0;
+
     QNetworkOperation *res = new QNetworkOperation( QNetworkProtocol::OpPut,
 						    u, QString::null, QString::null );
     res->setRawArg( 1, data );
@@ -1056,9 +1056,14 @@ void QUrlOperator::continueCopy( QNetworkOperation *op )
 {
     if ( op->operation() != QNetworkProtocol::OpGet )
 	return;
+    if ( op->state()!=QNetworkProtocol::StDone &&  op->state()!=QNetworkProtocol::StFailed ) {
+	return;
+    }
 
 #ifdef QURLOPERATOR_DEBUG
-    qDebug( "QUrlOperator: continue copy (get finished, put will start)" );
+    if ( op->state() != QNetworkProtocol::StFailed ) {
+	qDebug( "QUrlOperator: continue copy (get finished, put will start)" );
+    }
 #endif
 
     QNetworkOperation *put = d->getOpPutOpMap[ (void*)op ];
@@ -1072,13 +1077,23 @@ void QUrlOperator::continueCopy( QNetworkOperation *op )
     if ( pProt )
 	pProt->setAutoDelete( TRUE );
     if ( put && pProt ) {
-	pProt->addOperation( put );
-	d->currPut = pProt;
+	if ( op->state() != QNetworkProtocol::StFailed ) {
+	    pProt->addOperation( put );
+	    d->currPut = pProt;
+	} else {
+	    deleteOperation( put );
+	}
     }
-    if ( gProt )
+    if ( gProt ) {
 	gProt->setAutoDelete( TRUE );
-    if ( rm && gProt )
- 	gProt->addOperation( rm );
+    }
+    if ( rm && gProt ) {
+	if ( op->state() != QNetworkProtocol::StFailed ) {
+	    gProt->addOperation( rm );
+	} else {
+	    deleteOperation( rm );
+	}
+    }
     disconnect( gProt, SIGNAL( data( const QByteArray &, QNetworkOperation * ) ),
 		this, SLOT( copyGotData( const QByteArray &, QNetworkOperation * ) ) );
     disconnect( gProt, SIGNAL( finished( QNetworkOperation * ) ),
