@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qwidget_qws.cpp   2.3.2   edited 2001-10-24 $
+** $Id: qt/src/kernel/qwidget_qws.cpp   2.3.7   edited 2003-05-22 $
 **
 ** Implementation of QWidget and QWindow classes for FB
 **
@@ -49,6 +49,7 @@
 #include "qgfx_qws.h"
 #include "qwsmanager_qws.h"
 #include "qwsregionmanager_qws.h"
+#include "qinputcontext_p.h"
 
 void qt_insert_sip( QWidget*, int, int );	// defined in qapplication_x11.cpp
 int  qt_sip_count( QWidget* );			// --- "" ---
@@ -63,10 +64,17 @@ extern bool qt_xdnd_enable( QWidget* w, bool on );
 extern void qt_deferred_map_add( QWidget* ); // defined in qapplication_x11.const
 extern void qt_deferred_map_take( QWidget* );// defined in qapplication_x11.const
 extern void qt_leave_modal( QWidget* );
+extern int *qt_last_x;
+extern int *qt_last_y;
+extern WId qt_last_cursor;
+extern QWidget *qt_pressGrab;
+extern QWidget *qt_mouseGrb;
+extern bool qws_overrideCursor;
 
-extern QRect qt_maxWindowRect;
+extern Q_EXPORT QRect qt_maxWindowRect;
+
+extern void qwsUpdateActivePainters();
 
-static QWidget *mouseGrb    = 0;
 static QWidget *keyboardGrb = 0;
 
 static int takeLocalId()
@@ -104,6 +112,30 @@ static void paint_children(QWidget * p,const QRegion& r, bool post)
     }
 }
 
+// Paint the widget and its children
+
+static void paint_heirarchy(QWidget *w, bool post)
+{
+    if ( w && w->testWState(Qt::WState_Visible) ) {
+	if ( post )
+	    QApplication::postEvent(w,new QPaintEvent(w->rect(),
+			!w->testWFlags(QWidget::WRepaintNoErase) ) );
+	else
+	    w->repaint(w->rect(),
+		    !w->testWFlags(QWidget::WRepaintNoErase));
+
+	QObjectList *childObjects=(QObjectList*)w->children();
+	if ( childObjects ) {
+	    QObject * o;
+	    for(o=childObjects->first();o!=0;o=childObjects->next()) {
+		if( o->isWidgetType() ) {
+		    paint_heirarchy((QWidget *)o,post);
+		}
+	    }
+	}
+    }
+}
+
 /*****************************************************************************
   QWidget member functions
  *****************************************************************************/
@@ -116,16 +148,12 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 
     if ( !parentWidget() )
 	setWFlags( WType_TopLevel );		// top-level widget
-    else if ( parentWidget()->isVisible() )
-	parentWidget()->setChildrenAllocatedDirty();
 
     alloc_region_index = -1;
     alloc_region_revision = -1;
     isSettingGeometry = FALSE;
     overlapping_children = -1;
 
-    static int sw = -1, sh = -1;		// screen size
-
     bool topLevel = testWFlags(WType_TopLevel);
     bool popup = testWFlags(WType_Popup);
     bool modal = testWFlags(WType_Modal);
@@ -150,10 +178,8 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 	    setWFlags(WStyle_StaysOnTop);
     }
 
-    if ( sw < 0 ) {				// get the screen size
-	sw = dpy->width();
-	sh = dpy->height();
-    }
+    int sw = dpy->width();
+    int sh = dpy->height();
 
     if ( modal || popup || desktop ) {		// these are top-level, too
 	topLevel = TRUE;
@@ -208,6 +234,9 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 	}
     }
 
+    alloc_region_dirty=FALSE;
+    paintable_region_dirty=FALSE;
+
     if ( !initializeWindow ) {
 	// do no initialization
     } else if ( popup ) {			// popup widget
@@ -245,11 +274,14 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 	setWState( WState_OwnCursor );
     }
 
+    fpos = crect.topLeft();
     if ( topLevel ) {
 	if ( name( 0 ) )
 	    qwsDisplay()->nameRegion( winId(), name(), caption() );
 	else
 	    qwsDisplay()->nameRegion( winId(), "", caption() );
+	createTLExtra();
+	extra->topextra->fsize = crect.size();
 #ifndef QT_NO_QWS_MANAGER
 	if ( testWFlags(WStyle_DialogBorder)
 	     || testWFlags(WStyle_NormalBorder))
@@ -257,7 +289,9 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 	    // get size of wm decoration
 	    QRegion r = QApplication::qwsDecoration().region(this, crect);
 	    QRect br( r.boundingRect() );
-	    crect.moveBy( crect.x() - br.x(), crect.y() - br.y() );
+	    fpos = crect.topLeft();
+	    crect.moveBy( crect.x()-br.x(), crect.y()-br.y() );
+	    extra->topextra->fsize = br.size();
 	    topData()->qwsManager = new QWSManager(this);
 	}
 #endif
@@ -269,14 +303,17 @@ void QWidget::create( WId window, bool initializeWindow, bool /*destroyOldWindow
 	// If we are session managed, inform the window manager about it
 	if ( extra && !extra->mask.isNull() ) {
 	    req_region = extra->mask;
-	    req_region.translate(crect.x(),crect.y()); //###expensive?
+	    req_region.translate(crect.x(),crect.y());
 	    req_region &= crect; //??? this is optional
 	} else {
 	    req_region = crect;
 	}
 	req_region = qt_screen->mapToDevice( req_region, QSize(qt_screen->width(), qt_screen->height()) );
+    } else {
+	if ( extra && extra->topextra )	// already allocated due to reparent?
+	    extra->topextra->fsize = crect.size();
+	updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
     }
-    fpos = crect.topLeft();			// default frame rect
 }
 
 
@@ -295,8 +332,10 @@ void QWidget::destroy( bool destroyWindow, bool destroySubWindows )
 					     destroySubWindows);
 	    }
 	}
-	if ( mouseGrb == this )
-	    releaseMouse();
+	releaseMouse();
+	if ( qt_pressGrab == this )
+	    qt_pressGrab = 0;
+
 	if ( keyboardGrb == this )
 	    releaseKeyboard();
 	if ( testWFlags(WType_Modal) )		// just be sure we leave modal
@@ -313,6 +352,9 @@ void QWidget::destroy( bool destroyWindow, bool destroySubWindows )
 	}
 	setWinId( 0 );
     }
+#ifndef QT_NO_QWS_IM
+    QInputContext::notifyWidgetDeletion(this);
+#endif
 }
 
 
@@ -393,8 +435,8 @@ QPoint QWidget::mapToGlobal( const QPoint &pos ) const
     int	   x=pos.x(), y=pos.y();
     const QWidget* w = this;
     while (w) {
-	x += w->x();
-	y += w->y();
+	x += w->crect.x();
+	y += w->crect.y();
 	w = w->isTopLevel() ? 0 : w->parentWidget();
     }
     return QPoint( x, y );
@@ -405,23 +447,38 @@ QPoint QWidget::mapFromGlobal( const QPoint &pos ) const
     int	   x=pos.x(), y=pos.y();
     const QWidget* w = this;
     while (w) {
-	x -= w->x();
-	y -= w->y();
+	x -= w->crect.x();
+	y -= w->crect.y();
 	w = w->isTopLevel() ? 0 : w->parentWidget();
     }
     return QPoint( x, y );
 }
 
 void QWidget::setMicroFocusHint( int x, int y, int width, int height,
-				 bool /*text*/)
+				 bool text )
 {
     if ( QRect( x, y, width, height ) != microFocusHint() )
 	extraData()->micro_focus_hint.setRect( x, y, width, height );
-
-    //XXX not implemented
-#if 0
+#ifndef QT_NO_QWS_IM
     if ( text ) {
+	QWidget *tlw = topLevelWidget();
+	int winid = tlw->winId();
+	QPoint p( x, y + height );
+	QPoint gp = mapToGlobal( p );
 
+	QRect r = QRect( mapToGlobal( QPoint(0,0) ),
+			 size() );
+			 
+	r.setBottom( tlw->geometry().bottom() );
+
+	//qDebug( "QWidget::setMicroFocusHint %d %d %d %d", r.x(),
+	//	r.y(),  r.width(), r.height() );
+	QInputContext::setMicroFocusWidget( this );
+	
+	qwsDisplay()->setIMInfo( winid, gp.x(), gp.y(), r);
+
+	//send font info,  ###if necessary
+	qwsDisplay()->setInputFont( winid, font() );
     }
 #endif
 }
@@ -490,26 +547,23 @@ void QWidget::setBackgroundEmpty()
 
 void QWidget::setCursor( const QCursor &cursor )
 {
-    if ( 1/*cursor.handle() != arrowCursor.handle()*/
-	 || (extra && extra->curs) ) {
-	createExtra();
-	delete extra->curs;
-	extra->curs = new QCursor(cursor);
-    }
+    createExtra();
+    delete extra->curs;
+    extra->curs = new QCursor(cursor);
     setWState( WState_OwnCursor );
-//    QCursor *oc = QApplication::overrideCursor();
+    if ( isVisible() )
+	updateCursor( paintableRegion() );
 }
 
 void QWidget::unsetCursor()
 {
-    if ( !isTopLevel() ) {
-	if (extra ) {
-	    delete extra->curs;
-	    extra->curs = 0;
-	}
-	clearWState( WState_OwnCursor );
-	// XXX XDefineCursor( x11Display(), winId(), None );
+    if ( extra ) {
+	delete extra->curs;
+	extra->curs = 0;
     }
+    clearWState( WState_OwnCursor );
+    if ( isVisible() )
+	updateCursor( paintableRegion() );
 }
 #endif //QT_NO_CURSOR
 
@@ -553,31 +607,33 @@ void QWidget::setIconText( const QString &iconText )
 
 void QWidget::grabMouse()
 {
-    if ( mouseGrb )
-	mouseGrb->releaseMouse();
+    if ( qt_mouseGrb )
+	qt_mouseGrb->releaseMouse();
 
     qwsDisplay()->grabMouse(this,TRUE);
 
-    mouseGrb = this;
+    qt_mouseGrb = this;
+    qt_pressGrab = 0;
 }
 
 #ifndef QT_NO_CURSOR
 void QWidget::grabMouse( const QCursor &cursor )
 {
-    if ( mouseGrb )
-	mouseGrb->releaseMouse();
+    if ( qt_mouseGrb )
+	qt_mouseGrb->releaseMouse();
 
     qwsDisplay()->grabMouse(this,TRUE);
-    qwsDisplay()->selectCursor(this, (int)cursor.handle());
-    mouseGrb = this;
+    qwsDisplay()->selectCursor(this, (unsigned int)cursor.handle());
+    qt_mouseGrb = this;
+    qt_pressGrab = 0;
 }
 #endif
 
 void QWidget::releaseMouse()
 {
-    if ( mouseGrb == this ) {
+    if ( qt_mouseGrb == this ) {
 	qwsDisplay()->grabMouse(this,FALSE);
-	mouseGrb = 0;
+	qt_mouseGrb = 0;
     }
 }
 
@@ -602,7 +658,9 @@ void QWidget::releaseKeyboard()
 
 QWidget *QWidget::mouseGrabber()
 {
-    return mouseGrb;
+    if ( qt_mouseGrb )
+	return qt_mouseGrb;
+    return qt_pressGrab;
 }
 
 
@@ -666,7 +724,7 @@ void QWidget::repaint( const QRegion& reg, bool erase )
     if ( (widget_state & (WState_Visible|WState_BlockUpdates)) == WState_Visible && isVisible() ) {
 	if ( erase )
 	    this->erase(reg);
-	QPaintEvent e( reg );
+	QPaintEvent e( reg, erase );
 	qt_set_paintevent_clipping( this, reg );
 	QApplication::sendEvent( this, &e );
 	qt_clear_paintevent_clipping();
@@ -676,6 +734,7 @@ void QWidget::repaint( const QRegion& reg, bool erase )
 void QWidget::showWindow()
 {
     if ( testWFlags(WType_TopLevel) ) {
+	updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
 	QRegion r( req_region );
 #ifndef QT_NO_QWS_MANAGER
 	if ( extra && extra->topextra && extra->topextra->qwsManager ) {
@@ -691,19 +750,30 @@ void QWidget::showWindow()
 	qwsDisplay()->setAltitude( winId(),
 	    topData()->fullscreen ? 2 :
 		testWFlags(WStyle_StaysOnTop) ? 1 : 0, TRUE );
-
     } else if ( !topLevelWidget()->in_show ) {
+	updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
 	QWidget *p = parentWidget();
-	p->setChildrenAllocatedDirty();
+	p->setChildrenAllocatedDirty( geometry(), this );
 	p->paintable_region_dirty = TRUE;
 	p->overlapping_children = -1;
-	update();
+	paint_heirarchy( this, TRUE );
     }
 }
 
 
 void QWidget::hideWindow()
 {
+#ifndef QT_NO_QWS_IM
+    QWidget *w = QInputContext::microFocusWidget();
+    while ( w ) {
+	if ( w == this ) {
+	    QInputContext::reset( QInputContext::microFocusWidget() );
+	    break;
+	}
+	w = w->parentWidget();
+    }
+#endif
+
     deactivateWidgetCleanup();
 
     if ( testWFlags(WType_TopLevel) ) {
@@ -712,7 +782,7 @@ void QWidget::hideWindow()
 	qwsDisplay()->requestFocus(winId(),FALSE);
     } else {
 	QWidget *p = parentWidget();
-	p->setChildrenAllocatedDirty();
+	p->setChildrenAllocatedDirty( geometry(), this );
 	p->paintable_region_dirty = TRUE;
 	if ( p->overlapping_children )
 	    p->overlapping_children = -1;
@@ -721,6 +791,7 @@ void QWidget::hideWindow()
 	    paint_children( p,geometry(),TRUE );
 	}
     }
+    updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
 }
 
 void QWidget::showMinimized()
@@ -746,9 +817,19 @@ void QWidget::showMaximized()
     in_show_maximized = 1;
     if ( testWFlags(WType_TopLevel) ) {
 	createTLExtra();
+	if ( topData()->normalGeometry.width() < 0 )
+	    topData()->normalGeometry = geometry();
 #ifndef QT_NO_QWS_MANAGER
 	if ( extra && extra->topextra && extra->topextra->qwsManager ) {
+	    bool resetState = FALSE;
+	    if ( !testWState(WState_Maximized) ) {
+		// The window decoration needs to know if the widget is maximized
+		setWState(WState_Maximized);
+		resetState = TRUE;
+	    }
 	    extra->topextra->qwsManager->maximize();
+	    if ( resetState )
+		clearWState(WState_Maximized);
 	} else
 #endif
 	{
@@ -790,8 +871,8 @@ void QWidget::raise()
 	    setActiveWindow();
 	qwsDisplay()->setAltitude( winId(), 0 );
     } else if ( p ) {
-	p->setChildrenAllocatedDirty();
-	paint_children( p,geometry(),TRUE );
+	p->setChildrenAllocatedDirty( geometry(), this );
+	paint_heirarchy( this, TRUE );
     }
 }
 
@@ -803,7 +884,7 @@ void QWidget::lower()
     if ( isTopLevel() )
 	qwsDisplay()->setAltitude( winId(), -1 );
     else if ( p ) {
-	p->setChildrenAllocatedDirty();
+	p->setChildrenAllocatedDirty( geometry() );
 	paint_children( p,geometry(),TRUE );
     }
 }
@@ -855,10 +936,10 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 	w = 1;
     if ( h < 1 )
 	h = 1;
-    QPoint oldp = pos();
-    QSize  olds = size();
 
-    QRect  r( x, y, w, h );
+    QPoint oldp = geometry().topLeft();
+    QSize olds = size();
+    QRect r( x, y, w, h );
 
     bool isResize = olds != r.size();
 
@@ -875,13 +956,13 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
     if ( !r.contains(geometry()) ) // eg. might not FIT in max window rect
 	clearWState(WState_Maximized);
 
+    QRect oldf = QRect(fpos, (extra && extra->topextra)?extra->topextra->fsize:olds) ;
     setCRect( r );
 
     if ( testWFlags(WType_Desktop) )
 	return;
 
     if ( isTopLevel() ) {
-
 	//### ConfigPending not implemented, do we need it?
 	//setWState( WState_ConfigPending );
 	if ( isMove && ( w==olds.width() && h==olds.height() ) ) {
@@ -904,6 +985,9 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 	if ( isVisible() ) {
 	    if ( isMove && !isResize && alloc_region_index >= 0 ) {
 		qwsDisplay()->moveRegion(winId(), x - oldp.x(), y - oldp.y());
+		setChildrenAllocatedDirty();
+		//NOTE: setCRect has already taken care of fpos
+		//fpos += QPoint( x - oldp.x(), y - oldp.y() );
 	    } else {
 		QRegion rgn( req_region );
 #ifndef QT_NO_QWS_MANAGER
@@ -914,14 +998,33 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 		}
 #endif
 		qwsDisplay()->requestRegion(winId(), rgn);
+		// Adjust frame rect
+		QRect wbr = rgn.boundingRect();
+		wbr = qt_screen->mapFromDevice( wbr, QSize(qt_screen->deviceWidth(), qt_screen->deviceHeight()) );
+		fpos = wbr.topLeft();
+		extra->topextra->fsize = wbr.size();
 	    }
 	}
+#ifndef QT_NO_QWS_IM
+	if ( isMove ) {
+	    QWidget *w = QInputContext::microFocusWidget();
+	    while ( w ) {
+		if ( w == this ) {
+		    // Resend microfocus info when global geometry has changed
+		    QRect r = QInputContext::microFocusWidget()->microFocusHint();
+		    QInputContext::microFocusWidget()->setMicroFocusHint( r.x(),r.y(), r.width(), r.height() );
+		    break;
+		}
+		w = w->parentWidget();
+	    }
+	}
+#endif
     }
 
     if ( isVisible() ) {
 	isSettingGeometry = TRUE;
 	if ( isMove ) {
-	    QMoveEvent e( r.topLeft(), oldp );
+	    QMoveEvent e( fpos, oldf.topLeft() );
 	    QApplication::sendEvent( this, &e );
 #ifndef QT_NO_QWS_MANAGER
 	    if (extra && extra->topextra && extra->topextra->qwsManager)
@@ -945,18 +1048,18 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 */
 	}
 
+	updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
+
 	QWidget *p = parentWidget();
 	if ( !isTopLevel() || isResize ) {
 	    if ( p && !isTopLevel() ) {
 		p->paintable_region_dirty = TRUE;
 		QRegion oldr( QRect(oldp, olds) );
-		QRegion upd( ( QRegion(r) | oldr ) & p->rect() );
+		dirtyChildren.translate( x, y );
 		if ( p->isSettingGeometry ) {
-		    dirtyChildren.translate( x, y );
 		    if ( oldp != r.topLeft() ) {
+			QRegion upd( (QRegion(r) | oldr) & p->rect() );
 			dirtyChildren |= upd;
-			QApplication::postEvent( this, new QPaintEvent(rect(),
-			    !testWFlags(QWidget::WRepaintNoErase)) );
 		    } else {
 			dirtyChildren |= QRegion(r) - oldr;
 			QApplication::postEvent( this, new QPaintEvent(rect(),
@@ -964,6 +1067,9 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 		    }
 		    p->dirtyChildren |= dirtyChildren;
 		} else {
+		    QRegion upd( (QRegion(r) | oldr) & p->rect() );
+		    dirtyChildren |= upd;
+		    QRegion paintRegion = dirtyChildren;
 #define FAST_WIDGET_MOVE
 #ifdef FAST_WIDGET_MOVE
 		    if ( isMove && ( w==olds.width() && h==olds.height() ) ) {
@@ -987,25 +1093,31 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 			    scrollRegion = qt_screen->mapFromDevice( scrollRegion, ds );
 			    QPoint gp = p->mapToGlobal( QPoint(0,0) );
 			    scrollRegion.translate( -gp.x(), -gp.y() );
-			    upd -= scrollRegion;
+			    paintRegion -= scrollRegion;
 			}
 		    }
 #endif
-		    if ( !upd.isEmpty() )
-			QApplication::postEvent( p, new QPaintEvent(upd, TRUE) );
-		    dirtyChildren.translate( x, y );
-		    dirtyChildren |= upd;
-		    p->setChildrenAllocatedDirty();
-		    paint_children( p, dirtyChildren & p->rect(), FALSE );
+		    if ( !oldr.isEmpty() )
+			QApplication::postEvent( p, new QPaintEvent(oldr, TRUE) );
+		    p->setChildrenAllocatedDirty( dirtyChildren, this );
+		    qwsUpdateActivePainters();
+		    paint_children( p, paintRegion, isResize );
 		}
 		p->overlapping_children = -1;
 	    } else {
-		QApplication::postEvent( this, new QPaintEvent(rect(),
-		    !testWFlags(QWidget::WRepaintNoErase)) );
-		if ( oldp != r.topLeft() )
-		    dirtyChildren |= rect();
-		paint_children( this, dirtyChildren, TRUE );
+		if ( oldp != r.topLeft() ) {
+		    qwsUpdateActivePainters();
+		    paint_heirarchy( this, TRUE );
+		} else {
+		    setChildrenAllocatedDirty( dirtyChildren );
+		    qwsUpdateActivePainters();
+		    QApplication::postEvent( this, new QPaintEvent(rect(),
+			!testWFlags(QWidget::WResizeNoErase)) );
+		    paint_children( this, dirtyChildren, TRUE );
+		}
 	    }
+	} else {
+	    qwsUpdateActivePainters();
 	}
 #ifndef QT_NO_QWS_MANAGER
 	if (isResize && extra && extra->topextra && extra->topextra->qwsManager) {
@@ -1018,10 +1130,10 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
     } else {
 	if ( isMove )
 	    QApplication::postEvent( this,
-				     new QMoveEvent( r.topLeft(), oldp ) );
+				     new QMoveEvent( fpos, oldf.topLeft() ) );
 	if ( isResize )
 	    QApplication::postEvent( this,
-				     new QResizeEvent( r.size(), olds ) );
+				     new QResizeEvent(crect.size(), olds) );
     }
 }
 
@@ -1124,6 +1236,8 @@ void QWidget::erase( const QRegion& reg )
     if ( backgroundMode() == NoBackground )
 	return;
 
+    bool unclipped = testWFlags( WPaintUnclipped );
+    clearWFlags( WPaintUnclipped );
     QPainter p(this);
     p.setClipRegion( reg );
     if ( extra && extra->bg_pix ) {
@@ -1141,6 +1255,8 @@ void QWidget::erase( const QRegion& reg )
     } else {
 	p.fillRect(rect(),bg_col);
     }
+    if ( unclipped )
+	setWFlags( WPaintUnclipped );
 }
 
 void QWidget::scroll( int dx, int dy )
@@ -1196,6 +1312,8 @@ void QWidget::scroll( int dx, int dy, const QRect& r )
 
     paintable_region_dirty = TRUE;
 
+    QPoint gpos = mapToGlobal( QPoint() );
+
     if ( !valid_rect && children() ) {	// scroll children
 	setChildrenAllocatedDirty();
 	QPoint pd( dx, dy );
@@ -1208,6 +1326,7 @@ void QWidget::scroll( int dx, int dy, const QRect& r )
 		QPoint oldp = w->pos();
 		QRect  r( w->pos() + pd, w->size() );
 		w->setCRect( r );
+		w->updateRequestedRegion( gpos + w->pos() );
 		QMoveEvent e( r.topLeft(), oldp );
 		QApplication::sendEvent( w, &e );
 	    }
@@ -1215,11 +1334,9 @@ void QWidget::scroll( int dx, int dy, const QRect& r )
 	}
     }
 
-    QPoint p = mapToGlobal( QPoint() );
-
     QSize ds( qt_screen->deviceWidth(), qt_screen->deviceHeight() );
     scrollRegion = qt_screen->mapFromDevice( scrollRegion, ds );
-    scrollRegion.translate( -p.x(), -p.y() );
+    scrollRegion.translate( -gpos.x(), -gpos.y() );
 
     QRegion update( sr );
     update -= scrollRegion;
@@ -1320,38 +1437,53 @@ void QWidget::updateOverlappingChildren() const
 	    ++it;
 	    if ( ch->isWidgetType() && !((QWidget*)ch)->isTopLevel() ) {
 		QWidget *w = (QWidget *)ch;
-		QRegion rr( w->requestedRegion() );
-		QRegion ir = r & rr;
-		if ( !ir.isEmpty() ) {
-		    overlapping_children = 1;
-		    return;
+		if ( w->isVisible() ) {
+		    QRegion rr( w->req_region );
+		    QRegion ir = r & rr;
+		    if ( !ir.isEmpty() ) {
+			overlapping_children = 1;
+			return;
+		    }
+		    r |= rr;
 		}
-		r |= rr;
 	    }
 	}
     }
     overlapping_children = 0;
 }
 
-QRegion QWidget::requestedRegion() const
+void QWidget::updateRequestedRegion( const QPoint &gpos )
 {
-    if ( isTopLevel() ) {
-	return req_region;
-    } else {
-	if ( testWState( WState_Visible ) ) {
-	    QPoint gpos( mapToGlobal(QPoint(0,0)) );
-	    QRegion r( QRect(gpos,crect.size()) );
+    if ( !isTopLevel() ) {
+	if ( !testWState( WState_Visible ) || testWState(WState_ForceHide) ) {
+	    req_region = QRegion();
+	} else {
+	    req_region = QRect(gpos,crect.size());
 	    if ( extra && !extra->mask.isNull() ) {
 		QRegion maskr = extra->mask;
 		maskr.translate( gpos.x(), gpos.y() );
-		r &= maskr;
+		req_region &= maskr;
 	    }
-	    r = qt_screen->mapToDevice( r, QSize(qt_screen->width(), qt_screen->height()) );
-	    return r;
-	} else {
-	    return QRegion();
+	    req_region = qt_screen->mapToDevice( req_region, QSize(qt_screen->width(), qt_screen->height()) );
 	}
     }
+    const QObjectList *c = children();
+    if ( c ) {
+	QObjectListIt it(*c);
+	QObject* ch;
+	while ((ch=it.current())) {
+	    ++it;
+	    if ( ch->isWidgetType() && !((QWidget*)ch)->isTopLevel() ) {
+		QWidget *w = (QWidget *)ch;
+		w->updateRequestedRegion( gpos + w->pos() );
+	    }
+	}
+    }
+}
+
+QRegion QWidget::requestedRegion() const
+{
+    return req_region;
 }
 
 void QWidget::setChildrenAllocatedDirty()
@@ -1369,6 +1501,25 @@ void QWidget::setChildrenAllocatedDirty()
     }
 }
 
+void QWidget::setChildrenAllocatedDirty( const QRegion &r, const QWidget *dirty )
+{
+    const QObjectList *c = children();
+    if ( c ) {
+	QObjectListIt it(*c);
+	QObject* ch;
+	while ((ch=it.current())) {
+	    ++it;
+	    if ( ch->isWidgetType() ) {
+		QWidget *w = (QWidget *)ch;
+		if ( r.boundingRect().intersects( w->geometry() ) )
+		    w->alloc_region_dirty = TRUE;
+		if ( w == dirty )
+		    break;
+	    }
+	}
+    }
+}
+
 // check my hierarchy for dirty allocated regions
 bool QWidget::isAllocatedRegionDirty() const
 {
@@ -1395,7 +1546,7 @@ QRegion QWidget::allocatedRegion() const
 	} else {
 	    if ( isAllocatedRegionDirty() ) {
 		const QObjectList *c;
-		QRegion r( requestedRegion() );
+		QRegion r( req_region );
 		r &= parentWidget()->allocatedRegion();
 		parentWidget()->updateOverlappingChildren();
 		if ( parentWidget()->overlapping_children ) {
@@ -1407,11 +1558,12 @@ QRegion QWidget::allocatedRegion() const
 			while ((ch=it.current())) {
 			    ++it;
 			    if ( ch->isWidgetType() ) {
-				if ( ((QWidget*)ch) == this )
+				QWidget *w = (QWidget*)ch;
+				if ( w == this )
 				    clip=TRUE;
-				else if ( clip && !((QWidget*)ch)->isTopLevel()) {
-				    if ( ((QWidget*)ch)->geometry().intersects( geometry() ) )
-					r -= ((QWidget*)ch)->requestedRegion();
+				else if ( clip && !w->isTopLevel() && w->isVisible() ) {
+				    if ( w->geometry().intersects( geometry() ) )
+					r -= w->req_region;
 				}
 			    }
 			}
@@ -1453,12 +1605,17 @@ QRegion QWidget::paintableRegion() const
 		QObject* ch;
 		while ((ch=it.current())) {
 		    ++it;
-		    if ( ch->isWidgetType() && !((QWidget*)ch)->isTopLevel() ) {
-			paintable_region -= ((QWidget*)ch)->requestedRegion();
+		    if ( ch->isWidgetType() && !((QWidget*)ch)->isTopLevel() && ((QWidget*)ch)->isVisible() ) {
+			paintable_region -= ((QWidget*)ch)->req_region;
 		    }
 		}
 	    }
 	    paintable_region_dirty = FALSE;
+#ifndef QT_NO_CURSOR
+	    // The change in paintable region may have result in the
+	    // cursor now being within my region.
+	    updateCursor( paintable_region );
+#endif
 	}
 	if ( !isTopLevel() )
 	    return paintable_region;
@@ -1475,7 +1632,6 @@ QRegion QWidget::paintableRegion() const
     return QRegion();
 }
 
-
 void QWidget::setMask( const QRegion& region )
 {
     alloc_region_dirty = TRUE;
@@ -1508,6 +1664,7 @@ void QWidget::setMask( const QRegion& region )
 #endif
 	    qwsDisplay()->requestRegion(winId(), rgn);
 	} else {
+	    updateRequestedRegion( mapToGlobal(QPoint(0,0)) );
 	    parentWidget()->paintable_region_dirty = TRUE;
 	    parentWidget()->repaint(geometry());
 	    paint_children( parentWidget(),geometry(),TRUE );
@@ -1537,6 +1694,13 @@ QGfx * QWidget::graphicsContext(bool clip_children) const
 {
     QGfx * qgfx_qws;
     qgfx_qws=qwsDisplay()->screenGfx();
+    updateGraphicsContext( qgfx_qws, clip_children );
+
+    return qgfx_qws;
+}
+
+void QWidget::updateGraphicsContext( QGfx *qgfx_qws, bool clip_children ) const
+{
     QPoint offset=mapToGlobal(QPoint(0,0));
     QRegion r; // empty if not visible
     if ( isVisible() && topLevelWidget()->isVisible() ) {
@@ -1571,8 +1735,6 @@ QGfx * QWidget::graphicsContext(bool clip_children) const
     if ( extra && extra->topextra && extra->topextra->qwsManager )
 	qgfx_qws->setClipRegion(rect());
 #endif
-
-    return qgfx_qws;
 }
 
 unsigned char * QWidget::scanLine(int i) const
@@ -1592,3 +1754,23 @@ bool QWidget::isMaximized() const
 {
     return testWState(WState_Maximized);
 }
+
+void QWidget::resetInputContext()
+{
+#ifndef QT_NO_QWS_IM
+    QInputContext::reset();
+#endif
+}
+
+#ifndef QT_NO_CURSOR
+void QWidget::updateCursor( const QRegion &r ) const
+{
+    if ( qt_last_x && (!QWidget::mouseGrabber() || QWidget::mouseGrabber() == this) &&
+	    qt_last_cursor != (WId)cursor().handle() && !qws_overrideCursor ) {
+	QSize s( qt_screen->width(), qt_screen->height() );
+	QPoint pos = qt_screen->mapToDevice(QPoint(*qt_last_x, *qt_last_y), s);
+	if ( r.contains(pos) )
+	    qwsDisplay()->selectCursor((QWidget*)this, (unsigned int)cursor().handle());
+    }
+}
+#endif
