@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qwindowsystem_qws.cpp   2.3.2   edited 2001-10-16 $
+** $Id: qt/src/kernel/qwindowsystem_qws.cpp   2.3.7   edited 2003-07-09 $
 **
 ** Implementation of Qt/FB central server
 **
@@ -83,6 +83,8 @@
 
 #include <qgfx_qws.h>
 
+extern void qt_setMaxWindowRect(const QRect& r);
+
 QWSServer *qwsServer=0;
 
 class QWSServerData {
@@ -91,6 +93,8 @@ public:
     {
 	screensaverintervals = 0;
 	saver = 0;
+	cursorClient = 0;
+	mouseState = 0;
     }
     ~QWSServerData()
     {
@@ -101,6 +105,8 @@ public:
     QTimer* screensavertimer;
     int* screensaverintervals;
     QWSScreenSaver* saver;
+    QWSClient *cursorClient;
+    int mouseState;
 };
 
 QWSScreenSaver::~QWSScreenSaver()
@@ -118,7 +124,17 @@ typedef void SetAltitudeF( const QWSChangeAltitudeCommand* );
 extern QQueue<QWSCommand> *qt_get_server_queue();
 
 static QRect maxwindow_rect;
-static const char *defaultMouse = "Auto";
+extern Q_EXPORT QRect qt_maxWindowRect;
+static const char *defaultMouse =
+#if defined(QT_QWS_CASSIOPEIA) || defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX) || defined(QT_QWS_YOPY) || defined(QWS_CUSTOMTOUCHPANEL)
+    "TPanel"
+#else
+    "Auto"
+#endif
+#if defined(QT_QWS_CASSIOPEIA)
+    "/dev/tpanel"
+#endif
+    ;
 static const char *defaultKeyboard = "TTY";
 static int qws_keyModifiers = 0;
 
@@ -130,6 +146,16 @@ static int get_object_id()
     static int next=1000;
     return next++;
 }
+#ifndef QT_NO_QWS_IM
+static QWSInputMethod *current_IM = 0;
+static QFont *current_IM_Font = 0;
+static QRect current_IM_Rect;
+static QWSServer::IMState current_IM_State = QWSServer::IMEnd;
+static int current_IM_y=0;
+static QWSWindow* current_IM_win=0;
+static int current_IM_winId=-1;
+#endif
+
 
 
 //#define QWS_REGION_DEBUG
@@ -278,7 +304,7 @@ void QWSWindow::setCaption( const QString &c )
   \internal
   Adds \a r to the window's allocated region.
 */
-void QWSWindow::addAllocation( QWSRegionManager *rm, QRegion r )
+void QWSWindow::addAllocation( QWSRegionManager *rm, const QRegion &r )
 {
     QRegion added = r & requested_region;
     if ( !added.isEmpty() ) {
@@ -293,7 +319,7 @@ void QWSWindow::addAllocation( QWSRegionManager *rm, QRegion r )
   \internal
   Removes \a r from the window's allocated region
 */
-void QWSWindow::removeAllocation(QWSRegionManager *rm, QRegion r)
+void QWSWindow::removeAllocation(QWSRegionManager *rm, const QRegion &r)
 {
     QRegion nr = allocated_region - r;
     if ( nr != allocated_region ) {
@@ -338,6 +364,10 @@ void QWSWindow::operation( QWSWindowOperationEvent::Operation o )
 
 QWSWindow::~QWSWindow()
 {
+#ifndef QT_NO_QWS_IM
+    if ( current_IM_win == this )
+	current_IM_win = 0;
+#endif
     delete d;
 }
 
@@ -348,8 +378,8 @@ QWSWindow::~QWSWindow()
  *
  *********************************************************************/
 //always use frame buffer
-QWSClient::QWSClient( QObject* parent, int socket )
-    : QObject( parent), s(socket), command(0)
+QWSClient::QWSClient( QObject* parent, int socket, int id )
+    : QObject( parent), s(socket), command(0), cid(id)
 {
 #ifndef QT_NO_QWS_MULTIPROCESS
     if ( socket == -1 ) {
@@ -368,11 +398,14 @@ QWSClient::QWSClient( QObject* parent, int socket )
     }
 #else
     isClosed = FALSE;
-#endif //QT_NO_QWS_MULTIPROCESS    
+#endif //QT_NO_QWS_MULTIPROCESS
 }
 
 QWSClient::~QWSClient()
 {
+    while(cursors.begin()!=cursors.end()) {
+        cursors.remove(cursors.begin());
+    }
 }
 
 void QWSClient::setIdentity(const QString& i)
@@ -401,7 +434,7 @@ void QWSClient::errorHandler( int err )
 	s = "Socket Read";
 	break;
     }
-#endif    
+#endif
     //qDebug( "Client %p error %d (%s)", this, err, s.ascii() );
     isClosed = TRUE;
 #ifndef QT_NO_QWS_MULTIPROCESS
@@ -422,8 +455,8 @@ void QWSClient::sendEvent( QWSEvent* event )
     if ( csocket ) {
 	event->write( csocket );
 	csocket->flush();
-    } 
-    else 
+    }
+    else
 #endif
     {
 	qt_client_enqueue( event );
@@ -435,6 +468,7 @@ void QWSClient::sendConnectedEvent( const char *display_spec )
     QWSConnectedEvent event;
     event.simpleData.window = 0;
     event.simpleData.len = strlen( display_spec ) + 1;
+    event.simpleData.clientId = cid;
     char * tmp=(char *)display_spec;
     event.setData( tmp, event.simpleData.len );
     sendEvent( &event );
@@ -825,6 +859,29 @@ static void ignoreSignal( int )
   qwsServer variable.
 */
 
+/*! \enum QWSServer::WindowEvent
+
+  This enum type defines the Qt/Embedded window system events: <ul>
+
+      <li> \c Create - a new window has been created.
+      <li> \c Destroy - a window has been destroyed.
+      <li> \c Hide - a window has been hidden.
+      <li> \c Show - a window has been shown.
+      <li> \c Raise - a window has been raised.
+      <li> \c Lower - a window has been lowered.
+      <li> \c Geometry - the size and/or position of a window has changed.
+      <li> \c Active - a window has become the active window (has keyboard
+      focus).
+</ul>
+*/
+
+/*!
+  \fn void QWSServer::windowEvent(QWSWindow *window, QWSServer::WindowEvent event)
+
+  This signal is emitted when a window system event \a event has been
+  processed for \a window.
+*/
+
 /*!
   \fn const QList<QWSWindow> &QWSServer::clientWindows()
 
@@ -833,6 +890,19 @@ static void ignoreSignal( int )
   use.  The windows are sorted in stacking order from top-most to lowest.
 */
 
+/*!
+ \fn void QWSServer::newChannel(const QString& channel)
+
+  This signal is emitted when QCopChannel \a channel is created.
+*/
+
+/*!
+ \fn void QWSServer::removedChannel(const QString& channel)
+
+ This signal is emitted immediately after the QCopChannel \a channel is destroyed.
+ Note that a channel is not destroyed until all listeners have unregistered.
+*/
+
 /*!
   Construct a QWSServer class.
 
@@ -873,25 +943,24 @@ QWSServer::QWSServer( int flags, QObject *parent, const char *name ) :
     keyboardGrabber = 0;
     keyboardGrabbing = FALSE;
 #ifndef QT_NO_QWS_CURSOR
-    cursorNeedsUpdate=FALSE;
+    haveviscurs = FALSE;
+    cursor = 0;
     nextCursor = 0;
 #endif
 
 #ifndef QT_NO_QWS_MULTIPROCESS
     if ( !geteuid() ) {
-#if !defined(_OS_FREEBSD_)
-	if(mount(0,"/var/shm","shm",0,0)) {
-#else
-	if( mount("shm", "/var/shm", 0, 0) ) {
-#endif
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
+	if( mount(0, "/var/shm", "shm", 0, 0) ) {
 	    /* This just confuses people with 2.2 kernels
 	    if ( errno != EBUSY )
 		qDebug("Failed mounting shm fs on /var/shm: %s",strerror(errno));
 	    */
 	}
+#endif
     }
 #endif
-    
+
     // no selection yet
     selectionOwner.windowid = -1;
     selectionOwner.time.set( -1, -1, -1, -1 );
@@ -918,11 +987,14 @@ QWSServer::QWSServer( int flags, QObject *parent, const char *name ) :
     screenRegion = QRegion( 0, 0, swidth, sheight );
     paintBackground( screenRegion );
 
-    client[-1] = new QWSClient( this, -1 );
+    client[-1] = new QWSClient( this, -1, 0 );
 
 #ifndef QT_NO_SOUND
     soundserver = new QWSSoundServer(this);
 #endif
+#ifndef QT_NO_QWS_IM
+    microF = FALSE;
+#endif
 }
 
 /*!
@@ -933,6 +1005,10 @@ QWSServer::~QWSServer()
     // destroy all clients
     for (ClientIterator it = client.begin(); it != client.end(); ++it )
 	delete *it;
+
+    windows.setAutoDelete(TRUE);
+    windows.clear();
+
     delete bgColor;
     bgColor = 0;
     closeDisplay();
@@ -972,14 +1048,14 @@ void QWSServer::releaseKeyboard(QWSWindow* w)
     }
 }
 
- 
+
 #ifndef QT_NO_QWS_MULTIPROCESS
 /*!
   \internal
 */
 void QWSServer::newConnection( int socket )
 {
-    client[socket] = new QWSClient(this,socket);
+    client[socket] = new QWSClient(this,socket, get_object_id());
     connect( client[socket], SIGNAL(readyRead()),
 	     this, SLOT(doClient()) );
     connect( client[socket], SIGNAL(connectionClosed()),
@@ -1014,8 +1090,8 @@ void QWSServer::clientClosed()
 #ifndef QT_NO_COP
     // Enfore unsubscription from all channels.
     QCopChannel::detach( cl );
-#endif    
-    
+#endif
+
     QRegion exposed;
     {
 	// Shut down all windows for this client
@@ -1050,6 +1126,10 @@ void QWSServer::clientClosed()
 	}
     }
     client.remove( cl->socket() );
+    if ( cl == d->cursorClient )
+	d->cursorClient = 0;
+    if ( qt_screen->clearCacheFunc )
+	(qt_screen->clearCacheFunc)( qt_screen, cl->clientId() );  // remove any remaining cache entries.
     delete cl;
     exposeRegion( exposed );
     syncRegions();
@@ -1081,13 +1161,13 @@ QWSCommand* QWSClient::readMoreCommand()
 
 	// Not finished reading a whole command.
 	return 0;
-    } 
-    else 
-#endif    
+    }
+    else
+#endif
     {
 	return qt_get_server_queue()->dequeue();
     }
-    
+
 }
 
 
@@ -1113,6 +1193,18 @@ void QWSServer::doClient()
     QWSClient* client = (QWSClient*)sender();
     doClient( client );
     active = FALSE;
+
+#ifndef QT_NO_QWS_IM
+    //### Avoid reentrancy problems when the IM tries to
+    //do top-level widget operations (eg. move()) as a response to 
+    // setMicroFocus()
+    //### I hope we can find a cleaner way to do this.
+    if ( microF && current_IM ) {
+      current_IM->setMicroFocus( microX, microY );
+      current_IM_y = microY;
+      microF = FALSE;
+    }
+#endif
 }
 #endif
 
@@ -1201,14 +1293,42 @@ void QWSServer::doClient( QWSClient *client )
 	case QWSCommand::QCopSend:
 	    invokeQCopSend( (QWSQCopSendCommand*)cs->command, cs->client );
 	    break;
+#endif
+#ifndef QT_NO_QWS_IM
+	case QWSCommand::ResetIM:
+	    resetInputMethod();
+	    break;
+	case QWSCommand::SetIMFont:
+	    invokeSetIMFont((QWSSetIMFontCommand*)cs->command, cs->client );
+	    break;
+	case QWSCommand::SetIMInfo:
+	    //invokeSetMicroFocus( (QWSSetMicroFocusCommand*)cs->command, cs->client );
+	    {
+		QWSSetIMInfoCommand *cmd = (QWSSetIMInfoCommand*)cs->command;
+		microF = TRUE;
+		microX = cmd->simpleData.x;
+		microY = cmd->simpleData.y;
+		current_IM_Rect = QRect( cmd->simpleData.x1, 
+					 cmd->simpleData.y1,
+					 cmd->simpleData.w,
+					 cmd->simpleData.h );
+		//###reset ????
+		current_IM_winId = cmd->simpleData.windowid;
+	    }
+	    break;
+	case QWSCommand::IMMouse:
+	    {
+		if ( current_IM ) {
+		    QWSIMMouseCommand *cmd = (QWSIMMouseCommand *) cs->command;
+		    current_IM->mouseHandler( cmd->simpleData.index,
+					      cmd->simpleData.state );
+		}
+	    }
+	    break;
 #endif
 	}
 	delete cs->command;
 	delete cs;
-#ifndef QT_NO_QWS_CURSOR
-	if (cursorNeedsUpdate)
-	    showCursor();
-#endif
     }
 }
 
@@ -1304,12 +1424,14 @@ void QWSServer::setDefaultKeyboard( const char *k )
 */
 void QWSServer::sendMouseEvent(const QPoint& pos, int state)
 {
+    const int btnMask = LeftButton | RightButton | MidButton;
     qwsServer->showCursor();
 
     if ( state )
 	qwsServer->screenSaverWake();
 
     mousePosition = pos;
+    qwsServer->d->mouseState = state;
 
     QWSMouseEvent event;
 
@@ -1330,7 +1452,7 @@ void QWSServer::sendMouseEvent(const QPoint& pos, int state)
     }
 #endif
 
-    if ( state && !qwsServer->mouseGrabbing ) {
+    if ( (state&btnMask) && !qwsServer->mouseGrabbing ) {
 	qwsServer->mouseGrabber = win;
     }
 
@@ -1339,10 +1461,42 @@ void QWSServer::sendMouseEvent(const QPoint& pos, int state)
     event.simpleData.state=state | qws_keyModifiers;
     event.simpleData.time=qwsServer->timer.elapsed();
 
-    for (ClientIterator it = qwsServer->client.begin(); it != qwsServer->client.end(); ++it )
-	(*it)->sendEvent( &event );
 
-    if ( !state && !qwsServer->mouseGrabbing )
+    QWSClient *serverClient = qwsServer->client[-1];
+    QWSClient *winClient = win ? win->client() : 0;
+
+#ifndef QT_NO_QWS_IM
+    static int oldstate = 0;
+    bool isPress = state > oldstate;
+    oldstate = state;
+    if ( isPress && current_IM && current_IM_winId != -1 ) {
+	QWSWindow *kbw = keyboardGrabber ? keyboardGrabber :
+			 qwsServer->focusw;
+
+	
+	//checking for virtual keyboards ### could be better
+	QWidget *target = winClient == serverClient ? 
+			  QApplication::widgetAt( pos ) : 0;
+	if ( kbw != win && (!target || !target->testWFlags( WStyle_Tool ) || target->focusPolicy() != QWidget::NoFocus) )
+	    resetInputMethod();
+    }
+#endif
+
+
+    if ( serverClient )
+       serverClient->sendEvent( &event );
+    if ( winClient && winClient != serverClient )
+       winClient->sendEvent( &event );
+
+    // Make sure that if we leave a window, that window gets one last mouse
+    // event so that it knows the mouse has left.
+    QWSClient *oldClient = qwsServer->d->cursorClient;
+    if ( oldClient && oldClient != winClient && oldClient != serverClient )
+	oldClient->sendEvent( &event );
+
+    qwsServer->d->cursorClient = winClient;
+
+    if ( !(state&btnMask) && !qwsServer->mouseGrabbing )
 	qwsServer->releaseMouse(qwsServer->mouseGrabber);
 }
 
@@ -1377,7 +1531,7 @@ QList<QWSInternalWindowInfo> * QWSServer::windowList()
 	window=qwsServer->windows.next()) {
 	QWSInternalWindowInfo * qwi=new QWSInternalWindowInfo();
 	qwi->winid=window->winId();
-	qwi->clientid=(unsigned int)window->client();
+	qwi->clientid=window->client()->clientId();
 #ifndef QT_NO_QWS_PROPERTIES
 	char * name;
 	int len;
@@ -1411,7 +1565,7 @@ void QWSServer::sendQCopEvent( QWSClient *c, const QCString &ch,
 			       bool response )
 {
     ASSERT( c );
-    
+
     QWSQCopMessageEvent event;
     event.simpleData.is_response = response;
     event.simpleData.lchannel = ch.length();
@@ -1428,7 +1582,7 @@ void QWSServer::sendQCopEvent( QWSClient *c, const QCString &ch,
     memcpy( d, msg.data(), event.simpleData.lmessage );
     d += event.simpleData.lmessage;
     memcpy( d, data.data(), event.simpleData.ldata );
-    
+
     event.setData( raw.data(), l );
 
     c->sendEvent( &event );
@@ -1449,6 +1603,7 @@ QWSWindow *QWSServer::windowAt( const QPoint& pos )
     return 0;
 }
 
+#ifndef QT_NO_QWS_KEYBOARD
 static int keyUnicode(int keycode)
 {
     const QWSServer::KeyMap *km = QWSServer::keyMap();
@@ -1460,7 +1615,7 @@ static int keyUnicode(int keycode)
     }
     return 0xffff;
 }
-
+#endif
 /*!
   Send a key event. You can use this to send key events generated by
   "virtual keyboards".
@@ -1468,20 +1623,45 @@ static int keyUnicode(int keycode)
 void QWSServer::sendKeyEvent(int unicode, int keycode, int modifiers, bool isPress,
   bool autoRepeat)
 {
+    if ( !qwsServer ) {
+#if defined(CHECK_STATE)
+    qWarning( "QWSServer::sendKeyEvent called from non-server process." );
+#endif
+	return;
+    }
+    qws_keyModifiers = modifiers;
+
     if ( isPress ) {
 	if ( keycode != Key_F34 && keycode != Key_F35 )
 	    qwsServer->screenSaverWake();
     }
 
-    qws_keyModifiers = modifiers;
+
+#ifndef QT_NO_QWS_IM
+
+    if ( !current_IM || !current_IM->filter( unicode, keycode, modifiers, 
+					     isPress, autoRepeat ) 
+			&& current_IM_State == IMEnd )
+	sendKeyEventUnfiltered( unicode, keycode, modifiers, 
+				isPress, autoRepeat);
+}
+
+void QWSServer::sendKeyEventUnfiltered(int unicode, int keycode, int modifiers, bool isPress,
+  bool autoRepeat)
+{
+#endif
 
     QWSKeyEvent event;
-    QWSWindow *win = keyboardGrabber ? keyboardGrabber : 
+    QWSWindow *win = keyboardGrabber ? keyboardGrabber :
 	qwsServer->focusw;
 
     event.simpleData.window = win ? win->winId() : 0;
 
-    event.simpleData.unicode = unicode < 0 ? keyUnicode(keycode) : unicode;
+    event.simpleData.unicode = 
+#ifndef QT_NO_QWS_KEYBOARD
+	unicode < 0 ? keyUnicode(keycode) : 
+#endif
+	unicode;
     event.simpleData.keycode = keycode;
     event.simpleData.modifiers = modifiers;
     event.simpleData.is_press = isPress;
@@ -1491,6 +1671,149 @@ void QWSServer::sendKeyEvent(int unicode, int keycode, int modifiers, bool isPre
 	(*it)->sendEvent(&event);
     }
 }
+
+/*!
+  \internal
+  Disconnects the display.
+
+  \sa endDisplayReconfigure()
+*/
+void QWSServer::beginDisplayReconfigure()
+{
+    qwsServer->enablePainting( FALSE );
+    qwsServer->hideCursor();
+    QWSDisplay::grab( TRUE );
+    qt_screen->disconnect();
+}
+
+/*!
+  \internal
+  Reconnects the display and reinitializes the server.
+
+  \sa beginDisplayReconfigure()
+*/
+void QWSServer::endDisplayReconfigure()
+{
+    delete qwsServer->gfx;
+    qt_screen->connect( QString::null );
+    qwsServer->swidth = qt_screen->deviceWidth();
+    qwsServer->sheight = qt_screen->deviceHeight();
+    qwsServer->screenRegion = QRegion( 0, 0, qwsServer->swidth, qwsServer->sheight );
+    qwsServer->gfx = qt_screen->screenGfx();
+    QWSDisplay::ungrab();
+    qwsServer->showCursor();
+    qt_setMaxWindowRect( QRect(0, 0, qt_screen->deviceWidth(), qt_screen->deviceHeight()) );
+    QSize olds = qApp->desktop()->size();
+    qApp->desktop()->resize( qt_screen->width(), qt_screen->height() );
+    qApp->postEvent( qApp->desktop(), new QResizeEvent( qApp->desktop()->size(), olds ) );
+    qwsServer->enablePainting( TRUE );
+    qwsServer->refresh();
+    qDebug( "Desktop size: %dx%d", qApp->desktop()->width(), qApp->desktop()->height() );
+}
+
+void QWSServer::resetGfx()
+{
+    qwsServer->hideCursor();
+    qwsServer->showCursor();
+    delete qwsServer->gfx;
+    qwsServer->gfx = qt_screen->screenGfx();
+}
+
+#ifndef QT_NO_QWS_CURSOR
+/*!
+  Shows the cursor if \a visible is TRUE, otherwise hides the cursor.
+
+  \sa isCursorVisible()
+*/
+void QWSServer::setCursorVisible( bool visible )
+{
+    if ( qwsServer && qwsServer->haveviscurs != visible) {
+	QWSCursor* c = qwsServer->cursor;
+	qwsServer->setCursor(QWSCursor::systemCursor(BlankCursor));
+	qwsServer->haveviscurs = visible;
+	qwsServer->setCursor(c);
+    }
+}
+
+/*!
+  Returns TRUE is the cursor is visible, or FALSE if it is hidden.
+
+  \sa setCursorVisible()
+*/
+bool QWSServer::isCursorVisible()
+{
+    return qwsServer ? qwsServer->haveviscurs : TRUE;
+}
+#endif
+
+#ifndef QT_NO_QWS_IM
+
+void QWSServer::sendIMEvent( IMState state, const QString& txt, int cpos, int selLen )
+{
+    QWSIMEvent event;
+
+    QWSWindow *win = keyboardGrabber ? keyboardGrabber :
+	qwsServer->focusw;
+
+    if ( current_IM_State == IMCompose && current_IM_win )
+	win = current_IM_win;
+
+    event.simpleData.window = win ? win->winId() : 0;
+    event.simpleData.type = state;
+    event.simpleData.cpos = cpos;
+    event.simpleData.selLen = selLen;
+    event.simpleData.textLen = txt.length();
+
+    char * tmp=(char *)txt.unicode();
+    event.setData( tmp, event.simpleData.textLen*2 );
+
+//    qDebug("sendImEvent:: cmd %d, win %p cur_st %d", state, win, current_IM_State);
+    QWSClient *serverClient = qwsServer->client[-1];
+    if ( serverClient )
+       serverClient->sendEvent( &event );
+    if ( win && win->client() && win->client() != serverClient )
+       win->client()->sendEvent( &event );
+
+    current_IM_State = state;
+    current_IM_win = win;
+}
+
+void QWSServer::requestMarkedText()
+{
+    if ( !qwsServer )
+	return;
+    qwsServer->manager()->addProperty( 0, QT_QWS_PROPERTY_MARKEDTEXT );
+
+    QWSIMEvent event;
+    QWSWindow *win = keyboardGrabber ? keyboardGrabber :
+	qwsServer->focusw;
+
+    event.simpleData.window = win ? win->winId() : 0;
+    event.simpleData.type = IMMarkedText;
+    event.simpleData.cpos = 0;
+    event.simpleData.selLen = 0;
+    event.simpleData.textLen = 0;
+    event.setData( 0, 0 );
+
+    QWSClient *serverClient = qwsServer->client[-1];
+    if ( serverClient )
+       serverClient->sendEvent( &event );
+    if ( win && win->client() && win->client() != serverClient )
+       win->client()->sendEvent( &event );
+    
+}
+
+
+
+void QWSServer::setCurrentInputMethod( QWSInputMethod *im )
+{
+    if ( current_IM_State != IMEnd && im != current_IM && qwsServer )
+	qwsServer->sendIMEvent( IMEnd, "", -1, 0 );
+    current_IM = im;
+}
+
+#endif //QT_NO_QWS_IM
+
 #ifndef QT_NO_QWS_PROPERTIES
 /*!
   \internal
@@ -1501,7 +1824,7 @@ void QWSServer::sendPropertyNotifyEvent( int property, int state )
 	( *it )->sendPropertyNotifyEvent( property, state );
 }
 #endif
-void QWSServer::invokeIdentify( QWSIdentifyCommand *cmd, QWSClient *client )
+void QWSServer::invokeIdentify( const QWSIdentifyCommand *cmd, QWSClient *client )
 {
     client->setIdentity(cmd->id);
 }
@@ -1531,7 +1854,8 @@ void QWSServer::invokeRegion( QWSRegionCommand *cmd, QWSClient *client )
 
     QWSWindow* changingw = findWindow(cmd->simpleData.windowid, 0);
     if ( !changingw ) {
-	qWarning("Invalue window handle %08x",cmd->simpleData.windowid);
+	qWarning("Invalid window handle %08x",cmd->simpleData.windowid);
+	client->sendRegionModifyEvent( cmd->simpleData.windowid, QRegion(), TRUE );
 	return;
     }
     if ( !changingw->forClient(client) ) {
@@ -1555,6 +1879,7 @@ void QWSServer::invokeRegion( QWSRegionCommand *cmd, QWSClient *client )
 	emit windowEvent( changingw, Hide );
     if ( focusw == changingw && region.isEmpty() )
 	setFocus(changingw,FALSE);
+    updateClientCursorPos();
 }
 
 void QWSServer::invokeRegionMove( const QWSRegionMoveCommand *cmd, QWSClient *client )
@@ -1562,6 +1887,7 @@ void QWSServer::invokeRegionMove( const QWSRegionMoveCommand *cmd, QWSClient *cl
     QWSWindow* changingw = findWindow(cmd->simpleData.windowid, 0);
     if ( !changingw ) {
 	qWarning("invokeRegionMove: Invalid window handle %d",cmd->simpleData.windowid);
+	client->sendRegionModifyEvent( cmd->simpleData.windowid, QRegion(), TRUE );
 	return;
     }
     if ( !changingw->forClient(client) ) {
@@ -1633,11 +1959,26 @@ void QWSServer::invokeSetFocus( const QWSRequestFocusCommand *cmd, QWSClient *cl
 
 void QWSServer::setFocus( QWSWindow* changingw, bool gain )
 {
+#ifndef QT_NO_QWS_IM
+    /*
+      This is the logic:
+      QWSWindow *loser = 0;
+      if ( gain && focusw != changingw )
+         loser = focusw;
+      else if ( !gain && focusw == changingw )
+	 loser = focusw;
+      But these five lines can be reduced to one:
+    */
+    QWSWindow *loser =  (!gain == (focusw==changingw)) ? focusw : 0;
+    if ( loser && loser->winId() == current_IM_winId )
+	resetInputMethod();
+#endif
     if ( gain ) {
 	if ( focusw != changingw ) {
 	    if ( focusw ) focusw->focus(0);
 	    focusw = changingw;
 	    focusw->focus(1);
+	    emit windowEvent( focusw, Active );
 	}
     } else if ( focusw == changingw ) {
 	changingw->focus(0);
@@ -1656,6 +1997,7 @@ void QWSServer::setFocus( QWSWindow* changingw, bool gain )
 	focusw = bestw;
 	if ( focusw )
 	    focusw->focus(1);
+	emit windowEvent( focusw, Active );
     }
 }
 
@@ -1677,6 +2019,7 @@ void QWSServer::invokeSetAltitude( const QWSChangeAltitudeCommand *cmd,
     QWSWindow* changingw = findWindow(winId, 0);
     if ( !changingw ) {
 	qWarning("invokeSetAltitude: Invalid window handle %d", winId);
+	client->sendRegionModifyEvent( winId, QRegion(), TRUE );
 	return;
     }
 
@@ -1690,11 +2033,11 @@ void QWSServer::invokeSetAltitude( const QWSChangeAltitudeCommand *cmd,
 	lowerWindow( changingw, alt );
     else
 	raiseWindow( changingw, alt );
-    
+
     if ( !changingw->forClient(client) ) {
 	refresh();
     }
-    
+
 }
 #ifndef QT_NO_QWS_PROPERTIES
 void QWSServer::invokeAddProperty( QWSAddPropertyCommand *cmd )
@@ -1711,7 +2054,13 @@ void QWSServer::invokeSetProperty( QWSSetPropertyCommand *cmd )
 				    cmd->rawLen ) ) {
 	sendPropertyNotifyEvent( cmd->simpleData.property,
 				 QWSPropertyNotifyEvent::PropertyNewValue );
-   }
+#ifndef QT_NO_QWS_IM
+	if ( cmd->simpleData.property == QT_QWS_PROPERTY_MARKEDTEXT ) {
+	    QString s( (const QChar*)cmd->data, cmd->rawLen/2 );
+	    emit markedText( s );
+	}
+#endif	     
+    }
 }
 
 void QWSServer::invokeRemoveProperty( QWSRemovePropertyCommand *cmd )
@@ -1806,29 +2155,31 @@ void QWSServer::invokeSelectCursor( QWSSelectCursorCommand *cmd, QWSClient *clie
 	curs = QWSCursor::systemCursor(ArrowCursor);
     }
 
+    QWSWindow* win = findWindow(cmd->simpleData.windowid, 0);
     if (mouseGrabber) {
 	// If the mouse is being grabbed, we don't want just anyone to
         // be able to change the cursor.  We do want the cursor to be set
 	// correctly once mouse grabbing is stopped though.
-	QWSWindow* win = findWindow(cmd->simpleData.windowid, 0);
 	if (win != mouseGrabber)
 	    nextCursor = curs;
 	else
 	    setCursor(curs);
-    }
-    else
+    } else if (win && win->allocation().contains(mousePosition) ) {
+	// A non-grabbing window can only set the cursor shape if the
+	// cursor is within its allocated region.
 	setCursor(curs);
-
-    cursorNeedsUpdate = true;
+    }
 }
 #endif
 
 void QWSServer::invokeGrabMouse( QWSGrabMouseCommand *cmd, QWSClient *client )
 {
     QWSWindow* win = findWindow(cmd->simpleData.windowid, 0);
+    if ( !win )
+	return;
 
     if ( cmd->simpleData.grab ) {
-	if ( !mouseGrabber || ( mouseGrabber->client() == client ) ) {
+	if ( !mouseGrabber || mouseGrabber->client() == client ) {
 	    mouseGrabbing = TRUE;
 	    mouseGrabber = win;
 	}
@@ -1840,6 +2191,8 @@ void QWSServer::invokeGrabMouse( QWSGrabMouseCommand *cmd, QWSClient *client )
 void QWSServer::invokeGrabKeyboard( QWSGrabKeyboardCommand *cmd, QWSClient *client )
 {
     QWSWindow* win = findWindow(cmd->simpleData.windowid, 0);
+    if ( !win )
+	return;
 
     if ( cmd->simpleData.grab ) {
 	if ( !keyboardGrabber || ( keyboardGrabber->client() == client ) ) {
@@ -1862,8 +2215,9 @@ void QWSServer::invokePlaySound( QWSPlaySoundCommand *cmd, QWSClient * )
 void QWSServer::invokeRegisterChannel( QWSQCopRegisterChannelCommand *cmd,
 				       QWSClient *client )
 {
-    QCopChannel::registerChannel( cmd->channel, client );
-    emit newChannel( cmd->channel );
+  // QCopChannel will force us to emit the newChannel signal if this channel
+  // didn't already exist.
+  QCopChannel::registerChannel( cmd->channel, client );
 }
 
 void QWSServer::invokeQCopSend( QWSQCopSendCommand *cmd, QWSClient *client )
@@ -1872,11 +2226,66 @@ void QWSServer::invokeQCopSend( QWSQCopSendCommand *cmd, QWSClient *client )
 }
 
 #endif
+
+
+#ifndef QT_NO_QWS_IM
+void QWSServer::invokeSetIMInfo( const QWSSetIMInfoCommand *cmd,
+                                  QWSClient * )
+{
+    current_IM_y = cmd->simpleData.y;
+
+    current_IM_Rect = QRect( cmd->simpleData.x1, 
+			     cmd->simpleData.y1,
+			     cmd->simpleData.w,
+			     cmd->simpleData.h );
+
+    //??? reset if  windowid != current_IM_winId  ???
+    
+    current_IM_winId = cmd->simpleData.windowid;
+    
+    if ( current_IM )
+      current_IM->setMicroFocus( cmd->simpleData.x, cmd->simpleData.y );
+}
+
+void QWSServer::resetInputMethod()
+{
+    if ( current_IM_State == IMEnd )
+	current_IM_State = IMInternal;
+    
+    if ( current_IM && qwsServer ) {
+      current_IM->reset();
+    }
+    if ( current_IM_State != IMEnd ) // IM didn't send IMEnd
+	qwsServer->sendIMEvent( IMEnd, QString::null, -1, -1 );
+    current_IM_winId = -1;
+}
+
+
+
+void QWSServer::invokeSetIMFont( const QWSSetIMFontCommand *cmd,
+				 QWSClient * )
+{
+    if ( !current_IM_Font )
+	current_IM_Font = new QFont( cmd->font );
+    else
+	*current_IM_Font = cmd->font;
+
+}
+
+#endif
+
+
+
 QWSWindow* QWSServer::newWindow(int id, QWSClient* client)
 {
     // Make a new window, put it on top.
     QWSWindow* w = new QWSWindow(id,client);
     int idx = rgnMan->add( id, QRegion() );
+    if ( idx < 0 ) {
+	qWarning( "Exceeded maximum top-level windows" );
+	disconnectClient( client );
+	return 0;
+    }
     w->setAllocationIndex( idx );
     // insert after "stays on top" windows
     QWSWindow *win = windows.first();
@@ -1923,7 +2332,9 @@ void QWSServer::raiseWindow( QWSWindow *changingw, int alt )
 	    QWSWindow *next = windows.next();
 
 	    // Resize full-screen windows to the maximum window size.
-	    setWindowRegion( win, maxwindow_rect );
+	    QRect r( qt_screen->mapToDevice(qt_maxWindowRect,
+	            QSize(qt_screen->width(), qt_screen->height())) );
+	    setWindowRegion( win, r );
 
 	    // Set the position in the list to where we were before
 	    QWSWindow *w = windows.first();
@@ -1951,7 +2362,7 @@ void QWSServer::raiseWindow( QWSWindow *changingw, int alt )
     win = windows.first();
 
     // Update the altitude for non-fullscreen windows
-    if ( changingw->d->altitude != 2 ) 
+    if ( changingw->d->altitude != 2 )
 	changingw->d->altitude = alt;
 
     if ( changingw == windows.first() ) {
@@ -2059,6 +2470,7 @@ void QWSServer::moveWindowRegion( QWSWindow *changingw, int dx, int dy )
 */
     QWSDisplay::grab( TRUE );
     QRegion exposed = setWindowRegion( changingw, newRegion );
+
 /*
     for ( int i = 0; i < changingw->allocation().rects().count(); i++ )
 	qDebug( "newAlloc %d, %d %dx%d",
@@ -2079,13 +2491,13 @@ void QWSServer::moveWindowRegion( QWSWindow *changingw, int dx, int dy )
     QSize s = QSize(swidth, sheight);
     QPoint p1 = qt_screen->mapFromDevice( QPoint(0, 0), s );
     QPoint p2 = qt_screen->mapFromDevice( QPoint(dx, dy), s );
-    
     QRect br( cr.boundingRect() );
     br = qt_screen->mapFromDevice( br, s );
     gfx->setClipDeviceRegion( cr );
     gfx->scroll( br.x(), br.y(), br.width(), br.height(),
 		 br.x() - (p2.x() - p1.x()), br.y() - (p2.y() - p1.y()) );
-    gfx->setClipRegion( qt_screen->mapFromDevice( screenRegion, s ) );
+    gfx->setClipDeviceRegion( screenRegion );
+
 #ifndef QT_NO_PALETTE
     clearRegion( exposed, qApp->palette().color( QPalette::Active, QColorGroup::Background ) );
 #endif
@@ -2225,9 +2637,7 @@ void QWSServer::openMouse()
 {
     QString mice = getenv("QWS_MOUSE_PROTO");
     if ( mice.isEmpty() ) {
-#if defined(QT_QWS_CASSIOPEIA)
-	mice = "TPanel:/dev/tpanel";
-#elif !defined(QT_NO_QWS_VFB)
+#if !defined(QT_NO_QWS_VFB)
 	extern bool qvfbEnabled;
 	if ( qvfbEnabled )
 	    mice = "QVFbMouse";
@@ -2236,15 +2646,23 @@ void QWSServer::openMouse()
 	    mice = defaultMouse;
     }
     closeMouse();
-    if ( mice == "None" )
-	return;
+    bool needviscurs = FALSE;
+    if ( mice != "None" ) {
 #ifndef QT_NO_STRINGLIST
-    QStringList mouse = QStringList::split(" ",mice);
-    for (QStringList::Iterator m=mouse.begin(); m!=mouse.end(); ++m) {
-	(void)newMouseHandler(*m);
-    }
+	QStringList mouse = QStringList::split(" ",mice);
+	for (QStringList::Iterator m=mouse.begin(); m!=mouse.end(); ++m) {
+	    QString ms = *m;
 #else
-    (void)newMouseHandler(mice); //Assume only one
+	QString ms = mice; // } Assume only one
+	{
+#endif
+	    QWSMouseHandler* h = newMouseHandler(ms);
+	    if ( h && !h->inherits("QCalibratedMouseHandler") )
+		needviscurs = TRUE;
+	}
+    }
+#ifndef QT_NO_QWS_CURSOR
+    setCursorVisible( needviscurs );
 #endif
 }
 
@@ -2339,11 +2757,20 @@ void QWSServer::request_focus( const QWSRequestFocusCommand *cmd )
     invokeSetFocus( cmd, client[-1] );
 }
 
+void QWSServer::set_identity( const QWSIdentifyCommand *cmd )
+{
+    invokeIdentify( cmd, client[-1] );
+}
 
 void QWSServer::request_region( int wid, QRegion region )
 {
     QWSClient *serverClient = client[-1];
     QWSWindow* changingw = findWindow( wid, 0 );
+    if ( !changingw ) {
+	if ( !region.isEmpty() )
+	    serverClient->sendRegionModifyEvent( wid, QRegion(), TRUE );
+	return;
+    }
     if ( !region.isEmpty() )
 	changingw->setNeedAck( TRUE );
     bool isShow = !changingw->isVisible() && !region.isEmpty();
@@ -2369,6 +2796,28 @@ void QWSServer::name_region( const QWSRegionNameCommand *cmd )
     invokeRegionName( cmd, client[-1] );
 }
 
+#ifndef QT_NO_QWS_IM
+void QWSServer::set_im_info( const QWSSetIMInfoCommand *cmd )
+{
+    invokeSetIMInfo( cmd, client[-1] );
+}
+
+void QWSServer::reset_im( const QWSResetIMCommand * )
+{
+    resetInputMethod();
+}
+
+void QWSServer::set_im_font( const QWSSetIMFontCommand *cmd )
+{
+    invokeSetIMFont( cmd, client[-1] );
+}
+
+void QWSServer::send_im_mouse( const QWSIMMouseCommand *cmd )
+{
+    if ( current_IM )
+	current_IM->mouseHandler( cmd->simpleData.index, cmd->simpleData.state );
+}
+#endif
 
 void QWSServer::openDisplay()
 {
@@ -2383,6 +2832,7 @@ void QWSServer::openDisplay()
 
 void QWSServer::closeDisplay()
 {
+    delete gfx;
     qt_screen->shutdownDevice();
 }
 
@@ -2391,10 +2841,11 @@ void QWSServer::paintServerRegion()
 {
 }
 
-void QWSServer::paintBackground( QRegion r )
+void QWSServer::paintBackground( const QRegion &rr )
 {
     if ( bgImage && bgImage->isNull() )
 	return;
+    QRegion r = rr;
     if ( !r.isEmpty() ) {
 	ASSERT ( qt_fbdpy );
 
@@ -2410,7 +2861,7 @@ void QWSServer::paintBackground( QRegion r )
 	    gfx->setBrushOffset( br.x(), br.y() );
 	    gfx->tiledBlt( br.x(), br.y(), br.width(), br.height() );
 	}
-	gfx->setClipRegion( QRegion() );
+	gfx->setClipDeviceRegion( screenRegion );
     }
 }
 
@@ -2509,7 +2960,7 @@ void QWSServer::emergency_cleanup()
 }
 
 #ifndef QT_NO_QWS_KEYBOARD
-static QWSServer::KeyboardFilter *keyFilter;
+static QList<QWSServer::KeyboardFilter> *keyFilters = 0;
 
 /*!
   \internal
@@ -2517,20 +2968,38 @@ static QWSServer::KeyboardFilter *keyFilter;
 void QWSServer::processKeyEvent(int unicode, int keycode, int modifiers, bool isPress,
   bool autoRepeat)
 {
-    if ( !keyFilter ||
-	 !keyFilter->filter( unicode, keycode, modifiers, isPress, autoRepeat ) )
-	sendKeyEvent( unicode, keycode, modifiers, isPress, autoRepeat );
+    if ( keyFilters ) {
+	QListIterator<QWSServer::KeyboardFilter> it(*keyFilters);
+	QWSServer::KeyboardFilter *keyFilter;
+	while ((keyFilter=it.current())) {
+	    if ( keyFilter->filter( unicode, keycode, modifiers, isPress, autoRepeat ) )
+		return;
+	    ++it;
+	}
+    }
+    sendKeyEvent( unicode, keycode, modifiers, isPress, autoRepeat );
 }
 
 /*!
-  Sets a filter to be invoked for all key events from physical keyboard
+  Adds a filter to be invoked for all key events from physical keyboard
   drivers (events sent via processKeyEvent()).
+
   The filter is not invoked for keys generated by virtual keyboard
   drivers (events send via sendKeyEvent()).
+
+  If \a f is 0, the most-recently added filter is removed and deleted.
+  The caller is responsible for matching each addition with a
+  corresponding removal.
 */
 void QWSServer::setKeyboardFilter( KeyboardFilter *f )
 {
-    keyFilter = f;
+    if ( !keyFilters )
+	keyFilters = new QList<QWSServer::KeyboardFilter>;
+    if ( f ) {
+	keyFilters->prepend(f);
+    } else {
+	delete keyFilters->take(0);
+    }
 }
 #endif
 
@@ -2541,6 +3010,8 @@ void QWSServer::setKeyboardFilter( KeyboardFilter *f )
 */
 void QWSServer::setScreenSaverIntervals(int* ms)
 {
+    if ( !qwsServer )
+	return;
     delete [] qwsServer->d->screensaverintervals;
     if ( ms ) {
 	int* t=ms;
@@ -2596,7 +3067,7 @@ void QWSServer::screenSaverWake()
 void QWSServer::screenSaverSleep()
 {
     qt_screen->blank(TRUE);
-#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_EBX)
+#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_SL5XXX)
     d->screensavertimer->stop();
 #else
     if ( screensaverinterval ) {
@@ -2671,3 +3142,160 @@ void QWSServer::screenSaverActivate(bool activate)
     else
 	qwsServer->screenSaverWake();
 }
+
+void QWSServer::disconnectClient( QWSClient *c )
+{
+    QTimer::singleShot( 0, c, SLOT(closeHandler()) );
+}
+
+void QWSServer::updateClientCursorPos()
+{
+    QWSWindow *win = qwsServer->mouseGrabber ? qwsServer->mouseGrabber : qwsServer->windowAt( mousePosition );
+    QWSClient *winClient = win ? win->client() : 0;
+    if ( winClient && winClient != d->cursorClient )
+	sendMouseEvent( mousePosition, d->mouseState );
+}
+
+
+#ifndef QT_NO_QWS_IM
+/*!
+  \class QWSInputMethod
+  \brief International input methods for Qt/Embedded
+
+  Subclass this to implement your own input method
+
+  An input methods consists of a keyboard filter and optionally a
+  graphical interface. The keyboard filter intercepts key events
+  from physical or virtual keyboards by implementing the filter()
+  function.
+
+  Use sendIMEvent() to send composition events. Composition starts
+  with the input method sending the first \c IMCompose event.  This can be 
+  followed by a number of \c IMCompose events and ending with an \c IMEnd event.
+
+  The function setMicroFocus() is called when the focus widget changes
+  its cursor position.
+
+  The functions font() and inputRect() provide more information about
+  the state of the focus widget.
+
+
+  Use QWSServer::setCurrentInputMethod() to install an input method.
+
+ */
+
+/*!
+  Constructs a new input method
+*/
+
+QWSInputMethod::QWSInputMethod()
+{
+    
+}
+
+/*!
+  Destructs the input method uninstalling it if it is currently installed.
+ */
+QWSInputMethod::~QWSInputMethod()
+{
+    if ( current_IM == this )
+	current_IM = 0;
+}
+
+
+
+
+/*!
+  \fn bool QWSInputMethod::filter(int unicode, int keycode, int modifiers, bool isPress, bool autoRepeat)
+
+  Must be implemented in subclasses to handle key input from physical
+  or virtual keyboards. Returning TRUE will block the event from
+  further processing.
+
+  All normal key events should be blocked while in compose mode
+  (between \c IMCompose and \c IMEnd).
+
+  
+
+*/
+
+
+/*!
+  Implemented in subclasses to reset the state of the input method.
+*/
+
+void QWSInputMethod::reset()
+{
+    
+}
+
+
+/*!
+  \fn void QWSInputMethod::setMicroFocus( int x, int y )
+
+  Implemented in subclasses to handle microFocusHint changes in the
+  focus widget. \a x and \a y are the global coordinates of the 
+  cursor position.
+*/
+
+void QWSInputMethod::setMicroFocus( int, int )
+{
+    
+}
+
+/*!
+  \fn void QWSInputMethod::mouseHandler( int x, int state )
+
+  Implemented in subclasses to handle mouse presses/releases within
+  the on-the-spot text. The parameter \a x is the offset within
+  the string that was sent with the IMCompose event.
+  \a state is either \c QWSServer::MousePress or \c QWSServer::MouseRelease
+ */
+void QWSInputMethod::mouseHandler( int, int )
+{
+}
+
+
+/*!
+  Returns the font of the current input widget
+ */
+QFont QWSInputMethod::font() const
+{
+    if ( !current_IM_Font )
+	return QApplication::font(); //### absolutely last resort
+
+    return *current_IM_Font;
+}
+
+/*!
+  Returns the input rectangle of the current input widget. The input
+  rectangle covers the width of the input widget, and may extend below it.
+  This can be used to determine the geometry of an input widget for
+  over-the-spot input methods.
+ */
+QRect QWSInputMethod::inputRect() const
+{
+    QRect r = current_IM_Rect;
+    //QFontMetrics fm( font() );
+    //r.setTop( current_IM_y - fm.height() );
+    return r;
+}
+
+
+/*!
+  \fn QWSInputMethod::sendIMEvent( QWSServer::IMState state, const QString &txt, int cpos, int selLen )
+
+  Causes a QIMEvent to be sent to the focus widget. \a state may be either
+  \c QWSServer::IMCompose or \c QWSServer::IMEnd.
+
+  \a txt is the text being composed (or the finished text if state is
+  \c IMEnd). \a cpos is the current cursor position.
+
+  If state is \c IMCompose,
+  \a selLen is the number of characters in the composition string (
+  starting at \a cpos ) that should be marked as selected by the
+  input widget receiving the event.
+*/
+
+
+#endif // QT_NO_QWS_IM
