@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/kernel/qwsmouse_qws.cpp   2.3.2   edited 2001-10-14 $
+** $Id: qt/src/kernel/qwsmouse_qws.cpp   2.3.7   edited 2003-02-04 $
 **
 ** Implementation of Qt/Embedded mouse drivers
 **
@@ -68,8 +68,8 @@ typedef struct {
         unsigned short y;
         unsigned short pad;
 } TS_EVENT;
-#elif defined(QT_QWS_EBX)
-#define QT_QWS_EBX_RAW
+#elif defined(QT_QWS_SL5XXX)
+#define QT_QWS_SL5XXX_RAW
 #ifndef QT_QWS_CUSTOM
 typedef struct {
         unsigned short pressure;
@@ -84,9 +84,11 @@ typedef struct {
        long pressure;
        long long millisecs;
 } TS_EVENT;
-#define QT_QWS_TP_SAMPLE_SIZE 5
+#define QT_QWS_TP_SAMPLE_SIZE 10
+#define QT_QWS_TP_MINIMUM_SAMPLES 4
 #define QT_QWS_TP_PRESSURE_THRESHOLD 500
 #define QT_QWS_TP_MOVE_LIMIT 50
+#define QT_QWS_TP_JITTER_LIMIT 2
 #endif
 #endif
 
@@ -94,6 +96,10 @@ typedef struct {
 #define QT_QWS_TP_SAMPLE_SIZE 5
 #endif
 
+#ifndef QT_QWS_TP_MINIMUM_SAMPLES
+#define QT_QWS_TP_MINIMUM_SAMPLES 5
+#endif
+
 #ifndef QT_QWS_TP_PRESSURE_THRESHOLD
 #define QT_QWS_TP_PRESSURE_THRESHOLD 1
 #endif
@@ -102,6 +108,10 @@ typedef struct {
 #define QT_QWS_TP_MOVE_LIMIT 100
 #endif
 
+#ifndef QT_QWS_TP_JITTER_LIMIT
+#define QT_QWS_TP_JITTER_LIMIT 2
+#endif 
+
 //#define QWS_CUSTOMTOUCHPANEL
 
 /*!
@@ -151,15 +161,8 @@ enum MouseProtocol { Unknown = -1, Auto = 0,
 
 static void limitToScreen( QPoint &pt )
 {
-    static int w = -1;
-    static int h;
-    if ( w < 0 ) {
-	w = qt_screen->deviceWidth();
-	h = qt_screen->deviceHeight();
-    }
-
-    pt.setX( QMIN( w-1, QMAX( 0, pt.x() )));
-    pt.setY( QMIN( h-1, QMAX( 0, pt.y() )));
+    pt.setX( QMIN( qt_screen->deviceWidth()-1, QMAX( 0, pt.x() )));
+    pt.setY( QMIN( qt_screen->deviceHeight()-1, QMAX( 0, pt.y() )));
 }
 
 static QPoint &mousePos = QWSServer::mousePosition;
@@ -174,6 +177,7 @@ static const MouseConfig mouseConfig[] = {
     { "Auto",		Auto },
     { "MouseMan",	MouseMan },
     { "IntelliMouse",	IntelliMouse },
+    { "USB",		IntelliMouse },
     { "Microsoft",      Microsoft },
     { "QVFbMouse",      QVFBMouse },
     { "TPanel",         TPanel },
@@ -332,7 +336,7 @@ protected:
 	tty.c_oflag     = 0;
 	tty.c_lflag     = 0;
 	tty.c_cflag     = f | CREAD | CLOCAL | HUPCL;
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
 	tty.c_line      = 0;
 #endif
 	tty.c_cc[VTIME] = 0;
@@ -344,7 +348,7 @@ private:
     void initSerial()
     {
 	int speed[4] = { B9600, B4800, B2400, B1200 };
- 
+
         for (int n = 0; n < 4; n++) {
 	    setflags(CSTOPB | speed[n]);
 	    write(fd, "*q", 2);
@@ -572,7 +576,7 @@ void QAutoMouseHandler::openDevices()
 	sub[nsub++] = new QAutoMouseSubHandler_intellimouse(fd);
 	notify(fd);
     }
-#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_EBX)
+#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_SL5XXX)
     char fn[] = "/dev/ttyS?";
     for (int ch='0'; ch<='3'; ch++) {
 	fn[9] = ch;
@@ -665,8 +669,8 @@ static const MouseData mouseData[] = {
     { 3 },  // MouseMan
     { 4 },  // intelliMouse
     { 3 },  // Microsoft
-    { 0 },  // QVFBMouse, 
-    { 0 },  // TPanel, 
+    { 0 },  // QVFBMouse,
+    { 0 },  // TPanel,
     { 3 },  // BusMouse,
 };
 
@@ -697,7 +701,7 @@ void QWSMouseHandlerPrivate::readMouseData()
 {
     int n;
     if ( BusMouse == mouseProtocol ) {
-	// a workaround of linux busmouse driver interface.  
+	// a workaround of linux busmouse driver interface.
 	// It'll only read 3 bytes a time and return all other buffer zeroed, thus cause protocol errors
 	for (;;) {
 	    if ( mouseBufSize - mouseIdx < 3 )
@@ -781,12 +785,10 @@ void QWSMouseHandlerPrivate::handleMouseData()
 		break;
 	    }
 	    case Microsoft:
-	        if ( ((mb[0] & 0x20) >> 3) ) {
+	        if ( (mb[0] & 0x20) )
 		    bstate |= Qt::LeftButton;
-		}
-		if ( ((mb[0] & 0x10) >> 4) ) {
+		if ( (mb[0] & 0x10) )
 		    bstate |= Qt::RightButton;
-		}
 
 		dx=(signed char)(((mb[0] & 0x03) << 6) | (mb[1] & 0x3f));
 		dy=-(signed char)(((mb[0] & 0x0c) << 4) | (mb[2] & 0x3f));
@@ -794,12 +796,10 @@ void QWSMouseHandlerPrivate::handleMouseData()
 
 		break;
 	    case BusMouse:
-	        if ( ((mb[0] & 0x4))  ) {
+	        if ( !(mb[0] & 0x04) )
 		    bstate |= Qt::LeftButton;
-		}
-		if ( ((mb[0] & 0x01)) ) {
+		if ( !(mb[0] & 0x01) )
 		    bstate |= Qt::RightButton;
-		}
 
 		dx=(signed char)mb[1];
 		dy=(signed char)mb[2];
@@ -889,7 +889,7 @@ QWSMouseHandlerPrivate::QWSMouseHandlerPrivate( MouseProtocol protocol,
 		tty.c_iflag = IGNBRK | IGNPAR;
 		tty.c_oflag = 0;
 		tty.c_lflag = 0;
-#if !defined(_OS_FREEBSD_)
+#if !defined(_OS_FREEBSD_) && !defined(_OS_SOLARIS_)
 		tty.c_line = 0;
 #endif // _OS_FREEBSD_
 		tty.c_cc[VTIME] = 0;
@@ -1219,22 +1219,26 @@ private:
     static const int mouseBufSize = 2048;
     int mouseFD;
     QPoint oldmouse;
+    QPoint oldTotalMousePos;
     bool waspressed;
     QPointArray samples;
     unsigned int currSample;
+    unsigned int lastSample;
     unsigned int numSamples;
+    int skipCount;
     int mouseIdx;
     uchar mouseBuf[mouseBufSize];
-    
+
 private slots:
-    void readMouseData();   
+    void readMouseData();
 };
 
 
-QTPanelHandlerPrivate::QTPanelHandlerPrivate( MouseProtocol, QString )
-    : samples(QT_QWS_TP_SAMPLE_SIZE), currSample(0), numSamples(0)
+QTPanelHandlerPrivate::QTPanelHandlerPrivate( MouseProtocol, QString dev )
+    : samples(QT_QWS_TP_SAMPLE_SIZE), currSample(0), lastSample(0), 
+    numSamples(0), skipCount(0)
 {
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
 #if defined(QT_QWS_IPAQ)
 # ifdef QT_QWS_IPAQ_RAW
     if ((mouseFD = open( "/dev/h3600_tsraw", O_RDONLY | O_NDELAY)) < 0) {
@@ -1244,8 +1248,8 @@ QTPanelHandlerPrivate::QTPanelHandlerPrivate( MouseProtocol, QString )
         qWarning( "Cannot open /dev/h3600_ts (%s)", strerror(errno));
 	return;
     }
-#elif defined(QT_QWS_EBX)
-//# ifdef QT_QWS_EBX_TSRAW
+#elif defined(QT_QWS_SL5XXX)
+//# ifdef QT_QWS_SL5XXX_TSRAW
 # if 0
     if ((mouseFD = open( "/dev/tsraw", O_RDONLY | O_NDELAY)) < 0) {
         qWarning( "Cannot open /dev/tsraw (%s)", strerror(errno));
@@ -1270,7 +1274,7 @@ QTPanelHandlerPrivate::QTPanelHandlerPrivate( MouseProtocol, QString )
 
 QTPanelHandlerPrivate::~QTPanelHandlerPrivate()
 {
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
     if (mouseFD >= 0)
 	close(mouseFD);
 #endif
@@ -1278,7 +1282,7 @@ QTPanelHandlerPrivate::~QTPanelHandlerPrivate()
 
 void QTPanelHandlerPrivate::readMouseData()
 {
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
+#if defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
     if(!qt_screen)
 	return;
 
@@ -1291,64 +1295,78 @@ void QTPanelHandlerPrivate::readMouseData()
 
     TS_EVENT *data;
     int idx = 0;
+
+    // perhaps we shouldn't be reading EVERY SAMPLE.
     while ( mouseIdx-idx >= (int)sizeof( TS_EVENT ) ) {
 	uchar *mb = mouseBuf+idx;
 	data = (TS_EVENT *) mb;
 	if(data->pressure >= QT_QWS_TP_PRESSURE_THRESHOLD) {
-#ifdef QT_QWS_CUSTOM
+#ifdef QT_QWS_SL5XXX
 	    samples[currSample] = QPoint( 1000 - data->x, data->y );
 #else
 	    samples[currSample] = QPoint( data->x, data->y );
 #endif
+
 	    numSamples++;
-	    if ( numSamples >= samples.count() ) {
-		int maxd = 0;
-		unsigned int ignore = 0;
-		// throw away the "worst" sample
-		for ( unsigned int i = 0; i < samples.count(); i++ ) {
-		    int d = ( mousePos - samples[i] ).manhattanLength();
-		    if ( d > maxd ) {
-			maxd = d;
-			ignore = i;
-		    }
-		}
-		bool first = TRUE;
+	    if ( numSamples >= QT_QWS_TP_MINIMUM_SAMPLES ) {
+		int sampleCount = QMIN(numSamples + 1,samples.count());
+
 		// average the rest
-		for ( unsigned int i = 0; i < samples.count(); i++ ) {
-		    if ( ignore != i ) {
-			if ( first ) {
-			    mousePos = samples[i];
-			    first = FALSE;
-			} else {
-			    mousePos += samples[i];
-			}
-		    }
-		}
-		mousePos /= (int)(samples.count() - 1);
-# if defined(QT_QWS_IPAQ_RAW) || defined(QT_QWS_EBX_RAW)
+		mousePos = QPoint( 0, 0 );
+		QPoint totalMousePos = oldTotalMousePos;
+		totalMousePos += samples[currSample];
+		if(numSamples >= samples.count()) 
+		    totalMousePos -= samples[lastSample];
+
+		mousePos = totalMousePos / (sampleCount - 1);
+
+# if defined(QT_QWS_IPAQ_RAW) || defined(QT_QWS_SL5XXX_RAW)
 		mousePos = transform( mousePos );
 # endif
-		if ( waspressed ) {
-		    if ( QABS(mousePos.x()-oldmouse.x()) < QT_QWS_TP_MOVE_LIMIT &&
-			 QABS(mousePos.y()-oldmouse.y()) < QT_QWS_TP_MOVE_LIMIT ) {
-			if ( oldmouse != mousePos ) {
+		if(!waspressed)
+		    oldmouse = mousePos;
+		QPoint dp = mousePos - oldmouse;
+		int dxSqr = dp.x() * dp.x();
+		int dySqr = dp.y() * dp.y();
+		if ( dxSqr + dySqr < (QT_QWS_TP_MOVE_LIMIT * QT_QWS_TP_MOVE_LIMIT) ) {
+		    if ( waspressed ) {
+			if ( (dxSqr + dySqr > (QT_QWS_TP_JITTER_LIMIT * QT_QWS_TP_JITTER_LIMIT) ) || skipCount > 2) {
 			    emit mouseChanged(mousePos,Qt::LeftButton);
-			    oldmouse=mousePos;
+			    oldmouse = mousePos;
+			    skipCount = 0;
+			} else {
+			    skipCount++;
 			}
+		    } else {
+			emit mouseChanged(mousePos,Qt::LeftButton);
+			oldmouse=mousePos;
+			waspressed=true;
 		    }
+
+		    // save recuring information
+		    currSample++;
+		    if (numSamples >= samples.count()) 
+			lastSample++;
+		    oldTotalMousePos = totalMousePos;
 		} else {
-		    emit mouseChanged(mousePos,Qt::LeftButton);
-		    oldmouse=mousePos;
-		    waspressed=true;
+		    numSamples--; // don't use this sample, it was bad.
 		}
+	    } else {
+		// build up the average
+		oldTotalMousePos += samples[currSample];
+		currSample++;
 	    }
-	    currSample++;
 	    if ( currSample >= samples.count() )
 		currSample = 0;
+	    if ( lastSample >= samples.count() )
+		lastSample = 0;
 	} else {
+	    currSample = 0;
+	    lastSample = 0;
+	    numSamples = 0;
+	    skipCount = 0;
+	    oldTotalMousePos = QPoint(0,0);
 	    if ( waspressed ) {
-		currSample = 0;
-		numSamples = 0;
 		emit mouseChanged(oldmouse,0);
 		oldmouse = QPoint( -100, -100 );
 		waspressed=false;
@@ -1638,6 +1656,9 @@ QWSMouseHandler* QWSServer::newMouseHandler(const QString& spec)
 	mouseProto = spec;
     }
 
+    if ( mouseProto == "USB" && mouseDev.isEmpty() )
+	mouseDev = "/dev/input/mice";
+    
     MouseProtocol mouseProtocol = Unknown;
 
     int idx = 0;
@@ -1651,20 +1672,6 @@ QWSMouseHandler* QWSServer::newMouseHandler(const QString& spec)
 
     QWSMouseHandler *handler = 0;
 
-#ifdef QWS_CUSTOMTOUCHPANEL
-    handler=new QCustomTPanelHandlerPrivate(mouseProtocol,mouseDev);
-#endif
-
-#ifdef QT_QWS_YOPY
-    handler=new QYopyTPanelHandlerPrivate(mouseProtocol,mouseDev);
-#endif
-
-#if defined(QT_QWS_IPAQ) || defined(QT_QWS_EBX)
-    handler=new QTPanelHandlerPrivate(mouseProtocol,mouseDev);
-#endif
-
-#ifndef QWS_CUSTOMTOUCHPANEL
-#if !defined(QT_QWS_IPAQ) && !defined(QT_QWS_EBX)
     switch ( mouseProtocol ) {
 	case Auto:
 	    handler = new QAutoMouseHandler();
@@ -1682,14 +1689,20 @@ QWSMouseHandler* QWSServer::newMouseHandler(const QString& spec)
 	    break;
 
 	case TPanel:
+#if defined(QWS_CUSTOMTOUCHPANEL)
+	    handler = new QCustomTPanelHandlerPrivate(mouseProtocol,mouseDev);
+#elif defined(QT_QWS_YOPY)
+	    handler = new QYopyTPanelHandlerPrivate(mouseProtocol,mouseDev);
+#elif defined(QT_QWS_IPAQ) || defined(QT_QWS_SL5XXX)
+	    handler = new QTPanelHandlerPrivate(mouseProtocol,mouseDev);
+#elif defined(QT_QWS_CASSIOPEIA)
 	    handler = new QVrTPanelHandlerPrivate( mouseProtocol, mouseDev );
+#endif
 	    break;
 
 	default:
 	    qDebug( "Mouse type %s unsupported", spec.latin1() );
     }
-#endif
-#endif
 
     return handler;
 }
