@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/network/qdns.cpp   2.3.2   edited 2001-10-14 $
+** $Id: qt/src/network/qdns.cpp   2.3.7   edited 2002-04-03 $
 **
 ** Implementation of QDns class.
 **
@@ -815,11 +815,17 @@ public:
 
 static QDnsManager * globalManager;
 
+static void cleanupDns()
+{
+    delete globalManager;
+}
 
 QDnsManager * QDnsManager::manager()
 {
-    if ( !globalManager )
+    if ( !globalManager ) {
+        qAddPostRoutine(cleanupDns);
 	new QDnsManager();
+    }
     return globalManager;
 }
 
@@ -916,6 +922,7 @@ QDnsManager::~QDnsManager()
 {
     if ( globalManager )
 	globalManager = 0;
+    delete socket;
 }
 
 static Q_UINT32 lastSweep = 0;
@@ -1741,12 +1748,15 @@ bool QDns::isWorking() const
     QList<QDnsRR> * ll = QDnsDomain::cached( this );
     int queries = n.count();
     while( ll->current() != 0 ) {
-	if ( ll->current()->nxdomain )
+	if ( ll->current()->nxdomain ) {
 	    queries--;
-	else
+	} else {
+	    delete ll;
 	    return FALSE;
+	}
 	ll->next();
     }
+    delete ll;
 
     if ( queries <= 0 )
 	return FALSE;
@@ -2128,40 +2138,36 @@ static void doResInit()
     HKEY k1, k2;
 
     bool gotNetworkParams = FALSE;
-    if ( QApplication::winVersion() == Qt::WV_98 ||
-	 QApplication::winVersion() == Qt::WV_2000 ) {
-	// for 98 and 2000 try the API call GetNetworkParams()
-	HINSTANCE hinstLib = LoadLibraryA( "iphlpapi" ); // we don't need Unicode here
-	if ( hinstLib != 0 ) {
-	    GNP getNetworkParams = (GNP) GetProcAddress( hinstLib, "GetNetworkParams" );
-	    if ( getNetworkParams != 0 ) {
-		ULONG l = 0;
-		DWORD res;
-		res = getNetworkParams( 0, &l );
-		if ( res == ERROR_BUFFER_OVERFLOW ) {
-		    FIXED_INFO *finfo = (FIXED_INFO*)new char[l];
-		    res = getNetworkParams( finfo, &l );
-		    if ( res == ERROR_SUCCESS ) {
-			domainName = finfo->DomainName;
-			nameServer = "";
-			IP_ADDR_STRING *dnsServer = &finfo->DnsServerList;
-			while ( dnsServer != 0 ) {
-			    nameServer += dnsServer->IpAddress.String;
-			    dnsServer = dnsServer->Next;
-			    if ( dnsServer != 0 )
-				nameServer += " ";
-			}
-			searchList = "";
-			separator = ' ';
-			gotNetworkParams = TRUE;
+    // try the API call GetNetworkParams() first and use registry lookup only
+    // as a fallback
+    HINSTANCE hinstLib = LoadLibraryA( "iphlpapi" ); // we don't need Unicode here
+    if ( hinstLib != 0 ) {
+	GNP getNetworkParams = (GNP) GetProcAddress( hinstLib, "GetNetworkParams" );
+	if ( getNetworkParams != 0 ) {
+	    ULONG l = 0;
+	    DWORD res;
+	    res = getNetworkParams( 0, &l );
+	    if ( res == ERROR_BUFFER_OVERFLOW ) {
+		FIXED_INFO *finfo = (FIXED_INFO*)new char[l];
+		res = getNetworkParams( finfo, &l );
+		if ( res == ERROR_SUCCESS ) {
+		    domainName = finfo->DomainName;
+		    nameServer = "";
+		    IP_ADDR_STRING *dnsServer = &finfo->DnsServerList;
+		    while ( dnsServer != 0 ) {
+			nameServer += dnsServer->IpAddress.String;
+			dnsServer = dnsServer->Next;
+			if ( dnsServer != 0 )
+			    nameServer += " ";
 		    }
-		    delete[] finfo;
+		    searchList = "";
+		    separator = ' ';
+		    gotNetworkParams = TRUE;
 		}
+		delete[] finfo;
 	    }
-	    FreeLibrary( hinstLib );
 	}
-	if ( !gotNetworkParams )
-	    qWarning( "QDns: call GetNetworkParams() was unsuccessful!" );
+	FreeLibrary( hinstLib );
     }
     if ( !gotNetworkParams ) {
 	// this is for NT
@@ -2199,18 +2205,20 @@ static void doResInit()
 
     nameServer = nameServer.simplifyWhiteSpace();
     int first, last;
-    first = 0;
-    do {
-	last = nameServer.find( separator, first );
-	if ( last < 0 )
-	    last = nameServer.length();
-	QDns tmp( nameServer.mid( first, last-first ), QDns::A );
-	QValueList<QHostAddress> address = tmp.addresses();
-	int i = address.count();
-	while( i )
-	    ns->append( new QHostAddress(address[--i]) );
-	first = last+1;
-    } while( first < (int)nameServer.length() );
+    if ( !nameServer.isEmpty() ) {
+	first = 0;
+	do {
+	    last = nameServer.find( separator, first );
+	    if ( last < 0 )
+		last = nameServer.length();
+	    QDns tmp( nameServer.mid( first, last-first ), QDns::A );
+	    QValueList<QHostAddress> address = tmp.addresses();
+	    int i = address.count();
+	    while( i )
+		ns->append( new QHostAddress(address[--i]) );
+	    first = last+1;
+	} while( first < (int)nameServer.length() );
+    }
 
     searchList = searchList + " " + domainName;
     searchList = searchList.simplifyWhiteSpace().lower();
