@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qdatastream.cpp   2.3.2   edited 2001-10-14 $
+** $Id: qt/src/tools/qdatastream.cpp   2.3.7   edited 2002-10-03 $
 **
 ** Implementation of QDataStream class
 **
@@ -671,8 +671,11 @@ QDataStream &QDataStream::readRawBytes( char *s, uint len )
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
 	register Q_INT8 *p = (Q_INT8*)s;
-	while ( len-- )
-	    *this >> *p++;
+	while ( len-- ) {
+	    Q_INT32 tmp;
+	    *this >> tmp;
+	    *p++ = tmp;
+	}
     } else {					// read data char array
 	dev->readBlock( s, len );
     }
