@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qgdict.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/src/tools/qgdict.cpp   2.3.7   edited 2002-01-28 $
 **
 ** Implementation of QGDict and QGDictIterator classes
 **
@@ -88,7 +88,7 @@ int QGDict::hashKeyString( const QString &key )
 {
 #if defined(CHECK_NULL)
     if ( key.isNull() )
-	qWarning( "QGDict::hashStringKey: Invalid null key" );
+	qWarning( "QGDict::hashKeyString: Invalid null key" );
 #endif
     int i;
     register uint h=0;
