@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/tools/qtextcodec.cpp   2.3.2   edited 2001-07-31 $
+** $Id: qt/src/tools/qtextcodec.cpp   2.3.7   edited 2002-10-28 $
 **
 ** Implementation of QTextCodec class
 **
@@ -586,8 +586,9 @@ QTextCodec* QTextCodec::codecForLocale()
     // 2. CODESET from lang if it contains a .CODESET part
     // 3. ctype (maybe the locale is named "ISO-8859-1" or something)
     // 4. locale (ditto)
-    // 5. guess locale from ctype unless ctype is "C"
-    // 6. guess locale from lang
+    // 5. use 8859-15 if lang or ctype contain "@euro"
+    // 6. guess locale from ctype unless ctype is "C"
+    // 7. guess locale from lang
 
     // 1. CODESET from ctype if it contains a .CODESET part (e.g. en_US.ISO8859-15)
     char * codeset = ctype ? strchr( ctype, '.' ) : 0;
@@ -607,12 +608,17 @@ QTextCodec* QTextCodec::codecForLocale()
     if ( !localeMapper && lang && *lang != 0 )
 	localeMapper = codecForName( lang );
 
-    // 5. guess locale from ctype unless ctype is "C"
-    // 6. guess locale from lang
+    // 5. "@euro"
+    if ( (ctype && strstr( ctype, "@euro" )) || 
+	 (lang && strstr( lang, "@euro" )) )
+	localeMapper = codecForName( "ISO 8859-15" );
+
+    // 6. guess locale from ctype unless ctype is "C"
+    // 7. guess locale from lang
     char * try_by_name = ctype;
     if ( ctype && *ctype != 0 && qstrcmp (ctype, "C") != 0 )
 	try_by_name = lang;
-
+    
     if ( !localeMapper && try_by_name && *try_by_name != 0 ) {
         // if there is none, we default to 8859-1
         // We could perhaps default to 8859-15.
@@ -983,7 +989,7 @@ struct QMultiByteUnicodeTable {
     QMultiByteUnicodeTable* multibyte;
 };
 
-#ifndef QT_NO_CODECS
+#ifndef QT_NO_CODEC_CHARMAP
 
 static int getByte(char* &cursor)
 {
@@ -1338,11 +1344,11 @@ QString QTextCodecFromIODDecoder::toUnicode(const char* chars, int len)
 QTextCodec* QTextCodec::loadCharmap(QIODevice* iod)
 {
     QTextCodecFromIOD* r = new QTextCodecFromIOD(iod);
-    if ( !r->ok() ) {
+    if ( !r->ok() )
         delete r;
-        r = 0;
-    }
-    return r;
+    else
+	return r;
+    return 0;
 }
 
 /*!
@@ -1360,8 +1366,7 @@ QTextCodec* QTextCodec::loadCharmapFile(QString filename)
     }
     return 0;
 }
-#endif //QT_NO_CODECS
-
+#endif // QT_NO_CODEC_CHARMAP
 
 /*!
   Returns a string representing the current language.
@@ -1380,7 +1385,7 @@ const char* QTextCodec::locale()
 
 
 
-#ifndef QT_NO_CODECS
+#ifndef QT_NO_SIMPLE_CODECS
 
 class QSimpleTextCodec: public QTextCodec
 {
@@ -1409,7 +1414,7 @@ static struct {
     const char * cs;
     int mib;
     Q_UINT16 values[128];
-} unicodevalues[] = {
+} const unicodevalues[] = {
     // from RFC 1489, ftp://ftp.isi.edu/in-notes/rfc1489.txt
     { "KOI8-R", 2084,
       { 0x2500, 0x2502, 0x250C, 0x2510, 0x2514, 0x2518, 0x251C, 0x2524,
@@ -1901,23 +1906,27 @@ QSimpleTextCodec::~QSimpleTextCodec()
     }
 }
 
-// what happens if strlen(chars)<len?  what happens if !chars?  if len<1?
 QString QSimpleTextCodec::toUnicode(const char* chars, int len) const
 {
-    if(len <= 0)
+    if ( len <= 0 || chars == 0 )
         return QString::null;
 
-    int clen = qstrlen(chars);
-    len = QMIN(len, clen); // Note: NUL ends string
+    const unsigned char * c = (const unsigned char *)chars;
+    int i;
+
+    for ( i = 0; i < len; i++ ) {
+	if ( c[i] == '\0' )
+	    len = i;
+    }
 
     QString r;
     r.setUnicode(0, len);
     QChar* uc = (QChar*)r.unicode(); // const_cast
-    const unsigned char * c = (const unsigned char *)chars;
-    for( int i=0; i<len; i++ ) {
-        if ( c[i] > 127 )
+
+    for ( i = 0; i < len; i++ ) {
+	if ( c[i] > 127 )
             uc[i] = unicodevalues[forwardIndex].values[c[i]-128];
-        else
+	else
             uc[i] = c[i];
     }
     return r;
@@ -2029,7 +2038,7 @@ int QSimpleTextCodec::heuristicContentMatch(const char* chars, int len) const
 }
 
 
-#endif // QT_NO_CODECS
+#endif // QT_NO_SIMPLE_CODECS
 
 class QLatin1Codec: public QTextCodec
 {
@@ -2062,7 +2071,6 @@ QLatin1Codec::~QLatin1Codec()
 {
 }
 
-// what happens if strlen(chars)<len?  what happens if !chars?  if len<1?
 QString QLatin1Codec::toUnicode(const char* chars, int len) const
 {
     if(len <= 0)
@@ -2131,22 +2139,37 @@ static void setupBuiltinCodecs()
     (void)new QUtf8Codec;
     (void)new QUtf16Codec;
 
-#ifndef QT_NO_CODECS
+#ifndef QT_NO_SIMPLE_CODECS
     int i = 0;
     do {
         (void)new QSimpleTextCodec( i );
     } while( unicodevalues[i++].mib != LAST_MIB );
-
+#endif
+#ifndef QT_NO_EUCJPCODEC
     (void)new QEucJpCodec;
+#endif
+#ifndef QT_NO_SJISCODEC
     (void)new QSjisCodec;
+#endif
+#ifndef QT_NO_JISCODEC
     (void)new QJisCodec;
+#endif
+#ifndef QT_NO_EUCKRCODEC
     (void)new QEucKrCodec;
+#endif
+#ifndef QT_NO_GBKCODEC
     (void)new QGbkCodec;
+#endif
+#ifndef QT_NO_BIG5CODEC
     (void)new QBig5Codec;
-    (void)new QHebrewCodec;
+#endif
+#ifndef QT_NO_RTLCODEC
     (void)new QArabicCodec;
+    (void)new QHebrewCodec;
+#endif
+#ifndef QT_NO_TSCIICODEC
     (void)new QTsciiCodec;
-#endif // QT_NO_CODECS
+#endif
 }
 
 #endif // QT_NO_TEXTCODEC
