@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qheader.cpp   2.3.2   edited 2001-07-04 $
+** $Id: qt/src/widgets/qheader.cpp   2.3.7   edited 2001-11-26 $
 **
 ** Implementation of QHeader widget class (table header)
 **
@@ -847,6 +847,80 @@ void QHeader::removeLabel( int section )
 }
 
 
+/*!
+  Appends or removes (from the end) sections so that the total
+  number equals /a n. New sections get a default label.
+
+  Faster than calling addLabel()/removeLabel() in a loop.
+ */
+void QHeader::setNumLabels( int n )
+{
+    if ( n > d->count ) {
+	// add labels
+
+	int lastpos = 0;
+	if ( d->count )
+	    lastpos = d->positions[d->count-1] + d->sizes[d->i2s[d->count-1]];
+
+	d->labels.resize( n );
+	if ( int( d->iconsets.size() ) < n  )
+	    d->iconsets.resize( n );
+	d->sizes.resize( n );
+	d->positions.resize( n );
+	d->heights.resize( n );
+
+
+	d->i2s.resize( n );
+	d->s2i.resize( n );
+
+	d->clicks.resize( n );
+	d->resize.resize( n );
+
+	QFontMetrics fm(font());
+	int height = fm.lineSpacing() + 6;
+	int width = 20;
+
+	int size;
+	if ( orient == Horizontal )
+	    size = width;
+	else
+	    size = height;
+
+	for ( int section = d->count; section < n; section++ ) {
+	    d->labels.insert( section, new QString( QString::number(section) ) );
+	    /*
+	      int iw = 0;
+	      int ih = 0;
+	      if ( d->iconsets[section] != 0 ) {
+	      iw = d->iconsets[section]->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
+	      ih = d->iconsets[section]->pixmap( QIconSet::Small, QIconSet::Normal ).height();
+	      }
+	    */
+
+	    int index = section;
+	    d->sizes[section] = size;
+	    d->positions[index] = lastpos;
+	    // we abuse the heights as widths for vertical layout
+	    d->heights[section] = orient == Horizontal ? height : width;
+
+	    d->s2i[section] = index;
+	    d->i2s[index] = section;
+	    d->clicks.setBit( section, d->clicks_default );
+	    d->resize.setBit( section, d->resize_default );
+
+	    lastpos += size;
+
+	}
+
+	d->count = n;
+
+    } else if ( n < d->count ) {
+	qWarning( "not really implemented yet" );
+	while ( n < d->count )
+	    removeLabel( d->count - 1 );
+
+    }
+}
 
 /*!
   Adds a new section, with label text \a s. Returns the index.
