@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/widgets/qlineedit.cpp   2.3.2   edited 2001-10-19 $
+** $Id: qt/src/widgets/qlineedit.cpp   2.3.7   edited 2003-05-02 $
 **
 ** Implementation of QLineEdit widget class
 **
@@ -51,6 +51,10 @@
 #include "qguardedptr.h"
 #include <ctype.h>
 
+#if defined( _WS_QWS_ ) && !defined( QT_NO_QWS_IM )    
+#include "qwsdisplay_qws.h"
+#endif
+
 
 struct QLineEditUndoItem
 {
@@ -86,7 +90,11 @@ struct QLineEditPrivate {
 	dndTimer( l, "DnD Timer" ),
 	inDoubleClick( FALSE ), offsetDirty( FALSE ),
 	undo(TRUE), needundo( FALSE ), ignoreUndoWithDel( FALSE ),
-	mousePressed( FALSE ), dnd_primed( FALSE ), passwordChar( '*' ) {}
+	mousePressed( FALSE ), dnd_primed( FALSE ), passwordChar( '*' )
+#ifndef QT_NO_QWS_IM
+        ,preeditStart(-1), preeditLength( 0 ), preeditCPos(-1), preeditSelLen(0)
+#endif
+    {}
 
     bool frame;
     QLineEdit::EchoMode mode;
@@ -108,8 +116,19 @@ struct QLineEditPrivate {
     QPoint dnd_startpos;
     bool dnd_primed;
     QChar passwordChar;
+#ifndef QT_NO_QWS_IM
+    int preeditStart;
+    int preeditLength;
+    int preeditCPos;
+    int preeditSelLen;
+#endif
 };
 
+static bool inBlinkOn = FALSE;
+
+#ifndef QT_NO_QWS_IM
+extern QBrush *qt_im_compose_background;
+#endif
 
 // REVISED: arnt
 /*!
@@ -232,9 +251,9 @@ void QLineEdit::init()
     d = new QLineEditPrivate( this );
     connect( &d->blinkTimer, SIGNAL(timeout()),
 	     this, SLOT(blinkSlot()) );
-#ifndef QT_NO_DRAGANDDROP
     connect( &d->dragTimer, SIGNAL(timeout()),
 	     this, SLOT(dragScrollSlot()) );
+#ifndef QT_NO_DRAGANDDROP
     connect( &d->dndTimer, SIGNAL(timeout()),
 	     this, SLOT(doDrag()) );
 #endif
@@ -332,6 +351,11 @@ void QLineEdit::deselect()
 
 QString QLineEdit::text() const
 {
+#ifndef QT_NO_QWS_IM
+    if ( composeMode() ) 
+	return  tbuf.left( d->preeditStart ) 
+	    + tbuf.mid( d->preeditStart + d->preeditLength );
+#endif
     return tbuf;
 }
 
@@ -567,10 +591,16 @@ void QLineEdit::keyPressEvent( QKeyEvent *e )
 #endif
 #endif
 	case Key_Right:
-	    cursorWordForward( e->state() & ShiftButton );
+	    if ( echoMode() == Normal )
+		cursorWordForward( e->state() & ShiftButton );
+	    else
+		end( e->state() & ShiftButton );
 	    break;
 	case Key_Left:
-	    cursorWordBackward( e->state() & ShiftButton );
+	    if ( echoMode() == Normal )
+		cursorWordBackward( e->state() & ShiftButton );
+	    else
+		home( e->state() & ShiftButton );
 	    break;
 	case Key_Z:
 	    if ( !d->readonly )
@@ -716,6 +746,12 @@ void QLineEdit::paintEvent( QPaintEvent *e )
 	int markBegin = minMark();
 	int markEnd = maxMark();
 
+#ifndef QT_NO_QWS_IM
+	if ( d->preeditLength > 0 && d->preeditSelLen > 0 ) {
+	    markBegin = d->preeditStart + d->preeditCPos;
+	    markEnd = markBegin + d->preeditSelLen;
+	}
+#endif
 	p.fillRect( 0, 0, width(), height(), bg );
 
 	QString display = displayText();
@@ -723,12 +759,33 @@ void QLineEdit::paintEvent( QPaintEvent *e )
 	QString marked = display.mid( markBegin, markEnd - markBegin );
 	QString after = display.mid( markEnd, display.length() );
 
+
 	int y = (d->pm->height() + fm.height())/2 - fm.descent() - 1 ;
 
 	int x = offset + 2;
 	int w;
 
+
 	w = fm.width( before );
+
+
+#ifndef QT_NO_QWS_IM
+	//We're lazy and also fill in the about-to-be-selected part
+	if ( composeMode() ) {
+	    QString beforePre = display.mid( 0, d->preeditStart );
+	    QString preed = display.mid( d->preeditStart, d->preeditLength );
+	    int bw = fm.width( beforePre );
+	    int pw = fm.width( preed );
+
+	    if ( !qt_im_compose_background ) {
+		p.setPen( g.text() );
+		p.drawLine( x+bw, y+2, x+bw+pw, y+2 ); 
+	    } else {
+		p.fillRect( x+bw, 0, pw, height(), *qt_im_compose_background );
+	    }
+	}
+#endif
+
 	if ( x < d->pm->width() && x + w >= 0 ) {
 	    p.setPen( g.text() );
 	    p.drawText( x, y, before );
@@ -744,6 +801,8 @@ void QLineEdit::paintEvent( QPaintEvent *e )
 	}
 	x += w;
 
+
+
 	w = fm.width( after );
 	if ( x < d->pm->width() && x + w >= 0 ) {
 	    p.setPen( g.text() );
@@ -789,7 +848,8 @@ void QLineEdit::paintEvent( QPaintEvent *e )
 	}
 	// Now is the optimal time to set this - all the repaint-minimization
 	// then also reduces the number of calls to setMicroFocusHint().
-	setMicroFocusHint( curXPos, curYTop, 1, curYBot-curYTop+1 );
+	if ( inBlinkOn )
+	    setMicroFocusHint( curXPos, curYTop, 1, curYBot-curYTop+1 );
     } else {
 	delete d->pm;
 	d->pm = 0;
@@ -814,57 +874,207 @@ void QLineEdit::resizeEvent( QResizeEvent * )
 */
 bool QLineEdit::event( QEvent * e )
 {
-    if ( e->type() == QEvent::AccelOverride && !d->readonly ) {
-	QKeyEvent* ke = (QKeyEvent*) e;
-	if ( ke->state() == NoButton ) {
-	    if ( ke->key() < Key_Escape ) {
-		ke->accept();
-	    } else {
+    switch ( e->type() ) {
+    case QEvent::AccelOverride:
+	if (!d->readonly ) {
+	    QKeyEvent* ke = (QKeyEvent*) e;
+	    if ( ke->state() == NoButton ) {
+		if ( ke->key() < Key_Escape ) {
+		    ke->accept();
+		} else {
+		    switch ( ke->key() ) {
+		    case Key_Delete:
+		    case Key_Home:
+		    case Key_End:
+		    case Key_Backspace:
+			ke->accept();
+		    default:
+			break;
+		    }
+		}
+	    } else if ( ke->state() & ControlButton ) {
 		switch ( ke->key() ) {
-  		case Key_Delete:
-  		case Key_Home:
-  		case Key_End:
-  		case Key_Backspace:
-  		    ke->accept();
- 		default:
-  		    break;
-  		}
-	    }
-	} else if ( ke->state() & ControlButton ) {
-	    switch ( ke->key() ) {
-// Those are too frequently used for application functionality
-/*	    case Key_A:	
-	    case Key_B:
-	    case Key_D:
-	    case Key_E:
-	    case Key_F:
-	    case Key_H:
-	    case Key_K:
-*/
-	    case Key_C:
-	    case Key_V:
-	    case Key_X:	    
-	    case Key_Y:
-	    case Key_Z:	    
-	    case Key_Left:
-	    case Key_Right:
+		    // Those are too frequently used for application functionality
+		    /*	    case Key_A:	
+			    case Key_B:
+			    case Key_D:
+			    case Key_E:
+			    case Key_F:
+			    case Key_H:
+			    case Key_K:
+		    */
+		case Key_C:
+		case Key_V:
+		case Key_X:	    
+		case Key_Y:
+		case Key_Z:	    
+		case Key_Left:
+		case Key_Right:
 #if defined (_WS_WIN_)
-	    case Key_Insert:
+		case Key_Insert:
 #endif
-		ke->accept();
-	    default:
-		break;
+		    ke->accept();
+		default:
+		    break;
+		}
 	    }
 	}
+	break;
+#ifndef QT_NO_QWS_IM
+    case QEvent::IMStart: {
+	QIMEvent *i = (QIMEvent *) e;
+	imStartEvent(i);
+	if (! i->isAccepted())
+	    return FALSE;
     }
+    break;
+
+    case QEvent::IMCompose: {
+	QIMEvent *i = (QIMEvent *) e;
+	imComposeEvent(i);
+	if (! i->isAccepted())
+	    return FALSE;
+    }
+    break;
+
+    case QEvent::IMEnd: {
+	QIMEvent *i = (QIMEvent *) e;
+	imEndEvent(i);
+	if (! i->isAccepted())
+	    return FALSE;
+    }
+    break;
+#endif //QT_NO_QWS_IM
+    default:
+	break;
+
+    } 
+
     return QWidget::event( e );
 }
 
 
+#ifndef QT_NO_QWS_IM
+//### ??? Undo/Redo for IM events 
+
+void QLineEdit::imStartEvent( QIMEvent *e )
+{
+
+
+    /*
+    setMicroFocusHint( d->cursor->x() - d->offset, d->cursor->y(), 0,
+	d->cursor->paragraph()->rect().height(), TRUE );
+    */
+    //remove selected text:
+    insert(""); //####
+    d->preeditStart = cursorPos;
+    d->preeditLength = 0;
+    e->accept();
+
+
+//    qDebug( "%p QLineEdit::imStartEvent txt %s cp %d sl %d cursor %d", this,
+//	      e->text().latin1(), 
+//	      e->cursorPos(), e->selectionLength(), cursorPos );
+    d->mousePressed = FALSE; //### stop any dragging
+
+}
+
+
+void QLineEdit::imComposeEvent( QIMEvent *e )
+{
+    if ( isReadOnly() )
+	return;
+//    qDebug( "QLineEdit::imComposeEvent txt %s cp %d sl %d", 
+//    e->text().latin1(), 
+//	      e->cursorPos(), e->selectionLength() );
+    QString test( tbuf);
+
+    
+    test.remove( d->preeditStart, d->preeditLength );
+	
+    d->preeditLength = e->text().length();
+    test.insert( d->preeditStart, e->text() );
+
+    d->preeditCPos = e->cursorPos();
+    d->preeditSelLen = e->selectionLength();
+
+
+    cursorPos = d->preeditStart + e->cursorPos();
+    tbuf = test;
+    d->pmDirty = TRUE;
+    d->offsetDirty = TRUE;
+    blinkOn();
+
+    e->accept();
+}
+
+/*! \reimp
+ */
+void QLineEdit::imEndEvent( QIMEvent *e )
+{
+//    qDebug( "%p QLineEdit::imEndEvent txt %s cp %d sl %d", this,
+//	      e->text().latin1(), 
+//	      e->cursorPos(), e->selectionLength() );
+
+    QString test( tbuf);
+
+#if 0
+    d->ignoreUndoWithDel = TRUE;
+    if ( d->undo && ( (d->needundo && !d->ignoreUndoWithDel) || hasMarkedText() ) ) {
+	if ( d->undoList.isEmpty() || d->undoList.last().str != tbuf ) {
+	    d->undoList += QLineEditUndoItem(tbuf, cursorPos );
+	    d->redoList.clear();
+	}
+    }
+#endif
+    
+    test.remove( d->preeditStart, d->preeditLength );
+    test.insert( d->preeditStart, e->text() );
+    int newcp = d->preeditStart+e->text().length();
+
+    d->preeditStart = -1;
+    d->preeditLength = 0;
+
+    d->preeditCPos = -1;
+    d->preeditSelLen = 0;
+
+    if ( isReadOnly() )
+	return;
+
+    //#########  Validation doesn't work!!!!
+    //######### textChanged() probably doesn't work!
+    //######### alignment doesn't work
+    validateAndSet( test, newcp , newcp, newcp ); //### not a perfect fit, uses tbuf...
+
+    d->pmDirty = TRUE; //###
+    blinkOn();
+
+    emit textChanged( tbuf );
+    
+    e->accept();
+}
+
+bool QLineEdit::composeMode() const
+{
+    return d->preeditStart >= 0;
+}
+
+#endif //QT_NO_QWS_IM
+
 /*! \reimp
 */
 void QLineEdit::mousePressEvent( QMouseEvent *e )
 {
+#if defined( _WS_QWS_ ) && !defined( QT_NO_QWS_IM )    
+    if ( composeMode() ) {
+	int clickPos = xPosToCursorPos( e->pos().x() ) - d->preeditStart;
+	if ( clickPos >= 0 && clickPos <= d->preeditLength ) 
+	    QPaintDevice::qwsDisplay()->sendIMMouseEvent( clickPos, TRUE ); 
+	
+	return;
+    }
+#endif
+
     d->dnd_startpos = e->pos();
     d->dnd_primed = FALSE;
 #ifndef QT_NO_POPUPMENU
@@ -968,6 +1178,13 @@ void QLineEdit::doDrag()
 */
 void QLineEdit::mouseMoveEvent( QMouseEvent *e )
 {
+#ifndef QT_NO_QWS_IM
+    if ( composeMode() ) {
+
+	return;
+    }
+#endif
+
 #ifndef QT_NO_DRAGANDDROP
     if ( d->dndTimer.isActive() ) {
 	d->dndTimer.stop();
@@ -1009,6 +1226,16 @@ void QLineEdit::mouseMoveEvent( QMouseEvent *e )
 */
 void QLineEdit::mouseReleaseEvent( QMouseEvent * e )
 {
+#if defined( _WS_QWS_ ) && !defined ( QT_NO_QWS_IM )
+    if ( composeMode() ) {
+	int clickPos = xPosToCursorPos( e->pos().x() ) - d->preeditStart;
+	if ( clickPos >= 0 && clickPos <= d->preeditLength ) 
+	    QPaintDevice::qwsDisplay()->sendIMMouseEvent( clickPos, FALSE ); 
+	
+	return;
+    }
+#endif
+
     dragScrolling = FALSE;
     d->dnd_primed = FALSE;
     if ( d->dndTimer.isActive() ) {
@@ -1063,6 +1290,12 @@ void QLineEdit::mouseReleaseEvent( QMouseEvent * e )
 */
 void QLineEdit::mouseDoubleClickEvent( QMouseEvent * )
 {
+#ifndef QT_NO_QWS_IM
+    if ( composeMode() ) {
+	return;
+    }
+#endif
+
     d->inDoubleClick = TRUE;
     dragScrolling = FALSE;
     if ( echoMode() == Password )
@@ -1699,8 +1932,8 @@ void QLineEdit::insert( const QString &newText )
     }
     test.insert( cp, t );
     int ncp = QMIN( cp+t.length(), (uint)maxLength() );
-    blinkOn();
     validateAndSet( test, ncp, ncp, ncp );
+    blinkOn();
 }
 
 
@@ -1903,16 +2136,32 @@ void QLineEdit::updateOffset()
     int old = offset;
 
     if ( textWidth > w ) {
+
+	static const int rightDist = 2*fm.width("x");
+	static const int leftMargin = 2;
 	// may need to scroll.
 	QString dt = displayText();
-	dt += QString::fromLatin1( "  " );
-	dt = dt.left( cursorPos + 2 );
-	if ( cursorPos < 3 )
-	    offset = 0;
-	else if ( fm.width( dt.left( cursorPos - 2 ) ) + offset < 0 )
-	    offset = -fm.width( dt.left( cursorPos - 2 ) );
-	else if ( fm.width( dt ) + offset > w )
-	    offset = w - fm.width( dt );
+
+	int leftPos = fm.width( dt.left( QMAX( cursorPos-leftMargin, 0 ) ) );
+	int rightPos;
+
+#if !defined(QT_NO_QWS_IM)
+	if ( d->preeditSelLen )
+	    rightPos = fm.width( dt.left( cursorPos+d->preeditSelLen+1 ) );
+	else
+#endif
+	    rightPos = fm.width( dt.left( cursorPos ) ) + rightDist;
+	
+	if ( rightPos + offset > w )
+	    offset = w - rightPos;
+	if ( leftPos + offset < 0 )
+	    offset = -leftPos;
+	
+	//make sure we show as much as possible of the text:
+
+	if ( textWidth + offset < w )
+	    offset = w - textWidth;
+
     } else {
 	if ( textWidth < 5 ) {
 	    // nothing is to be drawn.  okay.
@@ -1969,7 +2218,9 @@ void QLineEdit::blinkOn()
 	return;
 
     d->blinkTimer.start( cursorOn?QApplication::cursorFlashTime() / 2 : 0, TRUE );
+    inBlinkOn = TRUE;
     blinkSlot();
+    inBlinkOn = FALSE;
 }
 
 
