@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/widgets/qlistbox.cpp   2.3.2   edited 2001-10-18 $
+** $Id: qt/src/widgets/qlistbox.cpp   2.3.7   edited 2002-01-30 $
 **
 ** Implementation of QListBox widget class
 **
@@ -502,7 +502,9 @@ QListBoxPixmap::QListBoxPixmap( QListBox* listbox, const QPixmap & pix, const QS
 
 void QListBoxPixmap::paint( QPainter *painter )
 {
-    painter->drawPixmap( 3, 0, pm );
+    // 2002/1/8 move the pixmap down two pixels as it interferes highlighted items
+    // in QComboBox...
+    painter->drawPixmap( 3, 2, pm );
     if ( !text().isEmpty() ) {
 	QFontMetrics fm = painter->fontMetrics();
 	int yPos;			// vertical text position
@@ -2435,9 +2437,6 @@ void QListBox::setSelected( QListBoxItem * item, bool select )
 
 bool QListBox::isSelected( int i ) const
 {
-    if ( selectionMode() == Single )
-	return i == currentItem();
-
     QListBoxItem * lbi = item( i );
     if ( !lbi )
 	return FALSE; // should not happen
