@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qmenubar.cpp   2.3.2   edited 2001-06-29 $
+** $Id: qt/src/widgets/qmenubar.cpp   2.3.7   edited 2003-02-20 $
 **
 ** Implementation of QMenuBar class
 **
@@ -48,6 +48,7 @@
 #include "qapplication.h"
 #include "qguardedptr.h"
 #include "qlayout.h"
+#include "qdatetime.h"
 #include <ctype.h>
 
 class QMenuDataData {
@@ -58,6 +59,7 @@ public:
     int aInt;
 };
 
+static QTime moveActiveTime;
 
 // NOT REVISED
 /*!
@@ -298,7 +300,7 @@ void QMenuBar::menuContentsChanged()
 	    ( (QMainWindow*)parent() )->update();
 	}
 #endif
-#ifndef QT_NO_LAYOUT	
+#ifndef QT_NO_LAYOUT
 	if ( parentWidget() && parentWidget()->layout() )
 	    parentWidget()->layout()->activate();
 #endif
@@ -324,20 +326,24 @@ void QMenuBar::menuStateChanged()
 
 void QMenuBar::menuInsPopup( QPopupMenu *popup )
 {
+#ifndef QT_NO_POPUPMENU
     popup->parentMenu = this;			// set parent menu
     connect( popup, SIGNAL(activatedRedirect(int)),
 	     SLOT(subActivated(int)) );
     connect( popup, SIGNAL(highlightedRedirect(int)),
 	     SLOT(subHighlighted(int)) );
+#endif
 }
 
 void QMenuBar::menuDelPopup( QPopupMenu *popup )
 {
+#ifndef QT_NO_POPUPMENU
     popup->parentMenu = 0;
     popup->disconnect( SIGNAL(activatedRedirect(int)), this,
 		       SLOT(subActivated(int)) );
     popup->disconnect( SIGNAL(highlightedRedirect(int)), this,
 		       SLOT(subHighlighted(int)) );
+#endif
 }
 
 void QMenuBar::frameChanged()
@@ -367,7 +373,15 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	return FALSE;
     }
 
-    if ( style() != WindowsStyle ||
+    if ( waitforalt && event->type() == QEvent::FocusOut ) {
+	// some window systems/managers use alt/meta as accelerator keys
+	// for switching between windows/desktops/etc.  If the focus
+	// widget gets unfocused, then we need to stop waiting for alt
+	// NOTE: this event came from the real focus widget, so we don't
+	// need to touch the event filters
+	waitforalt = 0;
+	return FALSE;
+    } else if ( style() != WindowsStyle ||
 	 !isVisible() ||
 	 !object->isWidgetType() ||
 	 !( event->type() == QEvent::Accel ||
@@ -525,6 +539,7 @@ void QMenuBar::goodbye( bool cancelled )
 
 void QMenuBar::openActPopup()
 {
+#ifndef QT_NO_POPUPMENU
     if ( actItem < 0 )
 	return;
 //     setWindowsAltMode( FALSE, actItem );
@@ -558,6 +573,7 @@ void QMenuBar::openActPopup()
     popup->snapToMouse = FALSE;
     popup->popup( pos );
     popup->snapToMouse = TRUE;
+#endif
 }
 
 /*!
@@ -567,6 +583,7 @@ void QMenuBar::openActPopup()
 
 void QMenuBar::hidePopups()
 {
+#ifndef QT_NO_POPUPMENU
     QMenuItemListIt it(*mitems);
     register QMenuItem *mi;
     while ( (mi=it.current()) ) {
@@ -575,6 +592,7 @@ void QMenuBar::hidePopups()
 	    mi->popup()->hide();
 	}
     }
+#endif
 }
 
 
@@ -898,8 +916,10 @@ void QMenuBar::mousePressEvent( QMouseEvent *e )
 	return;
     mouseBtDn = TRUE;				// mouse button down
     int item = itemAtPos( e->pos() );
-    if ( item == actItem && popupvisible )
-	toggleclose = 1;
+    if ( item == actItem && popupvisible ) {
+	if ( moveActiveTime.elapsed() > 50 )
+	    toggleclose = 1;
+    }
     setActiveItem( item, TRUE, FALSE );
 }
 
@@ -943,8 +963,11 @@ void QMenuBar::mouseMoveEvent( QMouseEvent *e )
 	setActiveItem( item, FALSE, FALSE );
 	return;
     }
-    if ( item != actItem && item >= 0  && ( popupvisible || mouseBtDn ) )
+    if ( item != actItem && item >= 0  && ( popupvisible || mouseBtDn ) ) {
 	setActiveItem( item, TRUE, FALSE );
+	if ( popupvisible )
+	    moveActiveTime.start();
+    }
 }
 
 
@@ -1116,13 +1139,16 @@ void QMenuBar::setActiveItem( int i, bool show, bool activate_first_item )
     if ( actItem < 0 || !popupvisible || !mi  )
 	return;
 
+#ifndef QT_NO_POPUPMENU
     QPopupMenu *popup = mi->popup();
     if ( popup ) {
 	emit highlighted( mi->id() );
 	openActPopup();
 	if ( activate_first_item )
 	    popup->setFirstItemActive();
-    } else {				// not a popup
+    } else
+#endif
+    {				// not a popup
 	goodbye( FALSE );
 	if ( mi->signal() )			// activate signal
 	    mi->signal()->activate();
@@ -1192,6 +1218,7 @@ void QMenuBar::setupAccelerators()
 		autoaccel->insertItem( i, mi->id() );
 	    }
 	}
+#ifndef QT_NO_POPUPMENU
 	if ( mi->popup() ) {
 	    // reuse
 	    QPopupMenu* popup = mi->popup();
@@ -1202,6 +1229,7 @@ void QMenuBar::setupAccelerators()
 	    if ( !popup->isEnabled() )
 		popup->enableAccel( FALSE );
 	}
+#endif
     }
 }
 #endif
