@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qmotifplusstyle.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/src/widgets/qmotifplusstyle.cpp   2.3.7   edited 2002-10-28 $
 **
 ** Implementation of QMotifPlusStyle class
 **
@@ -260,8 +260,10 @@ void QMotifPlusStyle::polish(QWidget *widget)
 	((QMenuBar *) widget)->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
 #endif
 
+#ifndef QT_NO_TOOLBAR
     if (widget->inherits("QToolBar"))
 	widget->layout()->setMargin(2);
+#endif
 
     if (useHoveringHighlight) {
 	if (widget->inherits("QButton") ||
@@ -713,15 +715,18 @@ void QMotifPlusStyle::drawPopupMenuItem(QPainter *p, bool checkable, int maxpmw,
 	return;
 
     if ( mi->isChecked() ) {
+#ifndef QT_NO_ICONSET
 	if ( mi->iconSet() ) {
 	    qDrawShadePanel( p, x+2, y+2, checkcol, h-2*2,
 			     g, TRUE, 1, &g.brush( QColorGroup::Midlight ) );
 	}
+#endif
     } else if ( !act ) {
 	p->fillRect(x+2, y+2, checkcol, h-2*2,
 		    g.brush( QColorGroup::Button ));
     }
 
+#ifndef QT_NO_ICONSET
     if ( mi->iconSet() ) {		// draw iconset
 	QIconSet::Mode mode = (enabled) ? QIconSet::Normal : QIconSet::Disabled;
 
@@ -741,7 +746,9 @@ void QMotifPlusStyle::drawPopupMenuItem(QPainter *p, bool checkable, int maxpmw,
 	p->setPen( itemg.text() );
 	p->drawPixmap( pmr.topLeft(), pixmap );
 
-    } else  if (checkable) {
+    } else
+#endif
+    if (checkable) {
 	int mw = checkcol;
 	int mh = h - 4;
 
