@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qscrollview.cpp   2.3.2   edited 2001-03-02 $
+** $Id: qt/src/widgets/qscrollview.cpp   2.3.7   edited 2001-10-23 $
 **
 ** Implementation of QScrollView class
 **
@@ -2383,7 +2383,7 @@ void QScrollView::stopDragAutoScroll()
 */
 void QScrollView::doDragAutoScroll()
 {
-    QPoint p = viewport()->mapFromGlobal( QCursor::pos() );
+    QPoint p = d->viewport.mapFromGlobal( QCursor::pos() );
 
     if ( d->autoscroll_accel-- <= 0 && d->autoscroll_time ) {
 	d->autoscroll_accel = initialScrollAccel;
