@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qtabbar.cpp   2.3.2   edited 2001-03-13 $
+** $Id: qt/src/widgets/qtabbar.cpp   2.3.7   edited 2002-10-28 $
 **
 ** Implementation of QTab and QTabBar classes
 **
@@ -144,8 +144,10 @@ struct QTabPrivate {
     QAccel * a;
 #endif
     QTabBar::Shape s;
+#ifndef QT_NO_TOOLBUTTON
     QToolButton* rightB;
     QToolButton* leftB;
+#endif
     bool  scrolls;
 };
 
@@ -177,12 +179,14 @@ QTabBar::QTabBar( QWidget * parent, const char *name )
 #endif
     d->s = RoundedAbove;
     d->scrolls = FALSE;
+#ifndef QT_NO_TOOLBUTTON
     d->leftB = new QToolButton( LeftArrow, this );
     connect( d->leftB, SIGNAL( clicked() ), this, SLOT( scrollTabs() ) );
     d->leftB->hide();
     d->rightB = new QToolButton( RightArrow, this );
     connect( d->rightB, SIGNAL( clicked() ), this, SLOT( scrollTabs() ) );
     d->rightB->hide();
+#endif
     l = new QList<QTab>;
     lstatic = new QList<QTab>;
     lstatic->setAutoDelete( TRUE );
@@ -367,7 +371,11 @@ QSize QTabBar::sizeHint() const
 
 QSize QTabBar::minimumSizeHint() const
 {
-    return QSize( d->rightB->sizeHint().width() * 2 + 75, sizeHint().height() );
+    return QSize(
+#ifndef QT_NO_TOOLBUTTON
+	d->rightB->sizeHint().width() * 2 +
+#endif
+	75, sizeHint().height() );
 }
 
 /*!\reimp
@@ -908,9 +916,11 @@ void QTabBar::focusOutEvent( QFocusEvent * )
 */
 void QTabBar::resizeEvent( QResizeEvent * )
 {
+#ifndef QT_NO_TOOLBUTTON
     const int arrowWidth = 16;
     d->rightB->setGeometry( width() - arrowWidth, 0, arrowWidth, height() );
     d->leftB->setGeometry( width() - 2*arrowWidth, 0, arrowWidth, height() );
+#endif
     layoutTabs();
     updateArrowButtons();
     makeVisible( tab( currentTab() ));
@@ -918,6 +928,7 @@ void QTabBar::resizeEvent( QResizeEvent * )
 
 void QTabBar::scrollTabs()
 {
+#ifndef QT_NO_TOOLBUTTON
     QTab* left = 0;
     QTab* right = 0;
     for ( QTab* t = lstatic->first(); t; t = lstatic->next() ) {
@@ -931,10 +942,12 @@ void QTabBar::scrollTabs()
 	makeVisible( left );
     else  if ( sender() == d->rightB )
 	makeVisible( right );
+#endif
 }
 
 void QTabBar::makeVisible( QTab* tab  )
 {
+#ifndef QT_NO_TOOLBUTTON
     bool tooFarLeft = ( tab && tab->r.left() < 0 );
     bool tooFarRight = ( tab && tab->r.right() >= d->leftB->x() );
 
@@ -959,10 +972,12 @@ void QTabBar::makeVisible( QTab* tab  )
 
 
     update();
+#endif
 }
 
 void QTabBar::updateArrowButtons()
 {
+#ifndef QT_NO_TOOLBUTTON
     bool b = lstatic->last() &&	( lstatic->last()->r.right() > width() );
     d->scrolls = b;
     if ( d->scrolls ) {
@@ -974,5 +989,6 @@ void QTabBar::updateArrowButtons()
 	d->leftB->hide();
 	d->rightB->hide();
     }
+#endif
 }
 #endif
