@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id: qt/src/widgets/qtableview.cpp   2.3.2   edited 2001-08-15 $
+** $Id: qt/src/widgets/qtableview.cpp   2.3.7   edited 2002-12-11 $
 **
 ** Implementation of QTableView class
 **
@@ -1436,7 +1436,7 @@ void QTableView::paintEvent( QPaintEvent *e )
 
 /*!\reimp
 */
-void QTableView::resizeEvent( QResizeEvent * )
+void QTableView::resizeEvent( QResizeEvent * /* e */)
 {
     updateScrollBars( horValue | verValue | horSteps | horGeometry | horRange |
 		      verSteps | verGeometry | verRange );
@@ -1445,6 +1445,9 @@ void QTableView::resizeEvent( QResizeEvent * )
     int maxX = QMIN( xOffs, maxXOffset() );			// ### can be slow
     int maxY = QMIN( yOffs, maxYOffset() );
     setOffset( maxX, maxY );
+
+    if ( testWState(WState_AutoMask) )
+	updateMask();
 }
 
 
