@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qtoolbutton.cpp   2.3.2   edited 2001-07-09 $
+** $Id: qt/src/widgets/qtoolbutton.cpp   2.3.7   edited 2002-10-28 $
 **
 ** Implementation of QToolButton class
 **
@@ -58,9 +58,11 @@ class QToolButtonPrivate
 {
     // ### add tool tip magic here
 public:
+#ifndef QT_NO_POPUPMENU
     QGuardedPtr<QPopupMenu> popup;
     QTimer* popupTimer;
     int delay;
+#endif
     bool autoraise, repeat;
     Qt::ArrowType arrow;
 };
@@ -119,6 +121,7 @@ QToolButton::QToolButton( QWidget * parent, const char *name )
     : QButton( parent, name )
 {
     init();
+#ifndef QT_NO_TOOLBAR
     if ( parent && parent->inherits( "QToolBar" ) ) {
 	setAutoRaise( TRUE );
 	QToolBar* tb = (QToolBar*)parent;
@@ -132,7 +135,9 @@ QToolButton::QToolButton( QWidget * parent, const char *name )
 	} else {
 	    setUsesBigPixmap( FALSE );
 	}
-    } else {
+    } else
+#endif
+    {
 	setUsesBigPixmap( FALSE );
     }
 }
@@ -163,9 +168,11 @@ QToolButton::QToolButton( ArrowType type, QWidget *parent, const char *name )
 void QToolButton::init()
 {
     d = new QToolButtonPrivate;
+#ifndef QT_NO_POPUPMENU
     d->delay = 600;
     d->popup = 0;
     d->popupTimer = 0;
+#endif
     d->autoraise = FALSE;
     d->arrow = LeftArrow;
     bpID = bp.serialNumber();
@@ -184,6 +191,7 @@ void QToolButton::init()
 }
 
 
+#ifndef QT_NO_TOOLBAR
 /*!  Constructs a tool button that is a child of \a parent (which must be
   a QToolBar) and named \a name.
 
@@ -266,14 +274,17 @@ QToolButton::QToolButton( const QIconSet& iconSet, const QString &textLabel,
     }
 #endif
 }
+#endif
 
 
 /*! Destroys the object and frees any allocated resources. */
 
 QToolButton::~QToolButton()
 {
+#ifndef QT_NO_POPUPMENU
     d->popupTimer = 0;
     d->popup = 0;
+#endif
     delete d;
     delete s;
     delete son;
@@ -735,6 +746,7 @@ QIconSet QToolButton::iconSet( bool on ) const
 }
 
 
+#ifndef QT_NO_POPUPMENU
 /*!
   Associates the popup menu \a popup with this toolbutton.
 
@@ -785,10 +797,12 @@ void QToolButton::popupTimerDone()
 	setAutoRepeat( FALSE );
 	bool horizontal = TRUE;
 	bool topLeft = TRUE;
+#ifndef QT_NO_TOOLBAR
 	if ( parentWidget() && parentWidget()->inherits("QToolBar") ) {
 	    if ( ( (QToolBar*) parentWidget() )->orientation() == Vertical )
 		horizontal = FALSE;
 	}
+#endif
 	if ( horizontal ) {
 	    if ( topLeft ) {
 		if ( mapToGlobal( QPoint( 0, rect().bottom() ) ).y() + d->popup->sizeHint().height() <= qApp->desktop()->height() )
@@ -843,7 +857,7 @@ int QToolButton::popupDelay() const
 {
     return d->delay;
 }
-
+#endif
 
 /*!
   Enables or disables auto-raising according to \a enable.
