@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/widgets/qwindowsstyle.cpp   2.3.2   edited 2001-01-26 $
+** $Id: qt/src/widgets/qwindowsstyle.cpp   2.3.7   edited 2003-02-14 $
 **
 ** Implementation of Windows-like style class
 **
@@ -1162,6 +1162,8 @@ int QWindowsStyle::extraPopupMenuItemWidth( bool checkable, int maxpmw, QMenuIte
     w += windowsRightBorder; // windows has a strange wide border on the right side
 
     return w;
+#else
+    return 0;
 #endif
 }
 
@@ -1184,6 +1186,8 @@ int QWindowsStyle::popupMenuItemHeight( bool /*checkable*/, QMenuItem* mi, const
     if ( mi->custom() )
 	h = QMAX( h, mi->custom()->sizeHint().height() + 2*motifItemVMargin + 2*motifItemFrame );
     return h;
+#else
+    return 0;
 #endif
 }
 
