@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/src/workspace/qworkspace.cpp   2.3.2   edited 2001-10-17 $
+** $Id: qt/src/workspace/qworkspace.cpp   2.3.7   edited 2002-10-28 $
 **
 ** Implementation of the QWorkspace class
 **
@@ -1253,7 +1253,6 @@ void QWorkspace::showMaximizeControls()
 	QHBoxLayout* l = new QHBoxLayout( d->maxcontrols,
 					  d->maxcontrols->frameWidth(), 0 );
 	QToolButton* iconB = new QToolButton( d->maxcontrols, "iconify" );
-	QToolTip::add( iconB, tr( "Minimize" ) );
 	l->addWidget( iconB );
 	iconB->setFocusPolicy( NoFocus );
 	iconB->setIconSet( QPixmap( (const char **)minimize_xpm ));
@@ -1261,7 +1260,6 @@ void QWorkspace::showMaximizeControls()
 	connect( iconB, SIGNAL( clicked() ),
 		 this, SLOT( minimizeActiveWindow() ) );
 	QToolButton* restoreB = new QToolButton( d->maxcontrols, "restore" );
-	QToolTip::add( restoreB, tr( "Restore Down" ) );
 	l->addWidget( restoreB );
 	restoreB->setFocusPolicy( NoFocus );
 	restoreB->setIconSet( QPixmap( (const char **)normalize_xpm ));
@@ -1271,7 +1269,6 @@ void QWorkspace::showMaximizeControls()
 
 	l->addSpacing( 2 );
 	QToolButton* closeB = new QToolButton( d->maxcontrols, "close" );
-	QToolTip::add( closeB, tr( "Close" ) );
 	l->addWidget( closeB );
 	closeB->setFocusPolicy( NoFocus );
 	closeB->setIconSet( QPixmap( (const char **)close_xpm ) );
@@ -1281,6 +1278,12 @@ void QWorkspace::showMaximizeControls()
 
 	d->maxcontrols->setFixedSize( 3* BUTTON_WIDTH+2+2*d->maxcontrols->frameWidth(),
 				      BUTTON_HEIGHT+2*d->maxcontrols->frameWidth());
+
+#ifndef QT_NO_TOOLTIP
+	QToolTip::add( iconB, tr( "Minimize" ) );
+	QToolTip::add( restoreB, tr( "Restore Down" ) );
+	QToolTip::add( closeB, tr( "Close" ) );
+#endif
     }
 
     if ( d->controlId == -1 || b->indexOf( d->controlId ) == -1 ) {
@@ -1658,7 +1661,6 @@ QWorkspaceChildTitleBar::QWorkspaceChildTitleBar (QWorkspace* w, QWidget* win, Q
 
     titleL = new QWorkspaceChildTitleLabel( this, "__workspace_child_title_bar" );
     closeB = new QToolButton( this, "close" );
-    QToolTip::add( closeB, tr( "Close" ) );
     closeB->setFocusPolicy( NoFocus );
     closeB->setIconSet( QPixmap( (const char **)close_xpm ) );
     connect( closeB, SIGNAL( clicked() ),
@@ -1666,7 +1668,6 @@ QWorkspaceChildTitleBar::QWorkspaceChildTitleBar (QWorkspace* w, QWidget* win, Q
     closeB->installEventFilter( this );
 
     maxB = new QToolButton( this, "maximize" );
-    QToolTip::add( maxB, tr( "Maximize" ) );
     maxB->setFocusPolicy( NoFocus );
     maxB->setIconSet( QPixmap( (const char **)maximize_xpm ));
     connect( maxB, SIGNAL( clicked() ),
@@ -1678,7 +1679,6 @@ QWorkspaceChildTitleBar::QWorkspaceChildTitleBar (QWorkspace* w, QWidget* win, Q
     iconB->installEventFilter( this );
 
     shadeB = new QToolButton( this, "shade" );
-    QToolTip::add( shadeB, tr( "Roll up" ) );
     shadeB->setIconSet( QPixmap( (const char **)shade_xpm ) );
     shadeB->setFocusPolicy( NoFocus );
     connect( shadeB, SIGNAL( clicked() ),
@@ -1691,6 +1691,12 @@ QWorkspaceChildTitleBar::QWorkspaceChildTitleBar (QWorkspace* w, QWidget* win, Q
     iconL->setFocusPolicy( NoFocus );
     iconL->installEventFilter( this );
 
+#ifndef QT_NO_TOOLTIP
+    QToolTip::add( closeB, tr( "Close" ) );
+    QToolTip::add( maxB, tr( "Maximize" ) );
+    QToolTip::add( shadeB, tr( "Roll up" ) );
+#endif
+
     if ( window ) {
 	if ( !window->testWFlags( WStyle_Tool ) ) {
 #ifdef _WS_WIN_
@@ -1753,13 +1759,16 @@ QWorkspaceChildTitleBar::QWorkspaceChildTitleBar (QWorkspace* w, QWidget* win, Q
 
     if ( imode ) {
 	iconB->setIconSet( QPixmap( (const char **)normalizeup_xpm ) );
+#ifndef QT_NO_TOOLTIP
 	QToolTip::add( iconB, tr( "Restore Up" ) );
+#endif
 	connect( iconB, SIGNAL( clicked() ),
 		 this, SIGNAL( doNormal() ) );
-    }
-    else {
+    } else {
 	iconB->setIconSet( QPixmap( (const char **)minimize_xpm ) );
+#ifndef QT_NO_TOOLTIP
 	QToolTip::add( iconB, tr( "Minimize" ) );
+#endif
 	connect( iconB, SIGNAL( clicked() ),
 		 this, SIGNAL( doMinimize() ) );
     }
@@ -2475,16 +2484,22 @@ void QWorkspaceChild::showShaded()
     if ( !titlebar)
 	return;
 
+#ifndef QT_NO_TOOLTIP
     QToolTip::remove( titlebar->shadeB );
+#endif
     ((QWorkspace*)parentWidget())->activateWindow( windowWidget() );
     if ( shademode ) {
+#ifndef QT_NO_TOOLTIP
 	QToolTip::add( titlebar->shadeB, tr( "Roll up" ) );
+#endif
 	titlebar->shadeB->setIconSet( QPixmap( (const char **)shade_xpm ) );
 	shademode = FALSE;
 	resize( shadeRestore );
 	setMinimumSize( shadeRestoreMin );
     } else {
+#ifndef QT_NO_TOOLTIP
 	QToolTip::add( titlebar->shadeB, tr( "Roll down" ) );
+#endif
 	titlebar->shadeB->setIconSet( QPixmap( (const char **)unshade_xpm ) );
 	shadeRestore = size();
 	shadeRestoreMin = minimumSize();
@@ -2871,15 +2886,19 @@ void QWorkspaceChildTitleLabel::cutText()
     int maxw = contentsRect().width() - leftm - rightm;
 
     if ( fm.width( titletext+"m" ) > maxw ) {
+#ifndef QT_NO_TOOLTIP
 	QToolTip::remove( this );
 	QToolTip::add( this, titletext );
+#endif
 	int i = titletext.length();
 	while ( (fm.width(titletext.left( i ) + "...")  > maxw) && i>0 )
 	    i--;
 	cuttext = titletext.left( i ) + "...";
 	drawLabel();
     } else {
+#ifndef QT_NO_TOOLTIP
 	QToolTip::remove( this );
+#endif
 	cuttext = titletext;
     }
     drawLabel();
